@@ -43,8 +43,8 @@ static css_error named_generic_sibling_node(void *pw, void *node,
 		void **sibling);
 static css_error parent_node(void *pw, void *node, void **parent);
 static css_error sibling_node(void *pw, void *node, void **sibling);
-static css_error node_has_name(void *pw, void *node, 
-		const css_qname *qname, 
+static css_error node_has_name(void *pw, void *node,
+		const css_qname *qname,
 		bool *match);
 static css_error node_has_class(void *pw, void *node,
 		lwc_string *name,
@@ -95,7 +95,7 @@ static css_error node_is_target(void *pw, void *node, bool *match);
 static css_error node_is_lang(void *pw, void *node,
 		lwc_string *lang, bool *match);
 static css_error node_presentational_hint(void *pw, void *node,
-		uint32_t property, css_hint *hint);
+		uint32_t *nhints, css_hint **hints);
 static css_error ua_default_for_property(void *pw, uint32_t property,
 		css_hint *hint);
 static css_error compute_font_size(void *pw, const css_hint *parent,
@@ -161,6 +161,9 @@ int main(int argc, char **argv)
 	uint32_t count;
 	unsigned int hh;
 	css_stylesheet_params params;
+	css_media media = {
+		.type = CSS_MEDIA_SCREEN,
+	};
 
 	UNUSED(argc);
 	UNUSED(argv);
@@ -210,7 +213,7 @@ int main(int argc, char **argv)
 	if (code != CSS_OK)
 		die("css_select_ctx_create", code);
 	code = css_select_ctx_append_sheet(select_ctx, sheet, CSS_ORIGIN_AUTHOR,
-			CSS_MEDIA_ALL);
+			NULL);
 	if (code != CSS_OK)
 		die("css_select_ctx_append_sheet", code);
 	code = css_select_ctx_count_sheets(select_ctx, &count);
@@ -234,7 +237,7 @@ int main(int argc, char **argv)
 		lwc_intern_string(element, strlen(element), &element_name);
 
 		code = css_select_style(select_ctx, element_name,
-				CSS_MEDIA_SCREEN, NULL,
+				&media, NULL,
 				&select_handler, 0,
 				&style);
 		if (code != CSS_OK)
@@ -299,9 +302,9 @@ css_error node_name(void *pw, void *n, css_qname *qname)
 	lwc_string *node = n;
 
 	UNUSED(pw);
-	
+
 	qname->name = lwc_string_ref(node);
-	
+
 	return CSS_OK;
 }
 
@@ -625,13 +628,13 @@ css_error node_is_lang(void *pw, void *n,
 }
 
 css_error node_presentational_hint(void *pw, void *node,
-		uint32_t property, css_hint *hint)
+		uint32_t *nhints, css_hint **hints)
 {
 	UNUSED(pw);
 	UNUSED(node);
-	UNUSED(property);
-	UNUSED(hint);
-	return CSS_PROPERTY_NOT_SET;
+	*nhints = 0;
+	*hints = NULL;
+	return CSS_OK;
 }
 
 css_error ua_default_for_property(void *pw, uint32_t property, css_hint *hint)
@@ -691,17 +694,17 @@ css_error compute_font_size(void *pw, const css_hint *parent, css_hint *size)
 		size->data.length = sizes[size->status - 1];
 	} else if (size->status == CSS_FONT_SIZE_LARGER) {
 		/** \todo Step within table, if appropriate */
-		size->data.length.value = 
+		size->data.length.value =
 				FMUL(parent_size->value, FLTTOFIX(1.2));
 		size->data.length.unit = parent_size->unit;
 	} else if (size->status == CSS_FONT_SIZE_SMALLER) {
 		/** \todo Step within table, if appropriate */
-		size->data.length.value = 
+		size->data.length.value =
 				FMUL(parent_size->value, FLTTOFIX(1.2));
 		size->data.length.unit = parent_size->unit;
 	} else if (size->data.length.unit == CSS_UNIT_EM ||
 			size->data.length.unit == CSS_UNIT_EX) {
-		size->data.length.value = 
+		size->data.length.value =
 			FMUL(size->data.length.value, parent_size->value);
 
 		if (size->data.length.unit == CSS_UNIT_EX) {
