@@ -67,7 +67,8 @@ enum {
 	sMalformedAtRule = 22,
 	sInlineStyle = 23,
 	sISBody0 = 24,
-	sISBody = 25
+	sISBody = 25,
+	sMediaQuery = 26,
 };
 
 /**
@@ -99,16 +100,14 @@ struct css_parser
 	bool parseError;		/**< A parse error has occurred */
 	parserutils_stack *open_items;	/**< Stack of open brackets */
 
-	uint8_t match_char;		/**< Close bracket type for parseAny */
-
 	bool last_was_ws;		/**< Last token was whitespace */
 
 	css_parser_event_handler event;	/**< Client's event handler */
 	void *event_pw;			/**< Client data for event handler */
 };
 
-static css_error css__parser_create_internal(const char *charset, 
-		css_charset_source cs_source, parser_state initial, 
+static css_error css__parser_create_internal(const char *charset,
+		css_charset_source cs_source, parser_state initial,
 		css_parser **parser);
 
 static css_error transition(css_parser *parser, parser_state to,
@@ -146,8 +145,9 @@ static css_error parseMalformedAtRule(css_parser *parser);
 static css_error parseInlineStyle(css_parser *parser);
 static css_error parseISBody0(css_parser *parser);
 static css_error parseISBody(css_parser *parser);
+static css_error parseMediaQuery(css_parser *parser);
 
-static void unref_interned_strings_in_tokens(css_parser *parser);
+static void discard_tokens(css_parser *parser);
 
 /**
  * Dispatch table for parsing, indexed by major state number
@@ -178,7 +178,8 @@ static css_error (*parseFuncs[])(css_parser *parser) = {
 	parseMalformedAtRule,
 	parseInlineStyle,
 	parseISBody0,
-	parseISBody
+	parseISBody,
+	parseMediaQuery,
 };
 
 /**
@@ -219,6 +220,25 @@ css_error css__parser_create_for_inline_style(const char *charset,
 			initial, parser);
 }
 
+/**
+ * Create a CSS parser for a media query
+ *
+ * \param charset     Charset of data, if known, or NULL
+ * \param cs_source   Source of charset information, or CSS_CHARSET_DEFAULT
+ * \param parser      Pointer to location to receive parser instance
+ * \return CSS_OK on success,
+ *         CSS_BADPARM on bad parameters,
+ *         CSS_NOMEM on memory exhaustion
+ */
+css_error css__parser_create_for_media_query(const char *charset,
+		css_charset_source cs_source, css_parser **parser)
+{
+	parser_state initial = { sMediaQuery, 0 };
+
+	return css__parser_create_internal(charset, cs_source,
+			initial, parser);
+}
+
 /**
  * Destroy a CSS parser
  *
@@ -280,7 +300,7 @@ css_error css__parser_setopt(css_parser *parser, css_parser_opttype type,
  * \param len     Length of chunk
  * \return CSS_OK on success, appropriate error otherwise
  */
-css_error css__parser_parse_chunk(css_parser *parser, const uint8_t *data, 
+css_error css__parser_parse_chunk(css_parser *parser, const uint8_t *data,
 		size_t len)
 {
 	parserutils_error perror;
@@ -344,7 +364,7 @@ css_error css__parser_completed(css_parser *parser)
  * \param source  Pointer to location to receive charset source
  * \return Pointer to charset name (constant; do not free)
  */
-const char *css__parser_read_charset(css_parser *parser, 
+const char *css__parser_read_charset(css_parser *parser,
 		css_charset_source *source)
 {
 	const char *charset;
@@ -386,8 +406,8 @@ bool css__parser_quirks_permitted(css_parser *parser)
  *         CSS_BADPARM on bad parameters,
  *         CSS_NOMEM on memory exhaustion
  */
-css_error css__parser_create_internal(const char *charset, 
-		css_charset_source cs_source, parser_state initial, 
+css_error css__parser_create_internal(const char *charset,
+		css_charset_source cs_source, parser_state initial,
 		css_parser **parser)
 {
 	css_parser *p;
@@ -415,7 +435,7 @@ css_error css__parser_create_internal(const char *charset,
 		return error;
 	}
 
-	perror = parserutils_stack_create(sizeof(parser_state), 
+	perror = parserutils_stack_create(sizeof(parser_state),
 			STACK_CHUNK, &p->states);
 	if (perror != PARSERUTILS_OK) {
 		css__lexer_destroy(p->lexer);
@@ -424,7 +444,7 @@ css_error css__parser_create_internal(const char *charset,
 		return css_error_from_parserutils_error(perror);
 	}
 
-	perror = parserutils_vector_create(sizeof(css_token), 
+	perror = parserutils_vector_create(sizeof(css_token),
 			STACK_CHUNK, &p->tokens);
 	if (perror != PARSERUTILS_OK) {
 		parserutils_stack_destroy(p->states);
@@ -434,7 +454,7 @@ css_error css__parser_create_internal(const char *charset,
 		return css_error_from_parserutils_error(perror);
 	}
 
-	perror = parserutils_stack_create(sizeof(char), 
+	perror = parserutils_stack_create(sizeof(char),
 			STACK_CHUNK, &p->open_items);
 	if (perror != PARSERUTILS_OK) {
 		parserutils_vector_destroy(p->tokens);
@@ -459,7 +479,6 @@ css_error css__parser_create_internal(const char *charset,
 	p->quirks = false;
 	p->pushback = NULL;
 	p->parseError = false;
-	p->match_char = 0;
 	p->event = NULL;
 	p->last_was_ws = false;
 	p->event_pw = NULL;
@@ -481,7 +500,7 @@ css_error css__parser_create_internal(const char *charset,
  * \param subsequent  The state to return to
  * \return CSS_OK on success, appropriate error otherwise
  */
-css_error transition(css_parser *parser, parser_state to, 
+css_error transition(css_parser *parser, parser_state to,
 		parser_state subsequent)
 {
 	parser_state *state = parserutils_stack_get_current(parser->states);
@@ -556,7 +575,7 @@ css_error done(css_parser *parser)
 
 /**
  * Assert that the expected token is next in the input
- * 
+ *
  * \param parser  The parser instance
  * \param type    The expected token type
  * \return CSS_OK on success, appropriate error otherwise
@@ -638,7 +657,7 @@ css_error getToken(css_parser *parser, const css_token **token)
 	}
 
 	/* Append token to vector */
-	perror = parserutils_vector_append(parser->tokens, 
+	perror = parserutils_vector_append(parser->tokens,
 			(css_token *) (*token));
 	if (perror != PARSERUTILS_OK)
 		return css_error_from_parserutils_error(perror);
@@ -661,7 +680,7 @@ css_error pushBack(css_parser *parser, const css_token *token)
 
 	/* The pushback buffer depth is 1 token. Assert this. */
 	assert(parser->pushback == NULL);
-        
+
 	perror = parserutils_vector_remove_last(parser->tokens);
 	if (perror != PARSERUTILS_OK)
 		return css_error_from_parserutils_error(perror);
@@ -713,7 +732,7 @@ css_error parseStart(css_parser *parser)
 		printf("Begin stylesheet\n");
 #endif
 		if (parser->event != NULL) {
-			parser->event(CSS_PARSER_START_STYLESHEET, NULL, 
+			parser->event(CSS_PARSER_START_STYLESHEET, NULL,
 					parser->event_pw);
 		}
 
@@ -742,12 +761,11 @@ css_error parseStart(css_parser *parser)
 	printf("End stylesheet\n");
 #endif
 	if (parser->event != NULL) {
-		parser->event(CSS_PARSER_END_STYLESHEET, NULL, 
+		parser->event(CSS_PARSER_END_STYLESHEET, NULL,
 				parser->event_pw);
 	}
-        
-        unref_interned_strings_in_tokens(parser);
-	parserutils_vector_clear(parser->tokens);
+
+	discard_tokens(parser);
 
 	return done(parser);
 }
@@ -778,8 +796,7 @@ css_error parseStylesheet(css_parser *parser)
 				if (error != CSS_OK)
 					return error;
 
-                                unref_interned_strings_in_tokens(parser);
-				parserutils_vector_clear(parser->tokens);
+				discard_tokens(parser);
 
 				return done(parser);
 			case CSS_TOKEN_CDO:
@@ -843,33 +860,32 @@ css_error parseRuleset(css_parser *parser)
 	const css_token *token;
 	css_error error;
 
-	/* ruleset -> selector '{' ws ruleset-end 
+	/* ruleset -> selector '{' ws ruleset-end
 	 *         -> '{' ws ruleset-end
 	 */
 
 	switch (state->substate) {
 	case Initial:
-                unref_interned_strings_in_tokens(parser);
-		parserutils_vector_clear(parser->tokens);
+		discard_tokens(parser);
 
 		error = getToken(parser, &token);
 		if (error != CSS_OK)
 			return error;
 
-		/* The grammar's ambiguous here -- selectors may start with a 
-		 * brace. We're going to assume that that won't happen, 
+		/* The grammar's ambiguous here -- selectors may start with a
+		 * brace. We're going to assume that that won't happen,
 		 * however. */
-		if (token->type == CSS_TOKEN_CHAR && 
-				lwc_string_length(token->idata) == 1 && 
+		if (token->type == CSS_TOKEN_CHAR &&
+				lwc_string_length(token->idata) == 1 &&
 				lwc_string_data(token->idata)[0] == '{') {
 #if !defined(NDEBUG) && defined(DEBUG_EVENTS)
 			printf("Begin ruleset\n");
 #endif
 			if (parser->event != NULL) {
-				if (parser->event(CSS_PARSER_START_RULESET, 
-						NULL, parser->event_pw) == 
+				if (parser->event(CSS_PARSER_START_RULESET,
+						NULL, parser->event_pw) ==
 						CSS_INVALID) {
-					parser_state to = 
+					parser_state to =
 						{ sMalformedSelector, Initial };
 
 					return transitionNoRet(parser, to);
@@ -890,17 +906,6 @@ css_error parseRuleset(css_parser *parser)
 		}
 		break;
 	case Brace:
-#if !defined(NDEBUG) && defined(DEBUG_EVENTS)
-		printf("Begin ruleset\n");
-		parserutils_vector_dump(parser->tokens, __func__, tprinter);
-#endif
-		if (parser->parseError == false && parser->event != NULL) {
-			if (parser->event(CSS_PARSER_START_RULESET, 
-					parser->tokens, parser->event_pw) ==
-					CSS_INVALID)
-				parser->parseError = true;
-		}
-
 		if (parser->parseError == true) {
 			parser_state to = { sMalformedSelector, Initial };
 
@@ -911,22 +916,40 @@ css_error parseRuleset(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
-		if (token->type == CSS_TOKEN_EOF) {
+		if (token->type != CSS_TOKEN_CHAR ||
+				lwc_string_length(token->idata) != 1 ||
+				lwc_string_data(token->idata)[0] != '{') {
+			/* FOLLOW(selector) contains only '{', but we may
+			 * also have seen EOF, which is a parse error. */
 			error = pushBack(parser, token);
 			if (error != CSS_OK)
 				return error;
 
+			parser->parseError = true;
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
-				lwc_string_length(token->idata) != 1 ||
-				lwc_string_data(token->idata)[0] != '{') {
-			/* This should never happen, as FOLLOW(selector) 
-			 * contains only '{' */
-			assert(0 && "Expected {");
+		/* We don't want to emit the brace, so push it back */
+		error = pushBack(parser, token);
+		if (error != CSS_OK)
+			return error;
+
+#if !defined(NDEBUG) && defined(DEBUG_EVENTS)
+		printf("Begin ruleset\n");
+		parserutils_vector_dump(parser->tokens, __func__, tprinter);
+#endif
+		if (parser->parseError == false && parser->event != NULL) {
+			if (parser->event(CSS_PARSER_START_RULESET,
+					parser->tokens, parser->event_pw) ==
+					CSS_INVALID)
+				parser->parseError = true;
 		}
-		
+
+		/* Re-read the brace */
+		error = getToken(parser, &token);
+		if (error != CSS_OK)
+			return error;
+
 		state->substate = WS;
 		/* Fall through */
 	case WS:
@@ -948,7 +971,7 @@ css_error parseRulesetEnd(css_parser *parser)
 	const css_token *token;
 	css_error error;
 
-	/* ruleset-end -> declaration decl-list '}' ws 
+	/* ruleset-end -> declaration decl-list '}' ws
 	 *             -> decl-list '}' ws
 	 */
 
@@ -969,7 +992,7 @@ css_error parseRulesetEnd(css_parser *parser)
 		 * attempt to parse a declaration. This will catch any invalid
 		 * input at this point and read to the start of the next
 		 * declaration. FIRST(decl-list) = (';', '}') */
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				(lwc_string_data(token->idata)[0] != '}' &&
 				lwc_string_data(token->idata)[0] != ';')) {
@@ -1001,7 +1024,7 @@ css_error parseRulesetEnd(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				lwc_string_data(token->idata)[0] != '}') {
 			/* This should never happen, as FOLLOW(decl-list)
@@ -1041,8 +1064,7 @@ css_error parseAtRule(css_parser *parser)
 
 	switch (state->substate) {
 	case Initial:
-                unref_interned_strings_in_tokens(parser);
-		parserutils_vector_clear(parser->tokens);
+		discard_tokens(parser);
 
 		error = getToken(parser, &token);
 		if (error != CSS_OK)
@@ -1077,9 +1099,9 @@ css_error parseAtRule(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
-		/* Grammar ambiguity: any0 can be followed by '{',';',')',']'. 
+		/* Grammar ambiguity: any0 can be followed by '{',';',')',']'.
 		 * at-rule can only be followed by '{' and ';'. */
-		if (token->type == CSS_TOKEN_CHAR && 
+		if (token->type == CSS_TOKEN_CHAR &&
 				lwc_string_length(token->idata) == 1) {
 			if (lwc_string_data(token->idata)[0] == ')' ||
 					lwc_string_data(
@@ -1119,7 +1141,7 @@ css_error parseAtRuleEnd(css_parser *parser)
 		parserutils_vector_dump(parser->tokens, __func__, tprinter);
 #endif
 		if (parser->event != NULL) {
-			if (parser->event(CSS_PARSER_START_ATRULE, 
+			if (parser->event(CSS_PARSER_START_ATRULE,
 					parser->tokens, parser->event_pw) ==
 					CSS_INVALID) {
 				parser_state to = { sMalformedAtRule, Initial };
@@ -1140,12 +1162,12 @@ css_error parseAtRuleEnd(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1) {
 			/* Should never happen FOLLOW(at-rule) == '{', ';'*/
 			assert(0 && "Expected { or ;");
 		}
-		
+
 		if (lwc_string_data(token->idata)[0] == '{') {
 			parser_state to = { sBlock, Initial };
 			parser_state subsequent = { sAtRuleEnd, AfterBlock };
@@ -1205,15 +1227,14 @@ css_error parseBlock(css_parser *parser)
 					parser->event_pw);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				lwc_string_data(token->idata)[0] != '{') {
 			/* This should never happen, as FIRST(block) == '{' */
 			assert(0 && "Expected {");
 		}
 
-                unref_interned_strings_in_tokens(parser);
-		parserutils_vector_clear(parser->tokens);
+		discard_tokens(parser);
 
 		state->substate = WS;
 		/* Fall through */
@@ -1244,10 +1265,10 @@ css_error parseBlock(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				lwc_string_data(token->idata)[0] != '}') {
-			/* This should never happen, as 
+			/* This should never happen, as
 			 * FOLLOW(block-content) == '}' */
 			assert(0 && "Expected }");
 		}
@@ -1269,8 +1290,7 @@ css_error parseBlock(css_parser *parser)
 		parser->event(CSS_PARSER_END_BLOCK, NULL, parser->event_pw);
 	}
 
-        unref_interned_strings_in_tokens(parser);
-	parserutils_vector_clear(parser->tokens);
+	discard_tokens(parser);
 
 	return done(parser);
 }
@@ -1282,7 +1302,7 @@ css_error parseBlockContent(css_parser *parser)
 	const css_token *token;
 	css_error error;
 
-	/* block-content -> any block-content 
+	/* block-content -> any block-content
 	 *               -> block block-content
 	 *               -> ATKEYWORD ws block-content
 	 *               -> ';' ws block-content
@@ -1299,12 +1319,12 @@ css_error parseBlockContent(css_parser *parser)
 			if (token->type == CSS_TOKEN_ATKEYWORD) {
 				state->substate = WS;
 			} else if (token->type == CSS_TOKEN_CHAR) {
-				if (lwc_string_length(token->idata) == 1 && 
+				if (lwc_string_length(token->idata) == 1 &&
 						lwc_string_data(
 						token->idata)[0] == '{') {
 					/* Grammar ambiguity. Assume block */
 					parser_state to = { sBlock, Initial };
-					parser_state subsequent = 
+					parser_state subsequent =
 						{ sBlockContent, Initial };
 
 					error = pushBack(parser, token);
@@ -1318,16 +1338,13 @@ css_error parseBlockContent(css_parser *parser)
 					if (parser->event != NULL) {
 						parser->event(
 							CSS_PARSER_BLOCK_CONTENT,
-							parser->tokens, 
+							parser->tokens,
 							parser->event_pw);
 					}
 
-					unref_interned_strings_in_tokens(
-							parser);
-					parserutils_vector_clear(
-							parser->tokens);
+					discard_tokens(parser);
 
-					return transition(parser, to, 
+					return transition(parser, to,
 							subsequent);
 				} else if (lwc_string_length(
 						token->idata) == 1 &&
@@ -1345,7 +1362,7 @@ css_error parseBlockContent(css_parser *parser)
 					if (parser->event != NULL) {
 						parser->event(
 							CSS_PARSER_BLOCK_CONTENT,
-							parser->tokens, 
+							parser->tokens,
 							parser->event_pw);
 					}
 
@@ -1353,10 +1370,7 @@ css_error parseBlockContent(css_parser *parser)
 					if (error != CSS_OK)
 						return error;
 
-					unref_interned_strings_in_tokens(
-							parser);
-					parserutils_vector_clear(
-							parser->tokens);
+					discard_tokens(parser);
 
 					state->substate = WS;
 				} else if (lwc_string_length(
@@ -1379,10 +1393,7 @@ css_error parseBlockContent(css_parser *parser)
 							parser->event_pw);
 					}
 
-					unref_interned_strings_in_tokens(
-							parser);
-					parserutils_vector_clear(
-							parser->tokens);
+					discard_tokens(parser);
 
 					return done(parser);
 				}
@@ -1401,15 +1412,14 @@ css_error parseBlockContent(css_parser *parser)
 							parser->event_pw);
 				}
 
-				unref_interned_strings_in_tokens(parser);
-				parserutils_vector_clear(parser->tokens);
+				discard_tokens(parser);
 
 				return done(parser);
 			}
 
 			if (state->substate == Initial) {
 				parser_state to = { sAny, Initial };
-				parser_state subsequent = 
+				parser_state subsequent =
 						{ sBlockContent, Initial };
 
 				error = pushBack(parser, token);
@@ -1445,8 +1455,7 @@ css_error parseSelector(css_parser *parser)
 		parser_state to = { sAny1, Initial };
 		parser_state subsequent = { sSelector, AfterAny1 };
 
-                unref_interned_strings_in_tokens(parser);
-		parserutils_vector_clear(parser->tokens);
+		discard_tokens(parser);
 
 		return transition(parser, to, subsequent);
 	}
@@ -1472,8 +1481,7 @@ css_error parseDeclaration(css_parser *parser)
 		parser_state to = { sProperty, Initial };
 		parser_state subsequent = { sDeclaration, Colon };
 
-                unref_interned_strings_in_tokens(parser);
-		parserutils_vector_clear(parser->tokens);
+		discard_tokens(parser);
 
 		return transition(parser, to, subsequent);
 	}
@@ -1498,7 +1506,7 @@ css_error parseDeclaration(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				lwc_string_data(token->idata)[0] != ':') {
 			/* parse error -- expected : */
@@ -1572,9 +1580,9 @@ css_error parseDeclList(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
-				(lwc_string_data(token->idata)[0] != '}' && 
+				(lwc_string_data(token->idata)[0] != '}' &&
 				lwc_string_data(token->idata)[0] != ';')) {
 			/* Should never happen */
 			assert(0 && "Expected ; or  }");
@@ -1611,7 +1619,7 @@ css_error parseDeclListEnd(css_parser *parser)
 	const css_token *token;
 	css_error error;
 
-	/* decl-list-end -> declaration decl-list 
+	/* decl-list-end -> declaration decl-list
 	 *               -> decl-list
 	 */
 
@@ -1621,12 +1629,12 @@ css_error parseDeclListEnd(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
-		if (token->type != CSS_TOKEN_CHAR || 
-				lwc_string_length(token->idata) != 1 || 
+		if (token->type != CSS_TOKEN_CHAR ||
+				lwc_string_length(token->idata) != 1 ||
 				(lwc_string_data(token->idata)[0] != ';' &&
 				lwc_string_data(token->idata)[0] != '}')) {
 			parser_state to = { sDeclaration, Initial };
-			parser_state subsequent = 
+			parser_state subsequent =
 					{ sDeclListEnd, AfterDeclaration };
 
 			error = pushBack(parser, token);
@@ -1717,7 +1725,7 @@ css_error parseValue1(css_parser *parser)
 			return error;
 
 		/* Grammar ambiguity -- assume ';' or '}' mark end */
-		if (token->type == CSS_TOKEN_CHAR && 
+		if (token->type == CSS_TOKEN_CHAR &&
 				lwc_string_length(token->idata) == 1 &&
 				(lwc_string_data(token->idata)[0] == ';' ||
 				lwc_string_data(token->idata)[0] == '}')) {
@@ -1746,7 +1754,7 @@ css_error parseValue0(css_parser *parser)
 	css_error error;
 
 	/* value0 -> value value0
-	 *        -> 
+	 *        ->
 	 */
 
 	while (1) {
@@ -1768,7 +1776,7 @@ css_error parseValue0(css_parser *parser)
 				return done(parser);
 
 			/* Grammar ambiguity -- assume ';' or '}' mark end */
-			if (token->type == CSS_TOKEN_CHAR && 
+			if (token->type == CSS_TOKEN_CHAR &&
 					lwc_string_length(
 						token->idata) == 1 &&
 					(lwc_string_data(
@@ -1813,8 +1821,8 @@ css_error parseValue(css_parser *parser)
 
 		if (token->type == CSS_TOKEN_ATKEYWORD) {
 			state->substate = WS;
-		} else if (token->type == CSS_TOKEN_CHAR && 
-				lwc_string_length(token->idata) == 1 && 
+		} else if (token->type == CSS_TOKEN_CHAR &&
+				lwc_string_length(token->idata) == 1 &&
 				lwc_string_data(token->idata)[0] == '{') {
 			/* Grammar ambiguity. Assume block. */
 			parser_state to = { sBlock, Initial };
@@ -1854,7 +1862,7 @@ css_error parseAny0(css_parser *parser)
 	css_error error;
 
 	/* any0 -> any any0
-	 *      -> 
+	 *      ->
 	 */
 
 	while (1) {
@@ -1875,9 +1883,9 @@ css_error parseAny0(css_parser *parser)
 			if (token->type == CSS_TOKEN_EOF)
 				return done(parser);
 
-			/* Grammar ambiguity: 
+			/* Grammar ambiguity:
 			 * assume '{', ';', ')', ']' mark end */
-			if (token->type == CSS_TOKEN_CHAR && 
+			if (token->type == CSS_TOKEN_CHAR &&
 					lwc_string_length(
 						token->idata) == 1 &&
 					(lwc_string_data(
@@ -1945,9 +1953,12 @@ css_error parseAny1(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
-		/* Grammar ambiguity: any0 can be followed by 
+		if (token->type == CSS_TOKEN_EOF)
+			return done(parser);
+
+		/* Grammar ambiguity: any0 can be followed by
 		 * '{', ';', ')', ']'. any1 can only be followed by '{'. */
-		if (token->type == CSS_TOKEN_CHAR && 
+		if (token->type == CSS_TOKEN_CHAR &&
 				lwc_string_length(token->idata) == 1) {
 			if (lwc_string_data(token->idata)[0] == ';' ||
 					lwc_string_data(
@@ -2003,20 +2014,20 @@ css_error parseAny(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
-		if (token->type != CSS_TOKEN_IDENT && 
-				token->type != CSS_TOKEN_NUMBER && 
-				token->type != CSS_TOKEN_PERCENTAGE && 
-				token->type != CSS_TOKEN_DIMENSION && 
-				token->type != CSS_TOKEN_STRING && 
-				token->type != CSS_TOKEN_CHAR && 
-				token->type != CSS_TOKEN_URI && 
-				token->type != CSS_TOKEN_HASH && 
-				token->type != CSS_TOKEN_UNICODE_RANGE && 
-				token->type != CSS_TOKEN_INCLUDES && 
-				token->type != CSS_TOKEN_DASHMATCH && 
-				token->type != CSS_TOKEN_PREFIXMATCH && 
-				token->type != CSS_TOKEN_SUFFIXMATCH && 
-				token->type != CSS_TOKEN_SUBSTRINGMATCH && 
+		if (token->type != CSS_TOKEN_IDENT &&
+				token->type != CSS_TOKEN_NUMBER &&
+				token->type != CSS_TOKEN_PERCENTAGE &&
+				token->type != CSS_TOKEN_DIMENSION &&
+				token->type != CSS_TOKEN_STRING &&
+				token->type != CSS_TOKEN_CHAR &&
+				token->type != CSS_TOKEN_URI &&
+				token->type != CSS_TOKEN_HASH &&
+				token->type != CSS_TOKEN_UNICODE_RANGE &&
+				token->type != CSS_TOKEN_INCLUDES &&
+				token->type != CSS_TOKEN_DASHMATCH &&
+				token->type != CSS_TOKEN_PREFIXMATCH &&
+				token->type != CSS_TOKEN_SUFFIXMATCH &&
+				token->type != CSS_TOKEN_SUBSTRINGMATCH &&
 				token->type != CSS_TOKEN_FUNCTION) {
 			/* parse error */
 			parser->parseError = true;
@@ -2025,18 +2036,19 @@ css_error parseAny(css_parser *parser)
 		}
 
 		if (token->type == CSS_TOKEN_FUNCTION) {
-			parser->match_char = ')';
+			parserutils_stack_push(parser->open_items, &")"[0]);
 			state->substate = WS;
-		} else if (token->type == CSS_TOKEN_CHAR && 
-				lwc_string_length(token->idata) == 1 && 
-				(lwc_string_data(token->idata)[0] == '(' || 
+		} else if (token->type == CSS_TOKEN_CHAR &&
+				lwc_string_length(token->idata) == 1 &&
+				(lwc_string_data(token->idata)[0] == '(' ||
 				lwc_string_data(token->idata)[0] == '[')) {
-			parser->match_char = lwc_string_data(
-					token->idata)[0] == '(' ? ')' : ']';
+			parserutils_stack_push(parser->open_items,
+					&(lwc_string_data(
+					token->idata)[0] == '(' ? ")" : "]")[0]);
 			state->substate = WS;
-		} 
-
-		state->substate = WS2;
+		} else {
+			state->substate = WS2;
+		}
 		/* Fall through */
 	case WS:
 	case WS2:
@@ -2063,11 +2075,24 @@ css_error parseAny(css_parser *parser)
 		if (error != CSS_OK)
 			return error;
 
+		if (token->type == CSS_TOKEN_EOF) {
+			error = pushBack(parser, token);
+			if (error != CSS_OK)
+				return error;
+
+			/* parse error */
+			parser->parseError = true;
+
+			return done(parser);
+		}
+
 		/* Match correct close bracket (grammar ambiguity) */
-		if (token->type == CSS_TOKEN_CHAR && 
+		if (token->type == CSS_TOKEN_CHAR &&
 				lwc_string_length(token->idata) == 1 &&
-				lwc_string_data(token->idata)[0] == 
-				parser->match_char) {
+				lwc_string_data(token->idata)[0] ==
+				((uint8_t *) parserutils_stack_get_current(
+						parser->open_items))[0]) {
+			parserutils_stack_pop(parser->open_items, NULL);
 			state->substate = WS2;
 			goto ws2;
 		}
@@ -2086,20 +2111,20 @@ css_error parseMalformedDeclaration(css_parser *parser)
 	const css_token *token = NULL;
 	css_error error;
 
-	/* Malformed declaration: read everything up to the next ; or } 
+	/* Malformed declaration: read everything up to the next ; or }
 	 * We must ensure that pairs of {}, (), [], are balanced */
 
 	switch (state->substate) {
 	case Initial:
 	{
 		/* Clear stack of open items */
-		while (parserutils_stack_pop(parser->open_items, NULL) == 
+		while (parserutils_stack_pop(parser->open_items, NULL) ==
 				PARSERUTILS_OK)
 			;
 
 		state->substate = Go;
-		/* Fall through */
 	}
+	/* Fall through */
 	case Go:
 		while (1) {
 			char want;
@@ -2156,13 +2181,13 @@ css_error parseMalformedDeclaration(css_parser *parser)
 				break;
 			}
 
-			/* Either pop off the stack, if we've matched the 
+			/* Either pop off the stack, if we've matched the
 			 * current item, or push the start token on */
 			if (match != NULL && *match == want) {
 				parserutils_stack_pop(
 					parser->open_items, NULL);
 			} else if (want == 0) {
-				parserutils_stack_push(parser->open_items, 
+				parserutils_stack_push(parser->open_items,
 						&data[0]);
 			}
 		}
@@ -2174,8 +2199,7 @@ css_error parseMalformedDeclaration(css_parser *parser)
 		return error;
 
 	/* Discard the tokens we've read */
-        unref_interned_strings_in_tokens(parser);
-	parserutils_vector_clear(parser->tokens);
+	discard_tokens(parser);
 
 	return done(parser);
 }
@@ -2193,7 +2217,7 @@ css_error parseMalformedSelector(css_parser *parser)
 	switch (state->substate) {
 	case Initial:
 		/* Clear the stack of open items */
-		while (parserutils_stack_pop(parser->open_items, NULL) == 
+		while (parserutils_stack_pop(parser->open_items, NULL) ==
 				PARSERUTILS_OK)
 			;
 
@@ -2243,21 +2267,21 @@ css_error parseMalformedSelector(css_parser *parser)
 				break;
 			}
 
-			/* Either pop off the stack, if we've matched the 
+			/* Either pop off the stack, if we've matched the
 			 * current item, or push the start token on */
 			if (match != NULL && *match == want) {
 				parserutils_stack_pop(
 					parser->open_items, NULL);
 			} else if (want == 0) {
-				parserutils_stack_push(parser->open_items, 
+				parserutils_stack_push(parser->open_items,
 						&data[0]);
 			}
 
 			/* If we encountered a '}', there was data on the stack
 			 * before, and the stack's now empty, then we've popped
-			 * a '{' off the stack. That means we've found the end 
+			 * a '{' off the stack. That means we've found the end
 			 * of the block, so we're done here. */
-			if (want == '{' && match != NULL && 
+			if (want == '{' && match != NULL &&
 					parserutils_stack_get_current(
 					parser->open_items) == NULL)
 				break;
@@ -2270,8 +2294,7 @@ css_error parseMalformedSelector(css_parser *parser)
 		return error;
 
 	/* Discard the tokens we've read */
-        unref_interned_strings_in_tokens(parser);
-	parserutils_vector_clear(parser->tokens);
+	discard_tokens(parser);
 
 	return done(parser);
 }
@@ -2291,13 +2314,13 @@ css_error parseMalformedAtRule(css_parser *parser)
 	case Initial:
 	{
 		/* Clear stack of open items */
-		while (parserutils_stack_pop(parser->open_items, NULL) == 
+		while (parserutils_stack_pop(parser->open_items, NULL) ==
 				PARSERUTILS_OK)
 			;
 
 		state->substate = Go;
-		/* Fall through */
 	}
+	/* Fall through */
 	case Go:
 		while (1) {
 			char want;
@@ -2352,21 +2375,21 @@ css_error parseMalformedAtRule(css_parser *parser)
 				break;
 			}
 
-			/* Either pop off the stack, if we've matched the 
+			/* Either pop off the stack, if we've matched the
 			 * current item, or push the start token on */
 			if (match != NULL && *match == want) {
 				parserutils_stack_pop(
 					parser->open_items, NULL);
 			} else if (want == 0) {
-				parserutils_stack_push(parser->open_items, 
+				parserutils_stack_push(parser->open_items,
 						&data[0]);
 			}
 
 			/* If we encountered a '}', there was data on the stack
 			 * before, and the stack's now empty, then we've popped
-			 * a '{' off the stack. That means we've found the end 
+			 * a '{' off the stack. That means we've found the end
 			 * of the block, so we're done here. */
-			if (want == '{' && match != NULL && 
+			if (want == '{' && match != NULL &&
 					parserutils_stack_get_current(
 					parser->open_items) == NULL)
 				break;
@@ -2379,8 +2402,7 @@ css_error parseMalformedAtRule(css_parser *parser)
 		return error;
 
 	/* Discard the tokens we've read */
-        unref_interned_strings_in_tokens(parser);
-	parserutils_vector_clear(parser->tokens);
+	discard_tokens(parser);
 
 	return done(parser);
 }
@@ -2395,7 +2417,7 @@ css_error parseInlineStyle(css_parser *parser)
 
 	switch (state->substate) {
 	case Initial:
-		/* Emit fake events such that the language parser knows 
+		/* Emit fake events such that the language parser knows
 		 * no different from a normal parse. */
 
 		if (parser->event != NULL) {
@@ -2425,12 +2447,12 @@ css_error parseInlineStyle(css_parser *parser)
 	}
 	case AfterISBody0:
 		/* Clean up any remaining tokens */
-		unref_interned_strings_in_tokens(parser);
+		discard_tokens(parser);
 
 		/* Emit remaining fake events to end the parse */
 		if (parser->event != NULL) {
 			/* 1) end ruleset */
-			parser->event(CSS_PARSER_END_RULESET, NULL, 
+			parser->event(CSS_PARSER_END_RULESET, NULL,
 					parser->event_pw);
 
 			/* 2) end stylesheet */
@@ -2497,7 +2519,7 @@ css_error parseISBody(css_parser *parser)
 	const css_token *token;
 	css_error error;
 
-	/* is-body      -> declaration decl-list '}' ws 
+	/* is-body      -> declaration decl-list '}' ws
 	 *              -> decl-list '}' ws
 	 *
 	 * Note that this looks suspiciously similar to ruleset-end.
@@ -2518,7 +2540,7 @@ css_error parseISBody(css_parser *parser)
 		 * attempt to parse a declaration. This will catch any invalid
 		 * input at this point and read to the start of the next
 		 * declaration. FIRST(decl-list) = (';', '}') */
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				(lwc_string_data(token->idata)[0] != '}' &&
 				lwc_string_data(token->idata)[0] != ';')) {
@@ -2550,7 +2572,7 @@ css_error parseISBody(css_parser *parser)
 			return done(parser);
 		}
 
-		if (token->type != CSS_TOKEN_CHAR || 
+		if (token->type != CSS_TOKEN_CHAR ||
 				lwc_string_length(token->idata) != 1 ||
 				lwc_string_data(token->idata)[0] != '}') {
 			/* This should never happen, as FOLLOW(decl-list)
@@ -2571,22 +2593,48 @@ css_error parseISBody(css_parser *parser)
 	return done(parser);
 }
 
+css_error parseMediaQuery(css_parser *parser)
+{
+	enum { Initial = 0, AfterAtRule = 1 };
+	parser_state *state = parserutils_stack_get_current(parser->states);
+
+	/* media-query = at-rule */
+
+	switch (state->substate) {
+	case Initial:
+	{
+		parser_state to = { sAtRule, Initial };
+		parser_state subsequent = { sMediaQuery, AfterAtRule };
+
+		return transition(parser, to, subsequent);
+	}
+	case AfterAtRule:
+		/* Clean up any remaining tokens */
+		discard_tokens(parser);
+		break;
+	}
+
+	return done(parser);
+}
+
 /**
- * Iterate the token vector and unref any interned strings in the tokens.
+ * Discard the contents of the token vector
  *
  * \param parser The parser whose tokens we are cleaning up.
  */
-void unref_interned_strings_in_tokens(css_parser *parser)
+void discard_tokens(css_parser *parser)
 {
-        int32_t ctx = 0;
-        const css_token *tok;
-        
-        while ((tok = parserutils_vector_iterate(
+	int32_t ctx = 0;
+	const css_token *tok;
+
+	while ((tok = parserutils_vector_iterate(
 			parser->tokens, &ctx)) != NULL) {
-                if (tok->idata != NULL) {
-                        lwc_string_unref(tok->idata);
+		if (tok->idata != NULL) {
+			lwc_string_unref(tok->idata);
 		}
-        }
+	}
+
+	parserutils_vector_clear(parser->tokens);
 }
 
 #ifndef NDEBUG
@@ -2604,7 +2652,11 @@ static void tprinter(void *token)
 {
 	css_token *t = token;
 
-	if (t->data.data)
+	if (t->idata) {
+		printf("%d: %.*s", t->type,
+				(int) lwc_string_length(t->idata),
+				lwc_string_data(t->idata));
+	} else if (t->data.data)
 		printf("%d: %.*s", t->type, (int) t->data.len, t->data.data);
 	else
 		printf("%d", t->type);
