@@ -17,7 +17,7 @@ typedef void (*handle_event)(struct dom_event *evt, void *pw);
 
 typedef struct dom_event_listener dom_event_listener;
 
-dom_exception dom_event_listener_create(struct dom_document *doc,
+dom_exception dom_event_listener_create(
 		handle_event handler, void *pw, dom_event_listener **listener);
 
 void dom_event_listener_ref(dom_event_listener *listener);
