@@ -124,25 +124,37 @@ dom_exception _dom_document_initialise(dom_document *doc,
 	if (err != DOM_NO_ERR)
 		return err;
 
+	err = dom_string_create_interned((const uint8_t *) "about:blank",
+			SLEN("about:blank"), &doc->uri);
+	if (err != DOM_NO_ERR) {
+		dom_string_unref(name);
+		return err;
+	}
+
 	doc->nodelists = NULL;
 
 	err = _dom_node_initialise(&doc->base, doc, DOM_DOCUMENT_NODE,
 			name, NULL, NULL, NULL);
 	dom_string_unref(name);
-        if (err != DOM_NO_ERR)
-          return err;
+	if (err != DOM_NO_ERR) {
+		dom_string_unref(doc->uri);
+		return err;
+	}
 
 	list_init(&doc->pending_nodes);
 
 	err = dom_string_create_interned((const uint8_t *) "id",
 					 SLEN("id"), &doc->id_name);
-	if (err != DOM_NO_ERR)
+	if (err != DOM_NO_ERR) {
+		dom_string_unref(doc->uri);
 		return err;
+	}
 	doc->quirks = DOM_DOCUMENT_QUIRKS_MODE_NONE;
 
 	err = dom_string_create_interned((const uint8_t *) "class",
 			SLEN("class"), &doc->class_string);
 	if (err != DOM_NO_ERR) {
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		return err;
 	}
@@ -150,6 +162,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 	err = dom_string_create_interned((const uint8_t *) "script",
 			SLEN("script"), &doc->script_string);
 	if (err != DOM_NO_ERR) {
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		return err;
@@ -161,6 +174,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 	err = dom_string_create_interned((const uint8_t *) " ", 0,
 					 &doc->_memo_empty);
 	if (err != DOM_NO_ERR) {
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -172,6 +186,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 					 &doc->_memo_domnodeinserted);
 	if (err != DOM_NO_ERR) {
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -184,6 +199,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 	if (err != DOM_NO_ERR) {
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -197,6 +213,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 		dom_string_unref(doc->_memo_domnoderemoved);
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -211,6 +228,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 		dom_string_unref(doc->_memo_domnoderemoved);
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -226,6 +244,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 		dom_string_unref(doc->_memo_domnoderemoved);
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -242,6 +261,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 		dom_string_unref(doc->_memo_domnoderemoved);
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -259,6 +279,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 		dom_string_unref(doc->_memo_domnoderemoved);
 		dom_string_unref(doc->_memo_domnodeinserted);
 		dom_string_unref(doc->_memo_empty);
+		dom_string_unref(doc->uri);
 		dom_string_unref(doc->id_name);
 		dom_string_unref(doc->class_string);
 		dom_string_unref(doc->script_string);
@@ -266,7 +287,7 @@ dom_exception _dom_document_initialise(dom_document *doc,
 	}
 
 	/* We should not pass a NULL when all things hook up */
-	return _dom_document_event_internal_initialise(doc, &doc->dei, daf, daf_ctx);
+	return _dom_document_event_internal_initialise(&doc->dei, daf, daf_ctx);
 }
 
 
@@ -298,6 +319,7 @@ bool _dom_document_finalise(dom_document *doc)
 	if (doc->id_name != NULL)
 		dom_string_unref(doc->id_name);
 
+	dom_string_unref(doc->uri);
 	dom_string_unref(doc->class_string);
 	dom_string_unref(doc->script_string);
 	dom_string_unref(doc->_memo_empty);
@@ -309,7 +331,7 @@ bool _dom_document_finalise(dom_document *doc)
 	dom_string_unref(doc->_memo_domcharacterdatamodified);
 	dom_string_unref(doc->_memo_domsubtreemodified);
 	
-	_dom_document_event_internal_finalise(doc, &doc->dei);
+	_dom_document_event_internal_finalise(&doc->dei);
 
 	return true;
 }
@@ -407,9 +429,9 @@ dom_exception _dom_document_get_document_element(dom_document *doc,
  * \param tag_name  The name of the element
  * \param result    Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::tag_name is invalid.
+ *         DOM_INVALID_CHARACTER_ERR if \p tag_name is invalid.
  *
- * ::doc and ::tag_name will have their reference counts increased.
+ * \p doc and \p tag_name will have their reference counts increased.
  *
  * The returned node will have its reference count increased. It is
  * the responsibility of the caller to unref the node once it has
@@ -548,7 +570,7 @@ dom_exception _dom_document_create_cdata_section(dom_document *doc,
  * \param data    The data for the node
  * \param result  Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::target is invalid,
+ *         DOM_INVALID_CHARACTER_ERR if \p target is invalid,
  *         DOM_NOT_SUPPORTED_ERR     if this is an HTML document.
  *
  * The returned node will have its reference count increased. It is
@@ -573,7 +595,7 @@ dom_exception _dom_document_create_processing_instruction(
  * \param name    The name of the attribute
  * \param result  Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::name is invalid.
+ *         DOM_INVALID_CHARACTER_ERR if \p name is invalid.
  *
  * The constructed attribute will always be classified as 'specified'.
  *
@@ -597,7 +619,7 @@ dom_exception _dom_document_create_attribute(dom_document *doc,
  * \param name    The name of the entity to reference
  * \param result  Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::name is invalid,
+ *         DOM_INVALID_CHARACTER_ERR if \p name is invalid,
  *         DOM_NOT_SUPPORTED_ERR     if this is an HTML document.
  *
  * The returned node will have its reference count increased. It is
@@ -643,7 +665,7 @@ dom_exception _dom_document_get_elements_by_tag_name(dom_document *doc,
  * \param result  Pointer to location to receive imported node in this document.
  * \return DOM_NO_ERR                on success,
  *         DOM_INVALID_CHARACTER_ERR if any of the names are invalid,
- *         DOM_NOT_SUPPORTED_ERR     if the type of ::node is unsupported
+ *         DOM_NOT_SUPPORTED_ERR     if the type of \p node is unsupported
  *
  * The returned node will have its reference count increased. It is
  * the responsibility of the caller to unref the node once it has
@@ -666,20 +688,20 @@ dom_exception _dom_document_import_node(dom_document *doc,
  * \param qname      The qualified name of the element
  * \param result     Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::qname is invalid,
- *         DOM_NAMESPACE_ERR         if ::qname is malformed, or it has a
- *                                   prefix and ::namespace is NULL, or
- *                                   ::qname has a prefix "xml" and
- *                                   ::namespace is not
+ *         DOM_INVALID_CHARACTER_ERR if \p qname is invalid,
+ *         DOM_NAMESPACE_ERR         if \p qname is malformed, or it has a
+ *                                   prefix and \p namespace is NULL, or
+ *                                   \p qname has a prefix "xml" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/XML/1998/namespace",
- *                                   or ::qname has a prefix "xmlns" and
- *                                   ::namespace is not
+ *                                   or \p qname has a prefix "xmlns" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/2000/xmlns", or
- *                                   ::namespace is
+ *                                   \p namespace is
  *                                   "http://www.w3.org/2000/xmlns" and
- *                                   ::qname is not (or is not prefixed by)
+ *                                   \p qname is not (or is not prefixed by)
  *                                   "xmlns",
- *         DOM_NOT_SUPPORTED_ERR     if ::doc does not support the "XML"
+ *         DOM_NOT_SUPPORTED_ERR     if \p doc does not support the "XML"
  *                                   feature.
  *
  * The returned node will have its reference count increased. It is
@@ -731,20 +753,20 @@ dom_exception _dom_document_create_element_ns(dom_document *doc,
  * \param qname      The qualified name of the attribute
  * \param result     Pointer to location to receive result
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::qname is invalid,
- *         DOM_NAMESPACE_ERR         if ::qname is malformed, or it has a
- *                                   prefix and ::namespace is NULL, or
- *                                   ::qname has a prefix "xml" and
- *                                   ::namespace is not
+ *         DOM_INVALID_CHARACTER_ERR if \p qname is invalid,
+ *         DOM_NAMESPACE_ERR         if \p qname is malformed, or it has a
+ *                                   prefix and \p namespace is NULL, or
+ *                                   \p qname has a prefix "xml" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/XML/1998/namespace",
- *                                   or ::qname has a prefix "xmlns" and
- *                                   ::namespace is not
+ *                                   or \p qname has a prefix "xmlns" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/2000/xmlns", or
- *                                   ::namespace is
+ *                                   \p namespace is
  *                                   "http://www.w3.org/2000/xmlns" and
- *                                   ::qname is not (or is not prefixed by)
+ *                                   \p qname is not (or is not prefixed by)
  *                                   "xmlns",
- *         DOM_NOT_SUPPORTED_ERR     if ::doc does not support the "XML"
+ *         DOM_NOT_SUPPORTED_ERR     if \p doc does not support the "XML"
  *                                   feature.
  *
  * The returned node will have its reference count increased. It is
@@ -1043,8 +1065,8 @@ dom_exception _dom_document_set_uri(dom_document *doc,
  * \param node    The node to adopt
  * \param result  Pointer to location to receive adopted node
  * \return DOM_NO_ERR                      on success,
- *         DOM_NO_MODIFICATION_ALLOWED_ERR if ::node is readonly,
- *         DOM_NOT_SUPPORTED_ERR           if ::node is of type Document or
+ *         DOM_NO_MODIFICATION_ALLOWED_ERR if \p node is readonly,
+ *         DOM_NOT_SUPPORTED_ERR           if \p node is of type Document or
  *                                         DocumentType
  *
  * The returned node will have its reference count increased. It is
@@ -1150,22 +1172,22 @@ dom_exception _dom_document_normalize(dom_document *doc)
  * \param qname      The new qualified name for the node
  * \param result     Pointer to location to receive renamed node
  * \return DOM_NO_ERR                on success,
- *         DOM_INVALID_CHARACTER_ERR if ::tag_name is invalid,
- *         DOM_WRONG_DOCUMENT_ERR    if ::node was created in a different
+ *         DOM_INVALID_CHARACTER_ERR if \p tag_name is invalid,
+ *         DOM_WRONG_DOCUMENT_ERR    if \p node was created in a different
  *                                   document
- *         DOM_NAMESPACE_ERR         if ::qname is malformed, or it has a
- *                                   prefix and ::namespace is NULL, or
- *                                   ::qname has a prefix "xml" and
- *                                   ::namespace is not
+ *         DOM_NAMESPACE_ERR         if \p qname is malformed, or it has a
+ *                                   prefix and \p namespace is NULL, or
+ *                                   \p qname has a prefix "xml" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/XML/1998/namespace",
- *                                   or ::qname has a prefix "xmlns" and
- *                                   ::namespace is not
+ *                                   or \p qname has a prefix "xmlns" and
+ *                                   \p namespace is not
  *                                   "http://www.w3.org/2000/xmlns", or
- *                                   ::namespace is
+ *                                   \p namespace is
  *                                   "http://www.w3.org/2000/xmlns" and
- *                                   ::qname is not (or is not prefixed by)
+ *                                   \p qname is not (or is not prefixed by)
  *                                   "xmlns",
- *         DOM_NOT_SUPPORTED_ERR     if ::doc does not support the "XML"
+ *         DOM_NOT_SUPPORTED_ERR     if \p doc does not support the "XML"
  *                                   feature.
  *
  * The returned node will have its reference count increased. It is
@@ -1371,23 +1393,23 @@ dom_exception _dom_find_element_by_id(dom_node_internal *root,
 		}
 
 		if (node->first_child != NULL) {
-			/* Has children */
+			/* Move to child */
 			node = node->first_child;
-		} else if (node->next != NULL) {
-			/* No children, but has siblings */
-			node = node->next;
 		} else {
-			/* No children or siblings. 
-			 * Find first unvisited relation. */
-			dom_node_internal *parent = node->parent;
-
-			while (parent != root &&
-					node == parent->last_child) {
-				node = parent;
-				parent = parent->parent;
+			while (node != NULL) {
+				if (node->next != NULL) {
+					/* Move to next sibling */
+					node = node->next;
+					break;
+				} else if (node->parent != root) {
+					/* Move back up to ancestors to
+					 * get to next siblings */
+					node = node->parent;
+				} else {
+					/* No more nodes below root. */
+					node = NULL;
+				}
 			}
-
-			node = node->next;
 		}
 	}
 
