@@ -294,6 +294,36 @@ void _dom_node_finalise(dom_node_internal *node)
 }
 
 
+/* ---------------------------------------------------------------------*/
+
+/* The public non-virtual function of this interface Node */
+
+dom_exception _dom_node_contains(struct dom_node_internal *node,
+				struct dom_node_internal *other,
+				bool *contains)
+{
+	assert(node != NULL);
+	assert(other != NULL);
+	assert(contains != NULL);
+
+	if (node->owner != other->owner) {
+		*contains = false;
+		return DOM_NO_ERR;
+	}
+
+	while (other != NULL) {
+		if (other == node) {
+			*contains = true;
+			return DOM_NO_ERR;
+		}
+		other = other->parent;
+	}
+
+	*contains = false;
+	return DOM_NO_ERR;
+}
+
+
 /* ---------------------------------------------------------------------*/
 
 /* The public virtual function of this interface Node */
@@ -1472,6 +1502,8 @@ dom_exception _dom_node_set_text_content(dom_node_internal *node,
 		err = dom_node_remove_child(node, p, (void *) &r);
 		if (err != DOM_NO_ERR)
 			return err;
+		/* The returned node was reffed, so unref it */
+		dom_node_unref(r);
 	}
 
 	doc = node->owner;
@@ -1480,12 +1512,15 @@ dom_exception _dom_node_set_text_content(dom_node_internal *node,
 	err = dom_document_create_text_node(doc, content, &text);
 	if (err != DOM_NO_ERR)
 		return err;
-	
-	err = dom_node_append_child(node, text, (void *) &r);
-	if (err != DOM_NO_ERR)
-		return err;
 
-	return DOM_NO_ERR;
+	err = dom_node_append_child(node, text, (void *) &r);
+
+	/* The node is held alive as a child here, so unref it */
+	dom_node_unref(text);
+	/* And unref it a second time because append_child reffed it too */
+	dom_node_unref(r);
+
+	return err;
 }
 
 /**
@@ -1933,6 +1968,7 @@ bool _dom_node_permitted_child(const dom_node_internal *parent,
 	case DOM_COMMENT_NODE:
 	case DOM_DOCUMENT_TYPE_NODE:
 	case DOM_NOTATION_NODE:
+	case DOM_NODE_TYPE_COUNT:
 		valid = false;
 		break;
 
