@@ -21,15 +21,13 @@ struct dom_text_event {
 };
 
 /* Constructor */
-dom_exception _dom_text_event_create(struct dom_document *doc, 
-		struct dom_text_event **evt);
+dom_exception _dom_text_event_create(struct dom_text_event **evt);
 
 /* Destructor */
 void _dom_text_event_destroy(struct dom_text_event *evt);
 
 /* Initialise function */
-dom_exception _dom_text_event_initialise(struct dom_document *doc, 
-		struct dom_text_event *evt);
+dom_exception _dom_text_event_initialise(struct dom_text_event *evt);
 
 /* Finalise function */
 void _dom_text_event_finalise(struct dom_text_event *evt);
