@@ -1119,6 +1119,7 @@ dom_exception _dom_html_document_get_body(dom_html_document *doc,
 		}
 
 		if (len == 0) {
+			dom_nodelist_unref(nodes);
 			exc = dom_document_get_elements_by_tag_name(doc,
 					doc->elements[DOM_HTML_ELEMENT_TYPE_FRAMESET],
 					&nodes);
@@ -1143,7 +1144,6 @@ dom_exception _dom_html_document_get_body(dom_html_document *doc,
 		}
 
 		*body = (dom_html_element *)node;
-		dom_node_unref(node);
 	}
 
 	return exc;
@@ -1183,8 +1183,10 @@ dom_exception _dom_html_document_get_images(dom_html_document *doc,
 	if (err != DOM_NO_ERR)
 		return err;
 
-	return _dom_html_collection_create(doc, (dom_node_internal *) root, 
+	err = _dom_html_collection_create(doc, (dom_node_internal *) root, 
 			images_callback, doc, col);
+	dom_node_unref(root);
+	return err;
 }
 
 bool applet_callback(struct dom_node_internal * node, void *ctx)
@@ -1232,8 +1234,10 @@ dom_exception _dom_html_document_get_applets(dom_html_document *doc,
 	if (err != DOM_NO_ERR)
 		return err;
 
-	return _dom_html_collection_create(doc, (dom_node_internal *) root, 
+	err = _dom_html_collection_create(doc, (dom_node_internal *) root, 
 			applets_callback, doc, col);
+	dom_node_unref(root);
+	return err;
 }
 
 /**
@@ -1275,8 +1279,10 @@ dom_exception _dom_html_document_get_links(dom_html_document *doc,
 	if (err != DOM_NO_ERR)
 		return err;
 
-	return _dom_html_collection_create(doc, (dom_node_internal *) root,
+	err = _dom_html_collection_create(doc, (dom_node_internal *) root,
 			links_callback, doc, col);
+	dom_node_unref(root);
+	return err;
 }
 
 static bool __dom_html_document_node_is_form(dom_node_internal *node,
@@ -1351,8 +1357,10 @@ dom_exception _dom_html_document_get_anchors(dom_html_document *doc,
 	if (err != DOM_NO_ERR)
 		return err;
 
-	return _dom_html_collection_create(doc, (dom_node_internal *) root,
+	err = _dom_html_collection_create(doc, (dom_node_internal *) root,
 			anchors_callback, doc, col);
+	dom_node_unref(root);
+	return err;
 }
 
 dom_exception _dom_html_document_get_cookie(dom_html_document *doc,
