@@ -170,7 +170,7 @@ dom_exception dom_html_field_set_element_get_form(
 	}
 
 	if (form_tmp != NULL) {
-		*form = (dom_html_form_element *) form_tmp;
+		*form = (dom_html_form_element *) dom_node_ref(form_tmp);
 		return DOM_NO_ERR;
 	}
 
