@@ -29,12 +29,12 @@ static bool is_option(struct dom_node_internal *node, void *ctx);
 /**
  * Create a dom_html_select_element object
  *
- * \param doc  The document object
- * \param ele  The returned element object
+ * \param params  The html element creation parameters
+ * \param ele     The returned element object
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-dom_exception _dom_html_select_element_create(struct dom_html_document *doc,
-		dom_string *namespace, dom_string *prefix,
+dom_exception _dom_html_select_element_create(
+		struct dom_html_element_create_params *params,
 		struct dom_html_select_element **ele)
 {
 	struct dom_node_internal *node;
@@ -48,25 +48,23 @@ dom_exception _dom_html_select_element_create(struct dom_html_document *doc,
 	node->base.vtable = &_dom_html_element_vtable;
 	node->vtable = &_protect_vtable;
 
-	return _dom_html_select_element_initialise(doc, namespace, prefix, *ele);
+	return _dom_html_select_element_initialise(params, *ele);
 }
 
 /**
  * Initialise a dom_html_select_element object
  *
- * \param doc  The document object
- * \param ele  The dom_html_select_element object
+ * \param params  The html element creation parameters
+ * \param ele     The dom_html_select_element object
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-dom_exception _dom_html_select_element_initialise(struct dom_html_document *doc,
-		dom_string *namespace, dom_string *prefix,
+dom_exception _dom_html_select_element_initialise(
+		struct dom_html_element_create_params *params,
 		struct dom_html_select_element *ele)
 {
 	ele->form = NULL;
 
-	return _dom_html_element_initialise(doc, &ele->base,
-			doc->memoised[hds_SELECT],
-			namespace, prefix);
+	return _dom_html_element_initialise(params, &ele->base);
 }
 
 /**
@@ -115,10 +113,45 @@ void _dom_virtual_html_select_element_destroy(dom_node_internal *node)
 }
 
 /* The virtual copy function, see src/core/node.c for detail */
-dom_exception _dom_html_select_element_copy(dom_node_internal *old,
-		dom_node_internal **copy)
+dom_exception _dom_html_select_element_copy(
+		dom_node_internal *old, dom_node_internal **copy)
 {
-	return _dom_html_element_copy(old, copy);
+	dom_html_select_element *new_node;
+	dom_exception err;
+
+	new_node = malloc(sizeof(dom_html_select_element));
+	if (new_node == NULL)
+		return DOM_NO_MEM_ERR;
+
+	err = dom_html_select_element_copy_internal(old, new_node);
+	if (err != DOM_NO_ERR) {
+		free(new_node);
+		return err;
+	}
+
+	*copy = (dom_node_internal *) new_node;
+
+	return DOM_NO_ERR;
+}
+
+dom_exception _dom_html_select_element_copy_internal(
+		dom_html_select_element *old,
+		dom_html_select_element *new)
+{
+	dom_exception err;
+
+	err = dom_html_element_copy_internal(old, new);
+	if (err != DOM_NO_ERR) {
+		return err;
+	}
+
+	/* TODO: We don't seem to keep a ref to form element, so just
+	 *       copy the pointer for now. */
+	new->form = old->form;
+
+	new->selected = old->selected;
+
+	return DOM_NO_ERR;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -695,7 +728,8 @@ bool is_option(struct dom_node_internal *node, void *ctx)
 	dom_html_select_element *ele = ctx;
 	dom_html_document *doc = (dom_html_document *) dom_node_get_owner(ele);
 
-	if (dom_string_isequal(node->name, doc->memoised[hds_OPTION]))
+	if (dom_string_isequal(node->name,
+			doc->elements[DOM_HTML_ELEMENT_TYPE_OPTION]))
 		return true;
 
 	return false;
