@@ -151,17 +151,26 @@ rufl_code rufl_init(void)
 
 		xfont_lose_font(font);
 	}
-	LOG("%s font manager", rufl_old_font_manager ? "old" : "new");
 
 	/* test if the font manager supports background blending */
 	rufl_fm_error = xfont_cache_addr(&fm_version, 0, 0);
-	if (rufl_fm_error)
+	if (rufl_fm_error) {
+		LOG("xfont_cache_addr: 0x%x: %s",
+				rufl_fm_error->errnum,
+				rufl_fm_error->errmess);
 		return rufl_FONT_MANAGER_ERROR;
+	}
 	if (fm_version >= 335)
 		rufl_can_background_blend = true;
 
+	LOG("%s font manager (v %d.%d)%s",
+		rufl_old_font_manager ? "old" : "new",
+		fm_version / 100, fm_version % 100,
+		rufl_broken_font_enumerate_characters ? " (broken fec)" : "");
+
 	code = rufl_init_font_list();
 	if (code != rufl_OK) {
+		LOG("rufl_init_font_list: 0x%x", code);
 		rufl_quit();
 		xhourglass_off();
 		return code;
@@ -230,7 +239,7 @@ rufl_code rufl_init(void)
 
 	code = rufl_init_family_menu();
 	if (code != rufl_OK) {
-		LOG("rufl_init_substitution_table: 0x%x", code);
+		LOG("rufl_init_family_menu: 0x%x", code);
 		rufl_quit();
 		xhourglass_off();
 		return code;
@@ -274,8 +283,10 @@ rufl_code rufl_init_font_list(void)
 			break;
 
 		code = rufl_init_add_font(identifier, local_name);
-		if (code != rufl_OK)
+		if (code != rufl_OK) {
+			LOG("rufl_init_add_font: 0x%x", code);
 			return code;
+		}
 	}
 
 	return rufl_OK;
@@ -469,7 +480,9 @@ rufl_code rufl_init_scan_font(unsigned int font_index)
 		rufl_fm_error = xfont_enumerate_characters(font, u, 
 				(int *) &next, (int *) &internal);
 		if (rufl_fm_error) {
-			LOG("xfont_enumerate_characters: 0x%x: %s",
+			LOG("xfont_enumerate_characters(\"%s\", "
+			    "U+%x, ...): 0x%x: %s",
+					font_name, u,
 					rufl_fm_error->errnum, 
 					rufl_fm_error->errmess);
 			xfont_lose_font(font);
@@ -536,7 +549,8 @@ rufl_code rufl_init_scan_font(unsigned int font_index)
 
 	if (rufl_fm_error) {
 		free(charset);
-		LOG("xfont_scan_string: 0x%x: %s",
+		LOG("xfont_scan_string(\"%s\", U+%x, ...): 0x%x: %s",
+				font_name, u,
 				rufl_fm_error->errnum, rufl_fm_error->errmess);
 		return rufl_FONT_MANAGER_ERROR;
 	}
@@ -675,7 +689,8 @@ rufl_code rufl_init_scan_font_no_enumerate(unsigned int font_index)
 
 	if (rufl_fm_error) {
 		free(charset);
-		LOG("xfont_scan_string: 0x%x: %s",
+		LOG("xfont_scan_string(\"%s\", U+%x, ...): 0x%x: %s",
+				font_name, u,
 				rufl_fm_error->errnum, rufl_fm_error->errmess);
 		return rufl_FONT_MANAGER_ERROR;
 	}
@@ -771,12 +786,22 @@ rufl_code rufl_init_scan_font_old(unsigned int font_index)
 		code = rufl_init_scan_font_in_encoding(font_name, encoding,
 				charset, umap + (num_umaps - 1), &last_used);
 		if (code != rufl_OK) {
+			LOG("rufl_init_scan_font_in_encoding(\"%s\", \"%s\", "
+			    "...): 0x%x (0x%x: %s)",
+					font_name, encoding, code,
+					code == rufl_FONT_MANAGER_ERROR ?
+						rufl_fm_error->errnum : 0,
+					code == rufl_FONT_MANAGER_ERROR ?
+						rufl_fm_error->errmess : "");
+
 			/* Not finding the font isn't fatal */
 			if (code != rufl_FONT_MANAGER_ERROR ||
 				(rufl_fm_error->errnum != 
-						error_FONT_NOT_FOUND &&
+					error_FONT_NOT_FOUND &&
 				rufl_fm_error->errnum !=
-						error_FILE_NOT_FOUND)) {
+					error_FILE_NOT_FOUND &&
+				rufl_fm_error->errnum !=
+					error_FONT_ENCODING_NOT_FOUND)) {
 				free(charset);
 				for (i = 0; i < num_umaps; i++)
 					free((umap + i)->encoding);
@@ -825,12 +850,22 @@ rufl_code rufl_init_scan_font_old(unsigned int font_index)
 		code = rufl_init_scan_font_in_encoding(font_name, NULL,
 				charset, umap, &last_used);
 		if (code != rufl_OK) {
+			LOG("rufl_init_scan_font_in_encoding(\"%s\", NULL, "
+			    "...): 0x%x (0x%x: %s)",
+					font_name, code,
+					code == rufl_FONT_MANAGER_ERROR ?
+						rufl_fm_error->errnum : 0,
+					code == rufl_FONT_MANAGER_ERROR ?
+						rufl_fm_error->errmess : "");
+
 			/* Not finding the font isn't fatal */
 			if (code != rufl_FONT_MANAGER_ERROR ||
 				(rufl_fm_error->errnum != 
-						error_FONT_NOT_FOUND &&
+					error_FONT_NOT_FOUND &&
 				rufl_fm_error->errnum !=
-						error_FILE_NOT_FOUND)) {
+					error_FILE_NOT_FOUND &&
+				rufl_fm_error->errnum !=
+					error_FONT_ENCODING_NOT_FOUND)) {
 				free(charset);
 				for (i = 0; i < num_umaps; i++)
 					free((umap + i)->encoding);
@@ -842,6 +877,13 @@ rufl_code rufl_init_scan_font_old(unsigned int font_index)
 		}
 	}
 
+	if (num_umaps == 0) {
+		/* No mappings found: font is empty or couldn't be found */
+		free(umap);
+		free(charset);
+		return rufl_OK;
+	}
+
 	/* shrink-wrap */
 	charset->size = offsetof(struct rufl_character_set, block) +
 			32 * last_used;
@@ -895,6 +937,8 @@ rufl_code rufl_init_scan_font_in_encoding(const char *font_name,
 
 	code = rufl_init_read_encoding(font, umap);
 	if (code != rufl_OK) {
+		LOG("rufl_init_read_encoding(\"%s\", ...): 0x%x",
+				buf, code);
 		xfont_lose_font(font);
 		return code;
 	}
@@ -942,7 +986,8 @@ rufl_code rufl_init_scan_font_in_encoding(const char *font_name,
 	xfont_lose_font(font);
 
 	if (rufl_fm_error) {
-		LOG("xfont_scan_string: 0x%x: %s",
+		LOG("xfont_scan_string(\"%s\", U+%x, ...) (c=%x): 0x%x: %s",
+				buf, umap->map[i].u, umap->map[i].c,
 				rufl_fm_error->errnum, rufl_fm_error->errmess);
 		return rufl_FONT_MANAGER_ERROR;
 	}
@@ -966,10 +1011,16 @@ rufl_code rufl_init_scan_font_in_encoding(const char *font_name,
 rufl_code rufl_init_read_encoding(font_f font,
 		struct rufl_unicode_map *umap)
 {
+	enum {
+		STATE_START,
+		STATE_COMMENT,
+		STATE_COLLECT,
+	} state = STATE_START;
+	bool emit = false;
 	unsigned int u = 0;
 	unsigned int i = 0;
+	unsigned int n = 0;
 	int c;
-	int n;
 	char filename[200];
 	char s[200];
 	struct rufl_glyph_map_entry *entry;
@@ -990,18 +1041,50 @@ rufl_code rufl_init_read_encoding(font_f font,
 	if (!fp)
 		return rufl_IO_ERROR;
 
-	while (!feof(fp) && u != 256) {
+	while (!feof(fp) && i < 256 && u < 256) {
 		c = fgetc(fp);
-		if (c == '%') {
-			/* comment line */
-			fgets(s, sizeof s, fp);
-		} else if (c == '/') {
-			/* character definition */
-			if (i++ < 32)
-				continue;
-			n = fscanf(fp, "%100s", s);
-			if (n != 1)
-				break;
+
+		if (state == STATE_START) {
+			if (c == '/') {
+				n = 0;
+				state = STATE_COLLECT;
+			} else if (c <= 0x20) {
+				/* Consume C0 and space */
+			} else {
+				/* Comment, or garbage */
+				state = STATE_COMMENT;
+			}
+		} else if (state == STATE_COMMENT) {
+			/* Consume until the next C0 */
+			if (c < 0x20) {
+				state = STATE_START;
+			}
+		} else {
+			if ((c >= '0' && c <= '9') ||
+					(c >= 'a' && c <= 'z') ||
+					(c >= 'A' && c <= 'Z') ||
+					(c == '.') || (c == '_')) {
+				/* Printable: append */
+				s[n++] = c;
+				if (n >= sizeof(s)) {
+					/* Too long: garbage */
+					state = STATE_COMMENT;
+				}
+			} else if (c > 0x20) {
+				/* Garbage */
+				state = STATE_COMMENT;
+			} else {
+				/* C0 or space: done */
+				s[n] = '\0';
+				if (n != 0) {
+					emit = true;
+				}
+				state = STATE_START;
+			}
+		}
+
+		/* Ignore first 32 character codes (these are control chars) */
+		if (emit && i > 31 && i < 256 && u < 256) {
 			entry = bsearch(s, rufl_glyph_map,
 					rufl_glyph_map_size,
 					sizeof rufl_glyph_map[0],
@@ -1014,13 +1097,18 @@ rufl_code rufl_init_read_encoding(font_f font,
 				for (; strcmp(s, entry->glyph_name) == 0;
 						entry++) {
 					umap->map[u].u = entry->u;
-					umap->map[u].c = i - 1;
+					umap->map[u].c = i;
 					u++;
 					if (u == 256)
 						break;
 				}
 			}
 		}
+
+		if (emit) {
+			i++;
+			emit = false;
+		}
 	}
 
 	if (fclose(fp) == EOF)
