@@ -1,7 +1,7 @@
 /*
- * Copyright 2006 Daniel Silverstone <dsilvers@digital-scurf.org>
+ * Copyright 2006-2019 Daniel Silverstone <dsilvers@digital-scurf.org>
+ * Copyright 2010-2018 Vincent Sanders <vince@netsurf-browser.org>
  * Copyright 2007 James Bursa <bursa@users.sourceforge.net>
- * Copyright 2003 Phil Mellor <monkeyson@users.sourceforge.net>
  *
  * This file is part of NetSurf.
  *
@@ -67,6 +67,21 @@
 /** maximum number of X509 certificates in chain for TLS connection */
 #define MAX_CERTS 10
 
+/* the ciphersuites we are willing to use */
+#define CIPHER_LIST						\
+	/* disable everything */				\
+	"-ALL:"							\
+	/* enable TLSv1.2 PFS suites */				\
+	"EECDH+AES+TLSv1.2:EDH+AES+TLSv1.2:"			\
+	/* enable PFS AES GCM suites */				\
+	"EECDH+AESGCM:EDH+AESGCM:"				\
+	/* Enable PFS AES CBC suites */				\
+	"EECDH+AES:EDH+AES:"					\
+	/* Enable non-PFS fallback suite */			\
+	"AES128-SHA:"						\
+	/* Remove any PFS suites using weak DSA key exchange */	\
+	"-DSS"
+
 /** SSL certificate info */
 struct cert_info {
 	X509 *cert;		/**< Pointer to certificate */
@@ -127,13 +142,39 @@ static char fetch_error_buffer[CURL_ERROR_SIZE];
 /** Proxy authentication details. */
 static char fetch_proxy_userpwd[100];
 
+/** Interlock to prevent initiation during callbacks */
+static bool inside_curl = false;
+
+
+/* OpenSSL 1.0.x to 1.1.0 certificate reference counting changed
+ * LibreSSL declares its OpenSSL version as 2.1 but only supports the old way
+ */
+#if (defined(LIBRESSL_VERSION_NUMBER) || (OPENSSL_VERSION_NUMBER < 0x1010000fL))
+static int ns_X509_up_ref(X509 *cert)
+{
+	cert->references++;
+	return 1;
+}
+
+static void ns_X509_free(X509 *cert)
+{
+	cert->references--;
+	if (cert->references == 0) {
+		X509_free(cert);
+	}
+}
+#else
+#define ns_X509_up_ref X509_up_ref
+#define ns_X509_free X509_free
+#endif
 
 /**
  * Initialise a cURL fetcher.
  */
 static bool fetch_curl_initialise(lwc_string *scheme)
 {
-	LOG("Initialise cURL fetcher for %s", lwc_string_data(scheme));
+	NSLOG(netsurf, INFO, "Initialise cURL fetcher for %s",
+	      lwc_string_data(scheme));
 	curl_fetchers_registered++;
 	return true; /* Always succeeds */
 }
@@ -149,17 +190,20 @@ static void fetch_curl_finalise(lwc_string *scheme)
 	struct cache_handle *h;
 
 	curl_fetchers_registered--;
-	LOG("Finalise cURL fetcher %s", lwc_string_data(scheme));
+	NSLOG(netsurf, INFO, "Finalise cURL fetcher %s",
+	      lwc_string_data(scheme));
 	if (curl_fetchers_registered == 0) {
 		CURLMcode codem;
 		/* All the fetchers have been finalised. */
-		LOG("All cURL fetchers finalised, closing down cURL");
+		NSLOG(netsurf, INFO,
+		      "All cURL fetchers finalised, closing down cURL");
 
 		curl_easy_cleanup(fetch_blank_curl);
 
 		codem = curl_multi_cleanup(fetch_curl_multi);
 		if (codem != CURLM_OK)
-			LOG("curl_multi_cleanup failed: ignoring");
+			NSLOG(netsurf, INFO,
+			      "curl_multi_cleanup failed: ignoring");
 
 		curl_global_cleanup();
 	}
@@ -229,7 +273,9 @@ fetch_curl_post_convert(const struct fetch_multipart_data *control)
 						"application/octet-stream",
 					CURLFORM_END);
 				if (code != CURL_FORMADD_OK)
-					LOG("curl_formadd: %d (%s)", code, control->name);
+					NSLOG(netsurf, INFO,
+					      "curl_formadd: %d (%s)", code,
+					      control->name);
 			} else {
 				char *mimetype = guit->fetch->mimetype(control->value);
 				code = curl_formadd(&post, &last,
@@ -240,7 +286,11 @@ fetch_curl_post_convert(const struct fetch_multipart_data *control)
 					(mimetype != 0 ? mimetype : "text/plain"),
 					CURLFORM_END);
 				if (code != CURL_FORMADD_OK)
-					LOG("curl_formadd: %d (%s=%s)", code, control->name, control->value);
+					NSLOG(netsurf, INFO,
+					      "curl_formadd: %d (%s=%s)",
+					      code,
+					      control->name,
+					      control->value);
 				free(mimetype);
 			}
 			free(leafname);
@@ -251,7 +301,9 @@ fetch_curl_post_convert(const struct fetch_multipart_data *control)
 					CURLFORM_COPYCONTENTS, control->value,
 					CURLFORM_END);
 			if (code != CURL_FORMADD_OK)
-				LOG("curl_formadd: %d (%s=%s)", code, control->name, control->value);
+				NSLOG(netsurf, INFO,
+				      "curl_formadd: %d (%s=%s)", code,
+				      control->name, control->value);
 		}
 	}
 
@@ -299,7 +351,7 @@ fetch_curl_setup(struct fetch *parent_fetch,
 
 	fetch->fetch_handle = parent_fetch;
 
-	LOG("fetch %p, url '%s'", fetch, nsurl_access(url));
+	NSLOG(netsurf, INFO, "fetch %p, url '%s'", fetch, nsurl_access(url));
 
 	/* construct a new fetch structure */
 	fetch->curl_handle = NULL;
@@ -438,7 +490,7 @@ fetch_curl_verify_callback(int verify_ok, X509_STORE_CTX *x509_ctx)
 	 */
 	if (!fetch->cert_data[depth].cert) {
 		fetch->cert_data[depth].cert = X509_STORE_CTX_get_current_cert(x509_ctx);
-		fetch->cert_data[depth].cert->references++;
+		ns_X509_up_ref(fetch->cert_data[depth].cert);
 		fetch->cert_data[depth].err = X509_STORE_CTX_get_error(x509_ctx);
 	}
 
@@ -521,10 +573,16 @@ fetch_curl_sslctxfun(CURL *curl_handle, void *_sslctx, void *parm)
 		/* Ensure server rejects the connection if downgraded too far */
 		SSL_CTX_set_mode(sslctx, SSL_MODE_SEND_FALLBACK_SCSV);
 #endif
+		/* Disable TLS1.2 ciphersuites */
+		SSL_CTX_set_cipher_list(sslctx, CIPHER_LIST ":-TLSv1.2");
 	}
 
 	SSL_CTX_set_options(sslctx, options);
 
+#ifdef SSL_OP_NO_TICKET
+	SSL_CTX_clear_options(sslctx, SSL_OP_NO_TICKET);
+#endif
+
 	return CURLE_OK;
 }
 
@@ -612,8 +670,8 @@ static CURLcode fetch_curl_set_options(struct curl_fetch_info *f)
 		SETOPT(CURLOPT_PROXY, NULL);
 	}
 
-	/* Disable SSL session ID caching, as some servers can't cope. */
-	SETOPT(CURLOPT_SSL_SESSIONID_CACHE, 0);
+	/* Force-enable SSL session ID caching, as some distros are odd. */
+	SETOPT(CURLOPT_SSL_SESSIONID_CACHE, 1);
 
 	if (urldb_get_cert_permissions(f->url)) {
 		/* Disable certificate verification */
@@ -655,6 +713,9 @@ fetch_curl_initiate_fetch(struct curl_fetch_info *fetch, CURL *handle)
 	code = fetch_curl_set_options(fetch);
 	if (code != CURLE_OK) {
 		fetch->curl_handle = 0;
+		/* The handle maybe went bad, eat it */
+		NSLOG(netsurf, WARNING, "cURL handle maybe went bad, retry later");
+		curl_easy_cleanup(handle);
 		return false;
 	}
 
@@ -692,6 +753,10 @@ static CURL *fetch_curl_get_handle(lwc_string *host)
 static bool fetch_curl_start(void *vfetch)
 {
 	struct curl_fetch_info *fetch = (struct curl_fetch_info*)vfetch;
+	if (inside_curl) {
+		NSLOG(netsurf, DEBUG, "Deferring fetch because we're inside cURL");
+		return false;
+	}
 	return fetch_curl_initiate_fetch(fetch,
 			fetch_curl_get_handle(fetch->host));
 }
@@ -754,7 +819,7 @@ static void fetch_curl_abort(void *vf)
 {
 	struct curl_fetch_info *f = (struct curl_fetch_info *)vf;
 	assert(f);
-	LOG("fetch %p, url '%s'", f, nsurl_access(f->url));
+	NSLOG(netsurf, INFO, "fetch %p, url '%s'", f, nsurl_access(f->url));
 	if (f->curl_handle) {
 		f->abort = true;
 	} else {
@@ -774,7 +839,7 @@ static void fetch_curl_stop(struct curl_fetch_info *f)
 	CURLMcode codem;
 
 	assert(f);
-	LOG("fetch %p, url '%s'", f, nsurl_access(f->url));
+	NSLOG(netsurf, INFO, "fetch %p, url '%s'", f, nsurl_access(f->url));
 
 	if (f->curl_handle) {
 		/* remove from curl multi handle */
@@ -815,10 +880,7 @@ static void fetch_curl_free(void *vf)
 	}
 
 	for (i = 0; i < MAX_CERTS && f->cert_data[i].cert; i++) {
-		f->cert_data[i].cert->references--;
-		if (f->cert_data[i].cert->references == 0) {
-			X509_free(f->cert_data[i].cert);
-		}
+		ns_X509_free(f->cert_data[i].cert);
 	}
 
 	free(f);
@@ -845,7 +907,7 @@ static bool fetch_curl_process_headers(struct curl_fetch_info *f)
 		assert(code == CURLE_OK);
 	}
 	http_code = f->http_code;
-	LOG("HTTP status code %li", http_code);
+	NSLOG(netsurf, INFO, "HTTP status code %li", http_code);
 
 	if (http_code == 304 && !f->post_urlenc && !f->post_multipart) {
 		/* Not Modified && GET request */
@@ -856,7 +918,7 @@ static bool fetch_curl_process_headers(struct curl_fetch_info *f)
 
 	/* handle HTTP redirects (3xx response codes) */
 	if (300 <= http_code && http_code < 400 && f->location != 0) {
-		LOG("FETCH_REDIRECT, '%s'", f->location);
+		NSLOG(netsurf, INFO, "FETCH_REDIRECT, '%s'", f->location);
 		msg.type = FETCH_REDIRECT;
 		msg.data.redirect = f->location;
 		fetch_send_callback(&msg, f->fetch_handle);
@@ -986,10 +1048,7 @@ curl_start_cert_validate(struct curl_fetch_info *f,
 					      X509_get_pubkey(certs[depth].cert));
 
 		/* and clean up */
-		certs[depth].cert->references--;
-		if (certs[depth].cert->references == 0) {
-			X509_free(certs[depth].cert);
-		}
+		ns_X509_free(certs[depth].cert);
 	}
 
 	msg.type = FETCH_CERT_ERR;
@@ -1021,7 +1080,7 @@ static void fetch_curl_done(CURL *curl_handle, CURLcode result)
 	assert(code == CURLE_OK);
 
 	abort_fetch = f->abort;
-	LOG("done %s", nsurl_access(f->url));
+	NSLOG(netsurf, INFO, "done %s", nsurl_access(f->url));
 
 	if ((abort_fetch == false) &&
 	    (result == CURLE_OK ||
@@ -1066,7 +1125,7 @@ static void fetch_curl_done(CURL *curl_handle, CURLcode result)
 		memset(f->cert_data, 0, sizeof(f->cert_data));
 		cert = true;
 	} else {
-		LOG("Unknown cURL response code %d", result);
+		NSLOG(netsurf, INFO, "Unknown cURL response code %d", result);
 		error = true;
 	}
 
@@ -1130,7 +1189,8 @@ static void fetch_curl_poll(lwc_string *scheme_ignored)
 				&exc_fd_set, &max_fd);
 		assert(codem == CURLM_OK);
 
-		LOG("Curl file descriptor states (maxfd=%i):", max_fd);
+		NSLOG(netsurf, DEEPDEBUG,
+		      "Curl file descriptor states (maxfd=%i):", max_fd);
 		for (i = 0; i <= max_fd; i++) {
 			bool read = false;
 			bool write = false;
@@ -1146,10 +1206,10 @@ static void fetch_curl_poll(lwc_string *scheme_ignored)
 				error = true;
 			}
 			if (read || write || error) {
-				LOG("  fd %i: %s %s %s", i,
-						read  ? "read" : "    ",
-						write ? "write" : "     ",
-						error ? "error" : "     ");
+				NSLOG(netsurf, DEEPDEBUG, "  fd %i: %s %s %s", i,
+				      read ? "read" : "    ",
+				      write ? "write" : "     ",
+				      error ? "error" : "     ");
 			}
 		}
 	}
@@ -1158,7 +1218,8 @@ static void fetch_curl_poll(lwc_string *scheme_ignored)
 	do {
 		codem = curl_multi_perform(fetch_curl_multi, &running);
 		if (codem != CURLM_OK && codem != CURLM_CALL_MULTI_PERFORM) {
-			LOG("curl_multi_perform: %i %s", codem, curl_multi_strerror(codem));
+			NSLOG(netsurf, WARNING, "curl_multi_perform: %i %s",
+			      codem, curl_multi_strerror(codem));
 			guit->misc->warning("MiscError", curl_multi_strerror(codem));
 			return;
 		}
@@ -1231,16 +1292,28 @@ fetch_curl_progress(void *clientp,
 
 
 /**
- * Ignore everything given to it.
- *
- * Used to ignore cURL debug.
+ * Format curl debug for nslog
  */
-static int fetch_curl_ignore_debug(CURL *handle,
-			    curl_infotype type,
-			    char *data,
-			    size_t size,
-			    void *userptr)
+static int
+fetch_curl_debug(CURL *handle,
+		 curl_infotype type,
+		 char *data,
+		 size_t size,
+		 void *userptr)
 {
+	static const char s_infotype[CURLINFO_END][3] = {
+		"* ", "< ", "> ", "{ ", "} ", "{ ", "} "
+	};
+	switch(type) {
+	case CURLINFO_TEXT:
+	case CURLINFO_HEADER_OUT:
+	case CURLINFO_HEADER_IN:
+		NSLOG(fetch, DEBUG, "%s%.*s", s_infotype[type], (int)size - 1, data);
+		break;
+
+	default:
+		break;
+	}
 	return 0;
 }
 
@@ -1254,6 +1327,9 @@ static size_t fetch_curl_data(char *data, size_t size, size_t nmemb, void *_f)
 	CURLcode code;
 	fetch_msg msg;
 
+	assert(inside_curl == false);
+	inside_curl = true;
+
 	/* ensure we only have to get this information once */
 	if (!f->http_code) {
 		code = curl_easy_getinfo(f->curl_handle, CURLINFO_HTTP_CODE,
@@ -1267,11 +1343,13 @@ static size_t fetch_curl_data(char *data, size_t size, size_t nmemb, void *_f)
 	 */
 	if (f->http_code == 401) {
 		f->http_code = 0;
+		inside_curl = false;
 		return size * nmemb;
 	}
 
 	if (f->abort || (!f->had_headers && fetch_curl_process_headers(f))) {
 		f->stopped = true;
+		inside_curl = false;
 		return 0;
 	}
 
@@ -1281,6 +1359,7 @@ static size_t fetch_curl_data(char *data, size_t size, size_t nmemb, void *_f)
 	msg.data.header_or_data.len = size * nmemb;
 	fetch_send_callback(&msg, f->fetch_handle);
 
+	inside_curl = false;
 	if (f->abort) {
 		f->stopped = true;
 		return 0;
@@ -1320,7 +1399,7 @@ fetch_curl_header(char *data, size_t size, size_t nmemb, void *_f)
 		free(f->location);
 		f->location = malloc(size);
 		if (!f->location) {
-			LOG("malloc failed");
+			NSLOG(netsurf, INFO, "malloc failed");
 			return size;
 		}
 		SKIP_ST(9);
@@ -1411,17 +1490,17 @@ nserror fetch_curl_register(void)
 		.finalise = fetch_curl_finalise
 	};
 
-	LOG("curl_version %s", curl_version());
+	NSLOG(netsurf, INFO, "curl_version %s", curl_version());
 
 	code = curl_global_init(CURL_GLOBAL_ALL);
 	if (code != CURLE_OK) {
-		LOG("curl_global_init failed.");
+		NSLOG(netsurf, INFO, "curl_global_init failed.");
 		return NSERROR_INIT_FAILED;
 	}
 
 	fetch_curl_multi = curl_multi_init();
 	if (!fetch_curl_multi) {
-		LOG("curl_multi_init failed.");
+		NSLOG(netsurf, INFO, "curl_multi_init failed.");
 		return NSERROR_INIT_FAILED;
 	}
 
@@ -1449,7 +1528,7 @@ nserror fetch_curl_register(void)
 	 */
 	fetch_blank_curl = curl_easy_init();
 	if (!fetch_blank_curl) {
-		LOG("curl_easy_init failed");
+		NSLOG(netsurf, INFO, "curl_easy_init failed");
 		return NSERROR_INIT_FAILED;
 	}
 
@@ -1459,15 +1538,17 @@ nserror fetch_curl_register(void)
 	if (code != CURLE_OK)						\
 		goto curl_easy_setopt_failed;
 
-	if (verbose_log) {
-	    SETOPT(CURLOPT_VERBOSE, 1);
-	} else {
-	    SETOPT(CURLOPT_VERBOSE, 0);
-	}
 	SETOPT(CURLOPT_ERRORBUFFER, fetch_error_buffer);
+	SETOPT(CURLOPT_DEBUGFUNCTION, fetch_curl_debug);
 	if (nsoption_bool(suppress_curl_debug)) {
-		SETOPT(CURLOPT_DEBUGFUNCTION, fetch_curl_ignore_debug);
+		SETOPT(CURLOPT_VERBOSE, 0);
+	} else {
+		SETOPT(CURLOPT_VERBOSE, 1);
 	}
+
+	/* Currently we explode if curl uses HTTP2, so force 1.1. */
+	SETOPT(CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
+
 	SETOPT(CURLOPT_WRITEFUNCTION, fetch_curl_data);
 	SETOPT(CURLOPT_HEADERFUNCTION, fetch_curl_header);
 	SETOPT(CURLOPT_PROGRESSFUNCTION, fetch_curl_progress);
@@ -1478,14 +1559,16 @@ nserror fetch_curl_register(void)
 	SETOPT(CURLOPT_LOW_SPEED_TIME, 180L);
 	SETOPT(CURLOPT_NOSIGNAL, 1L);
 	SETOPT(CURLOPT_CONNECTTIMEOUT, nsoption_uint(curl_fetch_timeout));
+	SETOPT(CURLOPT_SSL_CIPHER_LIST, CIPHER_LIST);
 
 	if (nsoption_charp(ca_bundle) &&
 	    strcmp(nsoption_charp(ca_bundle), "")) {
-		LOG("ca_bundle: '%s'", nsoption_charp(ca_bundle));
+		NSLOG(netsurf, INFO, "ca_bundle: '%s'",
+		      nsoption_charp(ca_bundle));
 		SETOPT(CURLOPT_CAINFO, nsoption_charp(ca_bundle));
 	}
 	if (nsoption_charp(ca_path) && strcmp(nsoption_charp(ca_path), "")) {
-		LOG("ca_path: '%s'", nsoption_charp(ca_path));
+		NSLOG(netsurf, INFO, "ca_path: '%s'", nsoption_charp(ca_path));
 		SETOPT(CURLOPT_CAPATH, nsoption_charp(ca_path));
 	}
 
@@ -1497,7 +1580,8 @@ nserror fetch_curl_register(void)
 		curl_with_openssl = false;
 	}
 
-	LOG("cURL %slinked against openssl", curl_with_openssl ? "" : "not ");
+	NSLOG(netsurf, INFO, "cURL %slinked against openssl",
+	      curl_with_openssl ? "" : "not ");
 
 	/* cURL initialised okay, register the fetchers */
 
@@ -1516,19 +1600,21 @@ nserror fetch_curl_register(void)
 		}
 
 		if (fetcher_add(scheme, &fetcher_ops) != NSERROR_OK) {
-			LOG("Unable to register cURL fetcher for %s", data->protocols[i]);
+			NSLOG(netsurf, INFO,
+			      "Unable to register cURL fetcher for %s",
+			      data->protocols[i]);
 		}
 	}
 
 	return NSERROR_OK;
 
 curl_easy_setopt_failed:
-	LOG("curl_easy_setopt failed.");
+	NSLOG(netsurf, INFO, "curl_easy_setopt failed.");
 	return NSERROR_INIT_FAILED;
 
 #if LIBCURL_VERSION_NUM >= 0x071e00
 curl_multi_setopt_failed:
-	LOG("curl_multi_setopt failed.");
+	NSLOG(netsurf, INFO, "curl_multi_setopt failed.");
 	return NSERROR_INIT_FAILED;
 #endif
 }
