@@ -25,14 +25,15 @@
 #include <string.h>
 #include <stdlib.h>
 #include <libwapcaplet/libwapcaplet.h>
+#include <nsutils/base64.h>
 
+#include "netsurf/inttypes.h"
 #include "utils/url.h"
 #include "utils/nsurl.h"
 #include "utils/corestrings.h"
 #include "utils/log.h"
 #include "utils/utils.h"
 #include "utils/ring.h"
-#include "utils/base64.h"
 
 #include "content/fetch.h"
 #include "content/fetchers.h"
@@ -40,7 +41,7 @@
 
 struct fetch_data_context {
 	struct fetch *parent_fetch;
-	char *url;
+	nsurl *url;
 	char *mimetype;
 	char *data;
 	size_t datalen;
@@ -56,14 +57,16 @@ static struct fetch_data_context *ring = NULL;
 
 static bool fetch_data_initialise(lwc_string *scheme)
 {
-	LOG("fetch_data_initialise called for %s", lwc_string_data(scheme));
+	NSLOG(netsurf, INFO, "fetch_data_initialise called for %s",
+	      lwc_string_data(scheme));
 
 	return true;
 }
 
 static void fetch_data_finalise(lwc_string *scheme)
 {
-	LOG("fetch_data_finalise called for %s", lwc_string_data(scheme));
+	NSLOG(netsurf, INFO, "fetch_data_finalise called for %s",
+	      lwc_string_data(scheme));
 }
 
 static bool fetch_data_can_fetch(const nsurl *url)
@@ -82,14 +85,7 @@ static void *fetch_data_setup(struct fetch *parent_fetch, nsurl *url,
 		return NULL;
 		
 	ctx->parent_fetch = parent_fetch;
-
-	/* TODO: keep as nsurl to avoid copy */
-	ctx->url = malloc(nsurl_length(url) + 1);
-	if (ctx->url == NULL) {
-		free(ctx);
-		return NULL;
-	}
-	memcpy(ctx->url, nsurl_access(url), nsurl_length(url) + 1);
+	ctx->url = nsurl_ref(url);
 
 	RING_INSERT(ring, ctx);
 	
@@ -105,7 +101,7 @@ static void fetch_data_free(void *ctx)
 {
 	struct fetch_data_context *c = ctx;
 
-	free(c->url);
+	nsurl_unref(c->url);
 	free(c->data);
 	free(c->mimetype);
 	RING_REMOVE(ring, c);
@@ -135,8 +131,8 @@ static bool fetch_data_process(struct fetch_data_context *c)
 {
 	nserror res;
 	fetch_msg msg;
-	char *params;
-	char *comma;
+	const char *params;
+	const char *comma;
 	char *unescaped;
 	size_t unescaped_len;
 	
@@ -146,9 +142,9 @@ static bool fetch_data_process(struct fetch_data_context *c)
 	 * data must still be there.
 	 */
 	
-	LOG("url: %.140s", c->url);
+	NSLOG(netsurf, DEEPDEBUG, "url: %.140s", nsurl_access(c->url));
 	
-	if (strlen(c->url) < 6) {
+	if (nsurl_length(c->url) < 6) {
 		/* 6 is the minimum possible length (data:,) */
 		msg.type = FETCH_ERROR;
 		msg.data.error = "Malformed data: URL";
@@ -157,7 +153,7 @@ static bool fetch_data_process(struct fetch_data_context *c)
 	}
 	
 	/* skip the data: part */
-	params = c->url + SLEN("data:");
+	params = nsurl_access(c->url) + SLEN("data:");
 	
 	/* find the comma */
 	if ( (comma = strchr(params, ',')) == NULL) {
@@ -202,29 +198,23 @@ static bool fetch_data_process(struct fetch_data_context *c)
 	}
 	
 	if (c->base64) {
-		base64_decode_alloc(unescaped, unescaped_len, &c->data,	&c->datalen);
-		if (c->data == NULL) {
+		if ((nsu_base64_decode_alloc((uint8_t *)unescaped,
+					     unescaped_len,
+					     (uint8_t **)&c->data,
+					     &c->datalen) != NSUERROR_OK) ||
+		    (c->data == NULL)) {
 			msg.type = FETCH_ERROR;
 			msg.data.error = "Unable to Base64 decode data: URL";
 			fetch_data_send_callback(&msg, c);
 			free(unescaped);
 			return false;
 		}
+		free(unescaped);
 	} else {
-		c->data = malloc(unescaped_len);
-		if (c->data == NULL) {
-			msg.type = FETCH_ERROR;
-			msg.data.error =
-				"Unable to allocate memory for data: URL";
-			fetch_data_send_callback(&msg, c);
-			free(unescaped);
-			return false;
-		}
 		c->datalen = unescaped_len;
-		memcpy(c->data, unescaped, unescaped_len);
+		c->data = unescaped;
 	}
 	
-	free(unescaped);
 	
 	return true;
 }
@@ -255,8 +245,10 @@ static void fetch_data_poll(lwc_string *scheme)
 			char header[64];
 
 			fetch_set_http_code(c->parent_fetch, 200);
-			LOG("setting data: MIME type to %s, length to %" PRIsizet,
-			    c->mimetype, c->datalen);
+			NSLOG(netsurf, INFO,
+			      "setting data: MIME type to %s, length to %"PRIsizet,
+			      c->mimetype,
+			      c->datalen);
 			/* Any callback can result in the fetch being aborted.
 			 * Therefore, we _must_ check for this after _every_
 			 * call to fetch_data_send_callback().
@@ -292,7 +284,8 @@ static void fetch_data_poll(lwc_string *scheme)
 				fetch_data_send_callback(&msg, c);
 			}
 		} else {
-			LOG("Processing of %s failed!", c->url);
+			NSLOG(netsurf, INFO, "Processing of %.140s failed!",
+			      nsurl_access(c->url));
 
 			/* Ensure that we're unlocked here. If we aren't, 
 			 * then fetch_data_process() is broken.
