@@ -27,9 +27,9 @@
  *
  * \todo Implement mmap retrieval where supported.
  *
- * \todo Implement static retrival for metadata objects as their heap
+ * \todo Implement static retrieval for metadata objects as their heap
  *         lifetime is typically very short, though this may be obsoleted
- *         by a small object storage stratagy.
+ *         by a small object storage strategy.
  *
  */
 
@@ -43,11 +43,11 @@
 #include <stdlib.h>
 #include <nsutils/unistd.h>
 
+#include "netsurf/inttypes.h"
 #include "utils/filepath.h"
 #include "utils/file.h"
 #include "utils/nsurl.h"
 #include "utils/log.h"
-#include "utils/utils.h"
 #include "utils/messages.h"
 #include "desktop/gui_internal.h"
 #include "netsurf/misc.h"
@@ -63,7 +63,7 @@
 /** Backing store file format version */
 #define CONTROL_VERSION 130
 
-/** Number of milliseconds after a update before control data maintinance is performed  */
+/** Number of milliseconds after a update before control data maintenance is performed  */
 #define CONTROL_MAINT_TIME 10000
 
 /** Get address from ident */
@@ -100,7 +100,7 @@
 #define BLOCK_USE_MAP_SIZE (1 << (BLOCK_ENTRY_COUNT - 3))
 
 /**
- * The type used to store index values refering to store entries. Care
+ * The type used to store index values referring to store entries. Care
  * must be taken with this type as it is used to build address to
  * entry mapping so changing the size will have large impacts on
  * memory usage.
@@ -109,7 +109,7 @@ typedef uint16_t entry_index_t;
 
 /**
  * The type used as a binary identifier for each entry derived from
- * the url. A larger identifier will have fewer collisions but
+ * the URL. A larger identifier will have fewer collisions but
  * requires proportionately more storage.
  */
 typedef uint32_t entry_ident_t;
@@ -184,7 +184,7 @@ struct store_entry_element {
  * @note Order is important to avoid excessive structure packing overhead.
  */
 struct store_entry {
-	int64_t last_used; /**< unix time the entry was last used */
+	int64_t last_used; /**< UNIX time the entry was last used */
 	entry_ident_t ident; /**< entry identifier */
 	uint16_t use_count; /**< number of times this entry has been accessed */
 	uint8_t flags; /**< entry flags */
@@ -227,7 +227,7 @@ struct store_state {
 	unsigned int entry_bits; /**< log2 number of bits in entry index. */
 	unsigned int last_entry; /**< index of last usable entry. */
 
-	/** flag indicating if the entries have been made persistant
+	/** flag indicating if the entries have been made persistent
 	 * since they were last changed.
 	 */
 	bool entries_dirty;
@@ -235,9 +235,9 @@ struct store_state {
 	/**
 	 * URL identifier to entry index mapping.
 	 *
-	 * This is an open coded index on the entries url field and
-	 * provides a computationaly inexpensive way to go from the
-	 * url to an entry.
+	 * This is an open coded index on the entries URL field and
+	 * provides a computationally inexpensive way to go from the
+	 * URL to an entry.
 	 */
 	entry_index_t *addrmap;
 
@@ -246,12 +246,12 @@ struct store_state {
 	struct block_file blocks[ENTRY_ELEM_COUNT][BLOCK_FILE_COUNT];
 
 	/** flag indicating if the block file use maps have been made
-	 * persistant since they were last changed.
+	 * persistent since they were last changed.
 	 */
 	bool blocks_dirty;
 
 	/** flag indicating if a block file has been opened for update
-	 * since maintinance was previously done.
+	 * since maintenance was previously done.
 	 */
 	bool blocks_opened;
 
@@ -283,8 +283,8 @@ struct store_state *storestate;
  *
  * @param[in] state The store state to use.
  * @param[in, out] bse Pointer to the entry to be removed.
- * @return NSERROR_OK and \a bse updated on succes or NSERROR_NOT_FOUND
- *         if no entry coresponds to the url.
+ * @return NSERROR_OK and \a bse updated on success or NSERROR_NOT_FOUND
+ *         if no entry corresponds to the URL.
  */
 static nserror
 remove_store_entry(struct store_state *state, struct store_entry **bse)
@@ -460,7 +460,7 @@ store_fname(struct store_state *state,
  * @param state The store state to use.
  * @param bse The entry to invalidate.
  * @param elem_idx The element index to invalidate.
- * @return NSERROR_OK on sucess or error code on failure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 invalidate_element(struct store_state *state,
@@ -500,7 +500,7 @@ invalidate_element(struct store_state *state,
  *
  * @param state The store state to use.
  * @param bse The entry to invalidate.
- * @return NSERROR_OK on sucess or error code on failure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 invalidate_entry(struct store_state *state, struct store_entry *bse)
@@ -519,11 +519,12 @@ invalidate_entry(struct store_state *state, struct store_entry *bse)
 		 * This entry cannot be immediately removed as it has
 		 * associated allocation so wait for allocation release.
 		 */
-		LOG("invalidating entry with referenced allocation");
+		NSLOG(netsurf, INFO,
+		      "invalidating entry with referenced allocation");
 		return NSERROR_OK;
 	}
 
-	LOG("Removing entry for %p", bse);
+	NSLOG(netsurf, INFO, "Removing entry for %p", bse);
 
 	/* remove the entry from the index */
 	ret = remove_store_entry(state, &bse);
@@ -533,12 +534,12 @@ invalidate_entry(struct store_state *state, struct store_entry *bse)
 
 	ret = invalidate_element(state, bse, ENTRY_ELEM_META);
 	if (ret != NSERROR_OK) {
-		LOG("Error invalidating metadata element");
+		NSLOG(netsurf, INFO, "Error invalidating metadata element");
 	}
 
 	ret = invalidate_element(state, bse, ENTRY_ELEM_DATA);
 	if (ret != NSERROR_OK) {
-		LOG("Error invalidating data element");
+		NSLOG(netsurf, INFO, "Error invalidating data element");
 	}
 
 	return NSERROR_OK;
@@ -554,7 +555,7 @@ static int compar(const void *va, const void *vb)
 	const struct store_entry *b = &BS_ENTRY(*(entry_ident_t *)vb, storestate);
 
 	/* consider the allocation flags - if an entry has an
-	 * allocation it is considered more valuble as it cannot be
+	 * allocation it is considered more valuable as it cannot be
 	 * freed.
 	 */
 	if ((a->elem[ENTRY_ELEM_DATA].flags == ENTRY_ELEM_FLAG_NONE) &&
@@ -620,8 +621,10 @@ static nserror store_evict(struct store_state *state)
 		return NSERROR_OK;
 	}
 
-	LOG("Evicting entries to reduce %"PRIu64" by %zd",
-	    state->total_alloc, state->hysteresis);
+	NSLOG(netsurf, INFO,
+	      "Evicting entries to reduce %"PRIu64" by %"PRIsizet,
+	      state->total_alloc,
+	      state->hysteresis);
 
 	/* allocate storage for the list */
 	elist = malloc(sizeof(entry_ident_t) * state->last_entry);
@@ -658,7 +661,8 @@ static nserror store_evict(struct store_state *state)
 
 	free(elist);
 
-	LOG("removed %zd in %d entries", removed, ent);
+	NSLOG(netsurf, INFO, "removed %"PRIsizet" in %d entries", removed,
+	      ent);
 
 	return ret;
 }
@@ -670,7 +674,7 @@ static nserror store_evict(struct store_state *state)
  * Serialise entry index out to storage.
  *
  * @param state The backing store state to serialise.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror write_entries(struct store_state *state)
 {
@@ -735,7 +739,7 @@ static nserror write_entries(struct store_state *state)
  * Serialise block file use map out to storage.
  *
  * \param state The backing store state to serialise.
- * \return NSERROR_OK on sucess or error code on faliure.
+ * \return NSERROR_OK on success or error code on failure.
  */
 static nserror write_blocks(struct store_state *state)
 {
@@ -773,7 +777,10 @@ static nserror write_blocks(struct store_state *state)
 				   &state->blocks[elem_idx][bfidx].use_map[0],
 				   BLOCK_USE_MAP_SIZE);
 			if (wr != BLOCK_USE_MAP_SIZE) {
-				LOG("writing block file %d use index on file number %d failed", elem_idx, bfidx);
+				NSLOG(netsurf, INFO,
+				      "writing block file %d use index on file number %d failed",
+				      elem_idx,
+				      bfidx);
 				goto wr_err;
 			}
 			written += wr;
@@ -812,11 +819,11 @@ wr_err:
  * Ensures block files are of the correct extent
  *
  * block files have their extent set to their maximum size to ensure
- * subsequent reads and writes do not need to extend teh file and are
+ * subsequent reads and writes do not need to extend the file and are
  * therefore faster.
  *
  * \param state The backing store state to set block extent for.
- * \return NSERROR_OK on sucess or error code on faliure.
+ * \return NSERROR_OK on success or error code on failure.
  */
 static nserror set_block_extents(struct store_state *state)
 {
@@ -829,19 +836,21 @@ static nserror set_block_extents(struct store_state *state)
 		return NSERROR_OK;
 	}
 
-	LOG("Starting");
+	NSLOG(netsurf, INFO, "Starting");
 	for (elem_idx = 0; elem_idx < ENTRY_ELEM_COUNT; elem_idx++) {
 		for (bfidx = 0; bfidx < BLOCK_FILE_COUNT; bfidx++) {
 			if (state->blocks[elem_idx][bfidx].fd != -1) {
 				/* ensure block file is correct extent */
 				ftr = ftruncate(state->blocks[elem_idx][bfidx].fd, 1U << (log2_block_size[elem_idx] + BLOCK_ENTRY_COUNT));
 				if (ftr == -1) {
-					LOG("Truncate failed errno:%d", errno);
+					NSLOG(netsurf, INFO,
+					      "Truncate failed errno:%d",
+					      errno);
 				}
 			}
 		}
 	}
-	LOG("Complete");
+	NSLOG(netsurf, INFO, "Complete");
 
 	state->blocks_opened = false;
 
@@ -849,7 +858,7 @@ static nserror set_block_extents(struct store_state *state)
 }
 
 /**
- * maintinance of control structures.
+ * maintenance of control structures.
  *
  * callback scheduled when control data has been update. Currently
  * this is for when the entries table is dirty and requires
@@ -886,7 +895,7 @@ get_store_entry(struct store_state *state, nsurl *url, struct store_entry **bse)
 	entry_ident_t ident;
 	unsigned int sei; /* store entry index */
 
-	LOG("url:%s", nsurl_access(url));
+	NSLOG(netsurf, INFO, "url:%s", nsurl_access(url));
 
 	/* use the url hash as the entry identifier */
 	ident = nsurl_hash(url);
@@ -894,13 +903,14 @@ get_store_entry(struct store_state *state, nsurl *url, struct store_entry **bse)
 	sei = BS_ENTRY_INDEX(ident, state);
 
 	if (sei == 0) {
-		LOG("Failed to find ident 0x%x in index", ident);
+		NSLOG(netsurf, INFO, "Failed to find ident 0x%x in index",
+		      ident);
 		return NSERROR_NOT_FOUND;
 	}
 
 	if (state->entries[sei].ident != ident) {
 		/* entry ident did not match */
-		LOG("ident did not match entry");
+		NSLOG(netsurf, INFO, "ident did not match entry");
 		return NSERROR_NOT_FOUND;
 	}
 
@@ -959,7 +969,7 @@ static block_index_t alloc_block(struct store_state *state, int elem_idx)
  * @param datalen The length of data in \a data
  * @param bse Pointer used to return value.
  * @return NSERROR_OK and \a bse updated on success or NSERROR_NOT_FOUND
- *         if no entry coresponds to the url.
+ *         if no entry corresponds to the url.
  */
 static nserror
 set_store_entry(struct store_state *state,
@@ -975,7 +985,7 @@ set_store_entry(struct store_state *state,
 	nserror ret;
 	struct store_entry_element *elem;
 
-	LOG("url:%s", nsurl_access(url));
+	NSLOG(netsurf, INFO, "url:%s", nsurl_access(url));
 
 	/* evict entries as required and ensure there is at least one
 	 * new entry available.
@@ -1013,7 +1023,10 @@ set_store_entry(struct store_state *state,
 			 * to see if the old entry is in use and if
 			 * not prefer the newly stored entry instead?
 			 */
-			LOG("Entry index collision trying to replace %x with %x", se->ident, ident);
+			NSLOG(netsurf, INFO,
+			      "Entry index collision trying to replace %x with %x",
+			      se->ident,
+			      ident);
 			return NSERROR_PERMISSION;
 		}
 	}
@@ -1026,7 +1039,8 @@ set_store_entry(struct store_state *state,
 		/* this entry cannot be removed as it has associated
 		 * allocation.
 		 */
-		LOG("attempt to overwrite entry with in use data");
+		NSLOG(netsurf, INFO,
+		      "attempt to overwrite entry with in use data");
 		return NSERROR_PERMISSION;
 	}
 
@@ -1045,12 +1059,12 @@ set_store_entry(struct store_state *state,
 	elem->size = datalen;
 	state->total_alloc += elem->size;
 
-	/* if the elemnt will fit in a small block attempt to allocate one */
+	/* if the element will fit in a small block attempt to allocate one */
 	if (elem->size <= (1U << log2_block_size[elem_idx])) {
 		elem->block = alloc_block(state, elem_idx);
 	}
 
-	/* ensure control maintinance scheduled. */
+	/* ensure control maintenance scheduled. */
 	state->entries_dirty = true;
 	guit->misc->schedule(CONTROL_MAINT_TIME, control_maintinance, state);
 
@@ -1085,7 +1099,7 @@ store_open(struct store_state *state,
 
 	fname = store_fname(state, ident, elem_idx);
 	if (fname == NULL) {
-		LOG("filename error");
+		NSLOG(netsurf, INFO, "filename error");
 		return -1;
 	}
 
@@ -1093,13 +1107,14 @@ store_open(struct store_state *state,
 	if (openflags & O_CREAT) {
 		ret = netsurf_mkdir_all(fname);
 		if (ret != NSERROR_OK) {
-			LOG("file path \"%s\" could not be created", fname);
+			NSLOG(netsurf, INFO,
+			      "file path \"%s\" could not be created", fname);
 			free(fname);
 			return -1;
 		}
 	}
 
-	LOG("opening %s", fname);
+	NSLOG(netsurf, INFO, "opening %s", fname);
 	fd = open(fname, openflags, S_IRUSR | S_IWUSR);
 
 	free(fname);
@@ -1118,7 +1133,7 @@ store_open(struct store_state *state,
  * we also compute the total storage in use.
  *
  * @param state The backing store global state.
- * @return NSERROR_OK on sucess or NSERROR_NOMEM if the map storage
+ * @return NSERROR_OK on success or NSERROR_NOMEM if the map storage
  *         could not be allocated.
  */
 static nserror
@@ -1126,9 +1141,9 @@ build_entrymap(struct store_state *state)
 {
 	unsigned int eloop;
 
-	LOG("Allocating %ld bytes for max of %d buckets",
-	    (1 << state->ident_bits) * sizeof(entry_index_t),
-	    1 << state->ident_bits);
+	NSLOG(netsurf, INFO, "Allocating %"PRIsizet" bytes for max of %d buckets",
+	      (1 << state->ident_bits) * sizeof(entry_index_t),
+	      1 << state->ident_bits);
 
 	state->addrmap = calloc(1 << state->ident_bits, sizeof(entry_index_t));
 	if (state->addrmap == NULL) {
@@ -1138,12 +1153,12 @@ build_entrymap(struct store_state *state)
 	state->total_alloc = 0;
 
 	for (eloop = 1; eloop < state->last_entry; eloop++) {
-		/*
-		LOG("entry:%d ident:0x%08x used:%d",
-		     eloop,
-		     BS_ADDRESS(state->entries[eloop].ident, state),
-		     state->entries[eloop].use_count);
-		*/
+
+		NSLOG(llcache, DEEPDEBUG,
+		      "entry:%d ident:0x%08x used:%d",
+		      eloop,
+		      BS_ADDRESS(state->entries[eloop].ident, state),
+		      state->entries[eloop].use_count);
 
 		/* update the address map to point at the entry */
 		BS_ENTRY_INDEX(state->entries[eloop].ident, state) = eloop;
@@ -1163,7 +1178,7 @@ build_entrymap(struct store_state *state)
  * Unlink entries file
  *
  * @param state The backing store state.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 unlink_entries(struct store_state *state)
@@ -1186,7 +1201,7 @@ unlink_entries(struct store_state *state)
  * Read description entries into memory.
  *
  * @param state The backing store state to put the loaded entries in.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on faliure.
  */
 static nserror
 read_entries(struct store_state *state)
@@ -1204,10 +1219,12 @@ read_entries(struct store_state *state)
 
 	entries_size = (1 << state->entry_bits) * sizeof(struct store_entry);
 
-	LOG("Allocating %zd bytes for max of %d entries of %ld length elements %ld length",
-	    entries_size, 1 << state->entry_bits,
-	    sizeof(struct store_entry),
-	    sizeof(struct store_entry_element));
+	NSLOG(netsurf, INFO,
+	      "Allocating %"PRIsizet" bytes for max of %d entries of %"PRIsizet" length elements %"PRIsizet" length",
+	      entries_size,
+	      1 << state->entry_bits,
+	      sizeof(struct store_entry),
+	      sizeof(struct store_entry_element));
 
 	state->entries = calloc(1, entries_size);
 	if (state->entries == NULL) {
@@ -1222,7 +1239,8 @@ read_entries(struct store_state *state)
 		close(fd);
 		if (rd > 0) {
 			state->last_entry = rd / sizeof(struct store_entry);
-			LOG("Read %d entries", state->last_entry);
+			NSLOG(netsurf, INFO, "Read %d entries",
+			      state->last_entry);
 		}
 	} else {
 		/* could rebuild entries from fs */
@@ -1236,7 +1254,7 @@ read_entries(struct store_state *state)
  * Read block file usage bitmaps.
  *
  * @param state The backing store state to put the loaded entries in.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 read_blocks(struct store_state *state)
@@ -1253,7 +1271,7 @@ read_blocks(struct store_state *state)
 		return ret;
 	}
 
-	LOG("Initialising block use map from %s", fname);
+	NSLOG(netsurf, INFO, "Initialising block use map from %s", fname);
 
 	fd = open(fname, O_RDWR);
 	free(fname);
@@ -1265,7 +1283,10 @@ read_blocks(struct store_state *state)
 					  &state->blocks[elem_idx][bfidx].use_map[0],
 					  BLOCK_USE_MAP_SIZE);
 				if (rd <= 0) {
-					LOG("reading block file %d use index on file number %d failed", elem_idx, bfidx);
+					NSLOG(netsurf, INFO,
+					      "reading block file %d use index on file number %d failed",
+					      elem_idx,
+					      bfidx);
 					goto rd_err;
 				}
 			}
@@ -1274,8 +1295,8 @@ read_blocks(struct store_state *state)
 		close(fd);
 
 	} else {
-		LOG("Initialising block use map to defaults");
-		/* ensure block 0 (invalid sentinal) is skipped */
+		NSLOG(netsurf, INFO, "Initialising block use map to defaults");
+		/* ensure block 0 (invalid sentinel) is skipped */
 		state->blocks[ENTRY_ELEM_DATA][0].use_map[0] = 1;
 		state->blocks[ENTRY_ELEM_META][0].use_map[0] = 1;
 	}
@@ -1293,7 +1314,7 @@ read_blocks(struct store_state *state)
  * Write the cache tag file.
  *
  * @param state The cache state.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 write_cache_tag(struct store_state *state)
@@ -1330,7 +1351,7 @@ write_cache_tag(struct store_state *state)
  * Write the control file for the current state.
  *
  * @param state The state to write to the control file.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 write_control(struct store_state *state)
@@ -1344,7 +1365,7 @@ write_control(struct store_state *state)
 		return ret;
 	}
 
-	LOG("writing control file \"%s\"", fname);
+	NSLOG(netsurf, INFO, "writing control file \"%s\"", fname);
 
 	ret = netsurf_mkdir_all(fname);
 	if (ret != NSERROR_OK) {
@@ -1375,7 +1396,7 @@ write_control(struct store_state *state)
  * Read and parse the control file.
  *
  * @param state The state to read from the control file.
- * @return NSERROR_OK on sucess or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 read_control(struct store_state *state)
@@ -1392,7 +1413,7 @@ read_control(struct store_state *state)
 		return ret;
 	}
 
-	LOG("opening control file \"%s\"", fname);
+	NSLOG(netsurf, INFO, "opening control file \"%s\"", fname);
 
 	fcontrol = fopen(fname, "rb");
 
@@ -1461,7 +1482,7 @@ control_error: /* problem with the control file */
  * Initialise the backing store.
  *
  * @param parameters to configure backing store.
- * @return NSERROR_OK on success or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 initialise(const struct llcache_store_parameters *parameters)
@@ -1509,7 +1530,8 @@ initialise(const struct llcache_store_parameters *parameters)
 	/* read store control and create new if required */
 	ret = read_control(newstate);
 	if (ret != NSERROR_OK) {
-		LOG("read control failed %s", messages_get_errorcode(ret));
+		NSLOG(netsurf, INFO, "read control failed %s",
+		      messages_get_errorcode(ret));
 		ret = write_control(newstate);
 		if (ret == NSERROR_OK) {
 			unlink_entries(newstate);
@@ -1543,6 +1565,7 @@ initialise(const struct llcache_store_parameters *parameters)
 	ret = build_entrymap(newstate);
 	if (ret != NSERROR_OK) {
 		/* that obviously went well */
+		free(newstate->entries);
 		free(newstate->path);
 		free(newstate);
 		return ret;
@@ -1551,6 +1574,8 @@ initialise(const struct llcache_store_parameters *parameters)
 	ret = read_blocks(newstate);
 	if (ret != NSERROR_OK) {
 		/* oh dear */
+		free(newstate->addrmap);
+		free(newstate->entries);
 		free(newstate->path);
 		free(newstate);
 		return ret;
@@ -1558,15 +1583,17 @@ initialise(const struct llcache_store_parameters *parameters)
 
 	storestate = newstate;
 
-	LOG("FS backing store init successful");
+	NSLOG(netsurf, INFO, "FS backing store init successful");
 
-	LOG("path:%s limit:%zd hyst:%zd addr:%d entries:%d",
-	    newstate->path,
-	    newstate->limit,
-	    newstate->hysteresis,
-	    newstate->ident_bits,
-	    newstate->entry_bits);
-	LOG("Using %"PRIu64"/%zd", newstate->total_alloc, newstate->limit);
+	NSLOG(netsurf, INFO,
+	      "path:%s limit:%"PRIsizet" hyst:%"PRIsizet" addr:%d entries:%d",
+	      newstate->path,
+	      newstate->limit,
+	      newstate->hysteresis,
+	      newstate->ident_bits,
+	      newstate->entry_bits);
+	NSLOG(netsurf, INFO, "Using %"PRIu64"/%"PRIsizet,
+	      newstate->total_alloc, newstate->limit);
 
 	return NSERROR_OK;
 }
@@ -1605,16 +1632,19 @@ finalise(void)
 
 		/* avoid division by zero */
 		if (op_count > 0) {
-			LOG("Cache total/hit/miss/fail (counts) %d/%zd/%zd/%d (100%%/%zd%%/%zd%%/%d%%)",
-			    op_count,
-			    storestate->hit_count,
-			    storestate->miss_count,
-			    0,
-			    (storestate->hit_count * 100) / op_count,
-			    (storestate->miss_count * 100) / op_count,
-			    0);
+			NSLOG(netsurf, INFO,
+			      "Cache total/hit/miss/fail (counts) %d/%"PRIsizet"/%"PRIsizet"/%d (100%%/%"PRIsizet"%%/%"PRIsizet"%%/%d%%)",
+			      op_count,
+			      storestate->hit_count,
+			      storestate->miss_count,
+			      0,
+			      (storestate->hit_count * 100) / op_count,
+			      (storestate->miss_count * 100) / op_count,
+			      0);
 		}
 
+		free(storestate->addrmap);
+		free(storestate->entries);
 		free(storestate->path);
 		free(storestate);
 		storestate = NULL;
@@ -1646,7 +1676,7 @@ static nserror store_write_block(struct store_state *state,
 		state->blocks[elem_idx][bf].fd = store_open(state, bf,
 				elem_idx + ENTRY_ELEM_COUNT, O_CREAT | O_RDWR);
 		if (state->blocks[elem_idx][bf].fd == -1) {
-			LOG("Open failed errno %d", errno);
+			NSLOG(netsurf, INFO, "Open failed errno %d", errno);
 			return NSERROR_SAVE_FAILED;
 		}
 
@@ -1654,28 +1684,28 @@ static nserror store_write_block(struct store_state *state,
 		state->blocks_opened = true;
 	}
 
-	offst = bi << log2_block_size[elem_idx];
+	offst = (unsigned int)bi << log2_block_size[elem_idx];
 
 	wr = nsu_pwrite(state->blocks[elem_idx][bf].fd,
 		    bse->elem[elem_idx].data,
 		    bse->elem[elem_idx].size,
 		    offst);
 	if (wr != (ssize_t)bse->elem[elem_idx].size) {
-		LOG("Write failed %zd of %d bytes from %p at 0x%jx block %d errno %d",
-		    wr,
-		    bse->elem[elem_idx].size,
-		    bse->elem[elem_idx].data,
-		    (uintmax_t)offst,
-		    bse->elem[elem_idx].block,
-		    errno);
+		NSLOG(netsurf, INFO,
+		      "Write failed %"PRIssizet" of %d bytes from %p at 0x%jx block %d errno %d",
+		      wr,
+		      bse->elem[elem_idx].size,
+		      bse->elem[elem_idx].data,
+		      (uintmax_t)offst,
+		      bse->elem[elem_idx].block,
+		      errno);
 		return NSERROR_SAVE_FAILED;
 	}
 
-	LOG("Wrote %zd bytes from %p at 0x%jx block %d",
-	    wr,
-	    bse->elem[elem_idx].data,
-	    (uintmax_t)offst,
-	    bse->elem[elem_idx].block);
+	NSLOG(netsurf, INFO,
+	      "Wrote %"PRIssizet" bytes from %p at 0x%jx block %d", wr,
+	      bse->elem[elem_idx].data, (uintmax_t)offst,
+	      bse->elem[elem_idx].block);
 
 	return NSERROR_OK;
 }
@@ -1699,7 +1729,7 @@ static nserror store_write_file(struct store_state *state,
 	fd = store_open(state, bse->ident, elem_idx, O_CREAT | O_WRONLY);
 	if (fd < 0) {
 		perror("");
-		LOG("Open failed %d errno %d", fd, errno);
+		NSLOG(netsurf, INFO, "Open failed %d errno %d", fd, errno);
 		return NSERROR_SAVE_FAILED;
 	}
 
@@ -1708,17 +1738,19 @@ static nserror store_write_file(struct store_state *state,
 
 	close(fd);
 	if (wr != (ssize_t)bse->elem[elem_idx].size) {
-		LOG("Write failed %zd of %d bytes from %p errno %d",
-		    wr,
-		    bse->elem[elem_idx].size,
-		    bse->elem[elem_idx].data,
-		    err);
+		NSLOG(netsurf, INFO,
+		      "Write failed %"PRIssizet" of %d bytes from %p errno %d",
+		      wr,
+		      bse->elem[elem_idx].size,
+		      bse->elem[elem_idx].data,
+		      err);
 
 		/** @todo Delete the file? */
 		return NSERROR_SAVE_FAILED;
 	}
 
-	LOG("Wrote %zd bytes from %p", wr, bse->elem[elem_idx].data);
+	NSLOG(netsurf, INFO, "Wrote %"PRIssizet" bytes from %p", wr,
+	      bse->elem[elem_idx].data);
 
 	return NSERROR_OK;
 }
@@ -1732,7 +1764,7 @@ static nserror store_write_file(struct store_state *state,
  * @param bsflags The flags to control how the object is stored.
  * @param data The objects source data.
  * @param datalen The length of the \a data.
- * @return NSERROR_OK on success or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 store(nsurl *url,
@@ -1759,7 +1791,7 @@ store(nsurl *url,
 	/* set the store entry up */
 	ret = set_store_entry(storestate, url, elem_idx, data, datalen, &bse);
 	if (ret != NSERROR_OK) {
-		LOG("store entry setting failed");
+		NSLOG(netsurf, INFO, "store entry setting failed");
 		return ret;
 	}
 
@@ -1782,7 +1814,7 @@ static nserror entry_release_alloc(struct store_entry_element *elem)
 	if ((elem->flags & ENTRY_ELEM_FLAG_HEAP) != 0) {
 		elem->ref--;
 		if (elem->ref == 0) {
-			LOG("freeing %p", elem->data);
+			NSLOG(netsurf, INFO, "freeing %p", elem->data);
 			free(elem->data);
 			elem->flags &= ~ENTRY_ELEM_FLAG_HEAP;
 		}
@@ -1814,7 +1846,7 @@ static nserror store_read_block(struct store_state *state,
 		state->blocks[elem_idx][bf].fd = store_open(state, bf,
 				elem_idx + ENTRY_ELEM_COUNT, O_CREAT | O_RDWR);
 		if (state->blocks[elem_idx][bf].fd == -1) {
-			LOG("Open failed errno %d", errno);
+			NSLOG(netsurf, INFO, "Open failed errno %d", errno);
 			return NSERROR_SAVE_FAILED;
 		}
 
@@ -1822,28 +1854,28 @@ static nserror store_read_block(struct store_state *state,
 		state->blocks_opened = true;
 	}
 
-	offst = bi << log2_block_size[elem_idx];
+	offst = (unsigned int)bi << log2_block_size[elem_idx];
 
 	rd = nsu_pread(state->blocks[elem_idx][bf].fd,
 		   bse->elem[elem_idx].data,
 		   bse->elem[elem_idx].size,
 		   offst);
 	if (rd != (ssize_t)bse->elem[elem_idx].size) {
-		LOG("Failed reading %zd of %d bytes into %p from 0x%jx block %d errno %d",
-		    rd,
-		    bse->elem[elem_idx].size,
-		    bse->elem[elem_idx].data,
-		    (uintmax_t)offst,
-		    bse->elem[elem_idx].block,
-		    errno);
+		NSLOG(netsurf, INFO,
+		      "Failed reading %"PRIssizet" of %d bytes into %p from 0x%jx block %d errno %d",
+		      rd,
+		      bse->elem[elem_idx].size,
+		      bse->elem[elem_idx].data,
+		      (uintmax_t)offst,
+		      bse->elem[elem_idx].block,
+		      errno);
 		return NSERROR_SAVE_FAILED;
 	}
 
-	LOG("Read %zd bytes into %p from 0x%jx block %d",
-	    rd,
-	    bse->elem[elem_idx].data,
-	    (uintmax_t)offst,
-	    bse->elem[elem_idx].block);
+	NSLOG(netsurf, INFO,
+	      "Read %"PRIssizet" bytes into %p from 0x%jx block %d", rd,
+	      bse->elem[elem_idx].data, (uintmax_t)offst,
+	      bse->elem[elem_idx].block);
 
 	return NSERROR_OK;
 }
@@ -1868,7 +1900,7 @@ static nserror store_read_file(struct store_state *state,
 	/* separate file in backing store */
 	fd = store_open(storestate, bse->ident, elem_idx, O_RDONLY);
 	if (fd < 0) {
-		LOG("Open failed %d errno %d", fd, errno);
+		NSLOG(netsurf, INFO, "Open failed %d errno %d", fd, errno);
 		/** @todo should this invalidate the entry? */
 		return NSERROR_NOT_FOUND;
 	}
@@ -1878,7 +1910,10 @@ static nserror store_read_file(struct store_state *state,
 			  bse->elem[elem_idx].data + tot,
 			  bse->elem[elem_idx].size - tot);
 		if (rd <= 0) {
-			LOG("read error returned %zd errno %d", rd, errno);
+			NSLOG(netsurf, INFO,
+			      "read error returned %"PRIssizet" errno %d",
+			      rd,
+			      errno);
 			ret = NSERROR_NOT_FOUND;
 			break;
 		}
@@ -1887,19 +1922,20 @@ static nserror store_read_file(struct store_state *state,
 
 	close(fd);
 
-	LOG("Read %zd bytes into %p", tot, bse->elem[elem_idx].data);
+	NSLOG(netsurf, INFO, "Read %"PRIsizet" bytes into %p", tot,
+	      bse->elem[elem_idx].data);
 
 	return ret;
 }
 
 /**
- * Retrive an object from the backing store.
+ * Retrieve an object from the backing store.
  *
  * @param[in] url The url is used as the unique primary key for the data.
  * @param[in] bsflags The flags to control how the object is retrieved.
  * @param[out] data_out The objects data.
  * @param[out] datalen_out The length of the \a data retrieved.
- * @return NSERROR_OK on success or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 fetch(nsurl *url,
@@ -1920,13 +1956,14 @@ fetch(nsurl *url,
 	/* fetch store entry */
 	ret = get_store_entry(storestate, url, &bse);
 	if (ret != NSERROR_OK) {
-		LOG("entry not found");
+		NSLOG(netsurf, INFO, "entry not found");
 		storestate->miss_count++;
 		return ret;
 	}
 	storestate->hit_count++;
 
-	LOG("retriving cache data for url:%s", nsurl_access(url));
+	NSLOG(netsurf, INFO, "retrieving cache data for url:%s",
+	      nsurl_access(url));
 
 	/* calculate the entry element index */
 	if ((bsflags & BACKING_STORE_META) != 0) {
@@ -1941,16 +1978,20 @@ fetch(nsurl *url,
 		/* use the existing allocation and bump the ref count. */
 		elem->ref++;
 
-		LOG("Using existing entry (%p) allocation %p refs:%d", bse, elem->data, elem->ref);
+		NSLOG(netsurf, INFO,
+		      "Using existing entry (%p) allocation %p refs:%d", bse,
+		      elem->data, elem->ref);
 
 	} else {
 		/* allocate from the heap */
 		elem->data = malloc(elem->size);
 		if (elem->data == NULL) {
-			LOG("Failed to create new heap allocation");
+			NSLOG(netsurf, INFO,
+			      "Failed to create new heap allocation");
 			return NSERROR_NOMEM;
 		}
-		LOG("Created new heap allocation %p", elem->data);
+		NSLOG(netsurf, INFO, "Created new heap allocation %p",
+		      elem->data);
 
 		/* mark the entry as having a valid heap allocation */
 		elem->flags |= ENTRY_ELEM_FLAG_HEAP;
@@ -1984,7 +2025,7 @@ fetch(nsurl *url,
  *
  * @param[in] url The url is used as the unique primary key to invalidate.
  * @param[in] bsflags The flags to control how the object data is released.
- * @return NSERROR_OK on success or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror release(nsurl *url, enum backing_store_flags bsflags)
 {
@@ -1999,7 +2040,7 @@ static nserror release(nsurl *url, enum backing_store_flags bsflags)
 
 	ret = get_store_entry(storestate, url, &bse);
 	if (ret != NSERROR_OK) {
-		LOG("entry not found");
+		NSLOG(netsurf, INFO, "entry not found");
 		return ret;
 	}
 
@@ -2032,7 +2073,7 @@ static nserror release(nsurl *url, enum backing_store_flags bsflags)
  * be returned as a result to the fetch or meta operations.
  *
  * @param url The url is used as the unique primary key to invalidate.
- * @return NSERROR_OK on success or error code on faliure.
+ * @return NSERROR_OK on success or error code on failure.
  */
 static nserror
 invalidate(nsurl *url)
