@@ -80,8 +80,6 @@ typedef struct {
 						 *   imports array */
 } nscss_import_ctx;
 
-static bool nscss_process_data(struct content *c, const char *data,
-		unsigned int size);
 static bool nscss_convert(struct content *c);
 static void nscss_destroy(struct content *c);
 static nserror nscss_clone(const struct content *old, struct content **newc);
@@ -98,7 +96,7 @@ static void nscss_destroy_css_data(struct content_css_data *c);
 
 static void nscss_content_done(struct content_css_data *css, void *pw);
 static css_error nscss_handle_import(void *pw, css_stylesheet *parent,
-		lwc_string *url, uint64_t media);
+		lwc_string *url);
 static nserror nscss_import(hlcache_handle *handle,
 		const hlcache_event *event, void *pw);
 static css_error nscss_import_complete(nscss_import_ctx *ctx);
@@ -136,7 +134,6 @@ nscss_create(const content_handler *handler,
 	const char *charset = NULL;
 	const char *xnsbase = NULL;
 	lwc_string *charset_value = NULL;
-	union content_msg_data msg_data;
 	nserror error;
 
 	result = calloc(1, sizeof(nscss_content));
@@ -171,8 +168,7 @@ nscss_create(const content_handler *handler,
 			xnsbase, charset, result->base.quirks,
 			nscss_content_done, result);
 	if (error != NSERROR_OK) {
-		msg_data.error = messages_get("NoMemory");
-		content_broadcast(&result->base, CONTENT_MSG_ERROR, msg_data);
+		content_broadcast_errorcode(&result->base, NSERROR_NOMEM);
 		if (charset_value != NULL)
 			lwc_string_unref(charset_value);
 		free(result);
@@ -247,16 +243,15 @@ static nserror nscss_create_css_data(struct content_css_data *c,
  * \param size  Number of bytes to process
  * \return true on success, false on failure
  */
-bool nscss_process_data(struct content *c, const char *data, unsigned int size)
+static bool
+nscss_process_data(struct content *c, const char *data, unsigned int size)
 {
 	nscss_content *css = (nscss_content *) c;
-	union content_msg_data msg_data;
 	css_error error;
 
 	error = nscss_process_css_data(&css->data, data, size);
 	if (error != CSS_OK && error != CSS_NEEDDATA) {
-		msg_data.error = "?";
-		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+		content_broadcast_errorcode(c, NSERROR_CSS);
 	}
 
 	return (error == CSS_OK || error == CSS_NEEDDATA);
@@ -286,13 +281,11 @@ static css_error nscss_process_css_data(struct content_css_data *c,
 bool nscss_convert(struct content *c)
 {
 	nscss_content *css = (nscss_content *) c;
-	union content_msg_data msg_data;
 	css_error error;
 
 	error = nscss_convert_css_data(&css->data);
 	if (error != CSS_OK) {
-		msg_data.error = "?";
-		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+		content_broadcast_errorcode(c, NSERROR_CSS);
 		return false;
 	}
 
@@ -327,9 +320,11 @@ static css_error nscss_convert_css_data(struct content_css_data *c)
 		const char *url;
 
 		if (css_stylesheet_get_url(c->sheet, &url) == CSS_OK) {
-			LOG("Failed converting %p %s (%d)", c, url, error);
+			NSLOG(netsurf, INFO, "Failed converting %p %s (%d)",
+			      c, url, error);
 		} else {
-			LOG("Failed converting %p (%d)", c, error);
+			NSLOG(netsurf, INFO, "Failed converting %p (%d)", c,
+			      error);
 		}
 	}
 
@@ -378,8 +373,8 @@ nserror nscss_clone(const struct content *old, struct content **newc)
 {
 	const nscss_content *old_css = (const nscss_content *) old;
 	nscss_content *new_css;
-	const char *data;
-	unsigned long size;
+	const uint8_t *data;
+	size_t size;
 	nserror error;
 
 	new_css = calloc(1, sizeof(nscss_content));
@@ -406,7 +401,9 @@ nserror nscss_clone(const struct content *old, struct content **newc)
 
 	data = content__get_source_data(&new_css->base, &size);
 	if (size > 0) {
-		if (nscss_process_data(&new_css->base, data, size) == false) {
+		if (nscss_process_data(&new_css->base,
+				       (char *)data,
+				       (unsigned int)size) == false) {
 			content_destroy(&new_css->base);
 			return NSERROR_CLONE_FAILED;
 		}
@@ -475,7 +472,6 @@ content_type nscss_content_type(void)
  */
 void nscss_content_done(struct content_css_data *css, void *pw)
 {
-	union content_msg_data msg_data;
 	struct content *c = pw;
 	uint32_t i;
 	size_t size;
@@ -484,8 +480,7 @@ void nscss_content_done(struct content_css_data *css, void *pw)
 	/* Retrieve the size of this sheet */
 	error = css_stylesheet_size(css->sheet, &size);
 	if (error != CSS_OK) {
-		msg_data.error = "?";
-		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+		content_broadcast_errorcode(c, NSERROR_CSS);
 		content_set_error(c);
 		return;
 	}
@@ -518,11 +513,10 @@ void nscss_content_done(struct content_css_data *css, void *pw)
  * \param pw      CSS object requesting the import
  * \param parent  Stylesheet requesting the import
  * \param url     URL of the imported sheet
- * \param media   Applicable media for the imported sheet
  * \return CSS_OK on success, appropriate error otherwise
  */
 css_error nscss_handle_import(void *pw, css_stylesheet *parent,
-		lwc_string *url, uint64_t media)
+		lwc_string *url)
 {
 	content_type accept = CONTENT_CSS;
 	struct content_css_data *c = pw;
@@ -568,7 +562,6 @@ css_error nscss_handle_import(void *pw, css_stylesheet *parent,
 	}
 
 	/* Create content */
-	c->imports[c->import_count].media = media;
 
 	/** \todo Why aren't we getting a relative url part, to join? */
 	nerror = nsurl_create(lwc_string_data(url), &ns_url);
@@ -606,7 +599,9 @@ css_error nscss_handle_import(void *pw, css_stylesheet *parent,
 	nsurl_unref(ns_ref);
 
 #ifdef NSCSS_IMPORT_TRACE
-	LOG("Import %d '%s' -> (handle: %p ctx: %p)", c->import_count, lwc_string_data(url), c->imports[c->import_count].c, ctx);
+	NSLOG(netsurf, INFO, "Import %d '%s' -> (handle: %p ctx: %p)",
+	      c->import_count, lwc_string_data(url),
+	      c->imports[c->import_count].c, ctx);
 #endif
 
 	c->import_count++;
@@ -629,7 +624,7 @@ nserror nscss_import(hlcache_handle *handle,
 	css_error error = CSS_OK;
 
 #ifdef NSCSS_IMPORT_TRACE
-	LOG("Event %d for %p (%p)", event->type, handle, ctx);
+	NSLOG(netsurf, INFO, "Event %d for %p (%p)", event->type, handle, ctx);
 #endif
 
 	assert(ctx->css->imports[ctx->index].c == handle);
@@ -639,6 +634,7 @@ nserror nscss_import(hlcache_handle *handle,
 		error = nscss_import_complete(ctx);
 		break;
 
+	case CONTENT_MSG_ERRORCODE:
 	case CONTENT_MSG_ERROR:
 		hlcache_handle_release(handle);
 		ctx->css->imports[ctx->index].c = NULL;
@@ -670,7 +666,8 @@ css_error nscss_import_complete(nscss_import_ctx *ctx)
 		error = nscss_register_imports(ctx->css);
 
 #ifdef NSCSS_IMPORT_TRACE
-	LOG("Destroying import context %p for %d", ctx, ctx->index);
+	NSLOG(netsurf, INFO, "Destroying import context %p for %d", ctx,
+	      ctx->index);
 #endif
 
 	/* No longer need import context */
