@@ -82,12 +82,6 @@ static void gui_default_window_set_icon(struct gui_window *g,
 {
 }
 
-static void gui_default_window_scroll_visible(struct gui_window *g,
-				       int x0, int y0,
-				       int x1, int y1)
-{
-	guit->window->set_scroll(g, x0, y0);
-}
 
 static void gui_default_window_new_content(struct gui_window *g)
 {
@@ -145,6 +139,15 @@ static void gui_default_window_start_selection(struct gui_window *g)
 {
 }
 
+static void
+gui_default_console_log(struct gui_window *gw,
+			browser_window_console_source src,
+			const char *msg,
+			size_t msglen,
+			browser_window_console_flags flags)
+{
+}
+
 
 /** verify window table is valid */
 static nserror verify_window_register(struct gui_window_table *gwt)
@@ -161,10 +164,7 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->destroy == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
-	if (gwt->redraw == NULL) {
-		return NSERROR_BAD_PARAMETER;
-	}
-	if (gwt->update == NULL) {
+	if (gwt->invalidate == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 	if (gwt->get_scroll == NULL) {
@@ -179,9 +179,6 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->update_extent == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
-	if (gwt->reformat == NULL) {
-		return NSERROR_BAD_PARAMETER;
-	}
 
 
 	/* fill in the optional entries with defaults */
@@ -218,9 +215,6 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->save_link == NULL) {
 		gwt->save_link = gui_default_window_save_link;
 	}
-	if (gwt->scroll_visible == NULL) {
-		gwt->scroll_visible = gui_default_window_scroll_visible;
-	}
 	if (gwt->new_content == NULL) {
 		gwt->new_content = gui_default_window_new_content;
 	}
@@ -243,6 +237,9 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->start_selection == NULL) {
 		gwt->start_selection = gui_default_window_start_selection;
 	}
+	if (gwt->console_log == NULL) {
+		gwt->console_log = gui_default_console_log;
+	}
 
 	return NSERROR_OK;
 }
@@ -694,10 +691,14 @@ static nserror gui_default_cert_verify(nsurl *url,
 	return NSERROR_NOT_IMPLEMENTED;
 }
 
-static void gui_default_401login_open(nsurl *url, const char *realm,
-		nserror (*cb)(bool proceed, void *pw), void *cbpw)
+static nserror gui_default_401login_open(nsurl *url, const char *realm,
+		const char *username, const char *password,
+		nserror (*cb)(const char *username,
+				const char *password,
+				void *pw),
+		void *cbpw)
 {
-	cb(false, cbpw);
+	return NSERROR_NOT_IMPLEMENTED;
 }
 
 static void
