@@ -62,6 +62,8 @@ struct hotlist_ctx {
 	struct treeview_field_desc fields[HL_N_FIELDS];
 	bool built;
 	struct hotlist_folder *default_folder;
+	char *save_path;
+	bool save_scheduled;
 };
 struct hotlist_ctx hl_ctx;
 
@@ -73,6 +75,113 @@ struct hotlist_entry {
 };
 
 
+/*
+ * Get path for writing hotlist to
+ *
+ * \param path		The final path of the hotlist
+ * \param loaded	Updated to the path to write the holist to
+ * \return NSERROR_OK on success, or appropriate error otherwise
+ */
+static nserror hotlist_get_temp_path(const char *path, char **temp_path)
+{
+	const char *extension = "-bk";
+	char *joined;
+	int len;
+
+	len = strlen(path) + strlen(extension);
+
+	joined = malloc(len + 1);
+	if (joined == NULL) {
+		return NSERROR_NOMEM;
+	}
+
+	if (snprintf(joined, len + 1, "%s%s", path, extension) != len) {
+		free(joined);
+		return NSERROR_UNKNOWN;
+	}
+
+	*temp_path = joined;
+	return NSERROR_OK;
+}
+
+
+/* Save the hotlist to to a file at the given path
+ *
+ * \param path  Path to save hotlist file to.  NULL path is a no-op.
+ * \return NSERROR_OK on success, or appropriate error otherwise
+ */
+static nserror hotlist_save(const char *path)
+{
+	nserror res = NSERROR_OK;
+	char *temp_path;
+
+	/* NULL path is a no-op. */
+	if (path == NULL) {
+		return NSERROR_OK;
+	}
+
+	/* Get path to export to */
+	res = hotlist_get_temp_path(path, &temp_path);
+	if (res != NSERROR_OK) {
+		return res;
+	}
+
+	/* Export to temp path */
+	res = hotlist_export(temp_path, NULL);
+	if (res != NSERROR_OK) {
+		goto cleanup;
+	}
+
+	/* Remove old hotlist to handle non-POSIX rename() implementations. */
+	(void)remove(path);
+
+	/* Replace any old hotlist file with the one we just saved */
+	if (rename(temp_path, path) != 0) {
+		res = NSERROR_SAVE_FAILED;
+		NSLOG(netsurf, INFO, "Error renaming hotlist: %s.",
+		      strerror(errno));
+		goto cleanup;
+	}
+
+cleanup:
+	free(temp_path);
+
+	return res;
+}
+
+
+/**
+ * Scheduler callback for saving the hotlist.
+ *
+ * \param p  Unused user data.
+ */
+static void hotlist_schedule_save_cb(void *p)
+{
+	hl_ctx.save_scheduled = false;
+	hotlist_save(hl_ctx.save_path);
+}
+
+
+/**
+ * Schedule a hotlist save.
+ *
+ * \return NSERROR_OK on success, or appropriate error otherwise
+ */
+static nserror hotlist_schedule_save(void)
+{
+	if (hl_ctx.save_scheduled == false && hl_ctx.save_path != NULL) {
+		nserror err = guit->misc->schedule(10 * 1000,
+				hotlist_schedule_save_cb, NULL);
+		if (err != NSERROR_OK) {
+			return err;
+		}
+		hl_ctx.save_scheduled = true;
+	}
+
+	return NSERROR_OK;
+}
+
+
 /**
  * Set a hotlist entry's data from the url_data.
  *
@@ -436,6 +545,8 @@ hotlist_tree_node_entry_cb(struct treeview_node_msg msg, void *data)
 	case TREE_MSG_NODE_DELETE:
 		e->entry = NULL;
 		hotlist_delete_entry_internal(e);
+
+		err = hotlist_schedule_save();
 		break;
 
 	case TREE_MSG_NODE_EDIT:
@@ -459,6 +570,8 @@ hotlist_tree_node_entry_cb(struct treeview_node_msg msg, void *data)
 				free((void *)old_text);
 			}
 
+			err = hotlist_schedule_save();
+
 		} else if (lwc_string_isequal(hl_ctx.fields[HL_URL].field,
 				msg.data.node_edit.field, &match) ==
 				lwc_error_ok && match == true &&
@@ -476,6 +589,8 @@ hotlist_tree_node_entry_cb(struct treeview_node_msg msg, void *data)
 				treeview_update_node_entry(hl_ctx.tree,
 						   e->entry, e->data, e);
 				nsurl_unref(old_url);
+
+				err = hotlist_schedule_save();
 			}
 		}
 		break;
@@ -538,20 +653,20 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 	/* The li must contain an "a" element */
 	a = libdom_find_first_element(li, corestring_lwc_a);
 	if (a == NULL) {
-		LOG("Missing <a> in <li>");
+		NSLOG(netsurf, INFO, "Missing <a> in <li>");
 		return NSERROR_INVALID;
 	}
 
 	derror = dom_node_get_text_content(a, &title1);
 	if (derror != DOM_NO_ERR) {
-		LOG("No title");
+		NSLOG(netsurf, INFO, "No title");
 		dom_node_unref(a);
 		return NSERROR_INVALID;
 	}
 
 	derror = dom_element_get_attribute(a, corestring_dom_href, &url1);
 	if (derror != DOM_NO_ERR || url1 == NULL) {
-		LOG("No URL");
+		NSLOG(netsurf, INFO, "No URL");
 		dom_string_unref(title1);
 		dom_node_unref(a);
 		return NSERROR_INVALID;
@@ -569,7 +684,8 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 	dom_string_unref(url1);
 
 	if (err != NSERROR_OK) {
-		LOG("Failed normalising '%s'", dom_string_data(url1));
+		NSLOG(netsurf, INFO, "Failed normalising '%s'",
+		      dom_string_data(url1));
 
 		if (title1 != NULL) {
 			dom_string_unref(title1);
@@ -596,7 +712,7 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 
 
 /*
- * Callback for libdom_iterate_child_elements, which dispite the namespace is
+ * Callback for libdom_iterate_child_elements, which despite the namespace is
  * a NetSurf function.
  *
  * \param node		Node that is a child of the directory UL node
@@ -649,7 +765,8 @@ nserror hotlist_load_directory_cb(dom_node *node, void *ctx)
 
 		error = dom_node_get_text_content(node, &title);
 		if (error != DOM_NO_ERR || title == NULL) {
-			LOG("Empty <h4> or memory exhausted.");
+			NSLOG(netsurf, INFO,
+			      "Empty <h4> or memory exhausted.");
 			dom_string_unref(name);
 			return NSERROR_DOM;
 		}
@@ -728,36 +845,6 @@ nserror hotlist_load_directory_cb(dom_node *node, void *ctx)
 }
 
 
-/*
- * Get path for writing hotlist to
- *
- * \param path		The final path of the hotlist
- * \param loaded	Updated to the path to write the holist to
- * \return NSERROR_OK on success, or appropriate error otherwise
- */
-static nserror hotlist_get_temp_path(const char *path, char **temp_path)
-{
-	const char *extension = "-bk";
-	char *joined;
-	int len;
-
-	len = strlen(path) + strlen(extension);
-
-	joined = malloc(len + 1);
-	if (joined == NULL) {
-		return NSERROR_NOMEM;
-	}
-
-	if (snprintf(joined, len + 1, "%s%s", path, extension) != len) {
-		free(joined);
-		return NSERROR_UNKNOWN;
-	}
-
-	*temp_path = joined;
-	return NSERROR_OK;
-}
-
-
 /*
  * Load the hotlist data from file
  *
@@ -777,7 +864,7 @@ static nserror hotlist_load(const char *path, bool *loaded)
 
 	/* Handle no path */
 	if (path == NULL) {
-		LOG("No hotlist file path provided.");
+		NSLOG(netsurf, INFO, "No hotlist file path provided.");
 		return NSERROR_OK;
 	}
 
@@ -918,45 +1005,6 @@ static nserror hotlist_generate(void)
 }
 
 
-/* Save the hotlist to to a file at the given path
- *
- * \param path  Path to save hostlist file to.
- * \return NSERROR_OK on success, or appropriate error otherwise
- */
-static nserror hotlist_save(const char *path)
-{
-	nserror res = NSERROR_OK;
-	char *temp_path;
-
-	/* Get path to export to */
-	res = hotlist_get_temp_path(path, &temp_path);
-	if (res != NSERROR_OK) {
-		return res;
-	}
-
-	/* Export to temp path */
-	res = hotlist_export(temp_path, NULL);
-	if (res != NSERROR_OK) {
-		goto cleanup;
-	}
-
-	/* Remove old hotlist to handle non-POSIX rename() implementations. */
-	(void)remove(path);
-
-	/* Replace any old hotlist file with the one we just saved */
-	if (rename(temp_path, path) != 0) {
-		res = NSERROR_SAVE_FAILED;
-		LOG("Error renaming hotlist: %s.", strerror(errno));
-		goto cleanup;
-	}
-
-cleanup:
-	free(temp_path);
-
-	return res;
-}
-
-
 struct treeview_export_walk_ctx {
 	FILE *fp;
 };
@@ -1150,8 +1198,10 @@ static nserror hotlist_initialise_entry_fields(void)
 		goto error;
 	}
 
-	hl_ctx.fields[HL_URL].flags = TREE_FLAG_ALLOW_EDIT |
-			TREE_FLAG_COPY_TEXT;
+	hl_ctx.fields[HL_URL].flags =
+			TREE_FLAG_ALLOW_EDIT |
+			TREE_FLAG_COPY_TEXT |
+			TREE_FLAG_SEARCHABLE;
 	label = "TreeviewLabelURL";
 	label = messages_get(label);
 	if (lwc_intern_string(label, strlen(label),
@@ -1227,8 +1277,9 @@ static nserror hotlist_populate(const char *path)
 
 
 /* Exported interface, documented in hotlist.h */
-nserror hotlist_init(struct core_window_callback_table *cw_t,
-		void *core_window_handle, const char *path)
+nserror hotlist_init(
+		const char *load_path,
+		const char *save_path)
 {
 	nserror err;
 
@@ -1237,32 +1288,44 @@ nserror hotlist_init(struct core_window_callback_table *cw_t,
 		return err;
 	}
 
-	LOG("Loading hotlist");
+	NSLOG(netsurf, INFO, "Loading hotlist");
 
 	hl_ctx.tree = NULL;
 	hl_ctx.built = false;
 	hl_ctx.default_folder = NULL;
 
+	/* Store the save path */
+	if (save_path != NULL) {
+		hl_ctx.save_path = strdup(save_path);
+		if (hl_ctx.save_path == NULL) {
+			return NSERROR_NOMEM;
+		}
+	} else {
+		hl_ctx.save_path = NULL;
+	}
+
 	/* Init. hotlist treeview entry fields */
 	err = hotlist_initialise_entry_fields();
 	if (err != NSERROR_OK) {
+		free(hl_ctx.save_path);
 		hl_ctx.tree = NULL;
 		return err;
 	}
 
 	/* Create the hotlist treeview */
 	err = treeview_create(&hl_ctx.tree, &hl_tree_cb_t,
-			HL_N_FIELDS, hl_ctx.fields,
-			cw_t, core_window_handle,
-			TREEVIEW_NO_FLAGS);
+			HL_N_FIELDS, hl_ctx.fields, NULL, NULL,
+			TREEVIEW_SEARCHABLE);
 	if (err != NSERROR_OK) {
+		free(hl_ctx.save_path);
 		hl_ctx.tree = NULL;
 		return err;
 	}
 
 	/* Populate the hotlist */
-	err = hotlist_populate(path);
+	err = hotlist_populate(load_path);
 	if (err != NSERROR_OK) {
+		free(hl_ctx.save_path);
 		return err;
 	}
 
@@ -1271,29 +1334,66 @@ nserror hotlist_init(struct core_window_callback_table *cw_t,
 	 * the treeview is built. */
 	hl_ctx.built = true;
 
-	/* Inform client of window height */
-	treeview_get_height(hl_ctx.tree);
-
-	LOG("Loaded hotlist");
+	NSLOG(netsurf, INFO, "Loaded hotlist");
 
 	return NSERROR_OK;
 }
 
 
 /* Exported interface, documented in hotlist.h */
-nserror hotlist_fini(const char *path)
+nserror hotlist_manager_init(struct core_window_callback_table *cw_t,
+		void *core_window_handle)
+{
+	nserror err;
+
+	/* Create the hotlist treeview */
+	err = treeview_cw_attach(hl_ctx.tree, cw_t, core_window_handle);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	/* Inform client of window height */
+	treeview_get_height(hl_ctx.tree);
+
+	return NSERROR_OK;
+}
+
+
+/* Exported interface, documented in hotlist.h */
+nserror hotlist_manager_fini(void)
+{
+	nserror err;
+
+	/* Create the hotlist treeview */
+	err = treeview_cw_detach(hl_ctx.tree);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	return NSERROR_OK;
+}
+
+
+/* Exported interface, documented in hotlist.h */
+nserror hotlist_fini(void)
 {
 	int i;
 	nserror err;
 
-	LOG("Finalising hotlist");
+	NSLOG(netsurf, INFO, "Finalising hotlist");
+
+	/* Remove any existing scheduled save callback */
+	guit->misc->schedule(-1, hotlist_schedule_save_cb, NULL);
+	hl_ctx.save_scheduled = false;
 
 	/* Save the hotlist */
-	err = hotlist_save(path);
+	err = hotlist_save(hl_ctx.save_path);
 	if (err != NSERROR_OK) {
-		LOG("Problem saving the hotlist.");
+		NSLOG(netsurf, INFO, "Problem saving the hotlist.");
 	}
 
+	free(hl_ctx.save_path);
+
 	/* Destroy the hotlist treeview */
 	err = treeview_destroy(hl_ctx.tree);
 	hl_ctx.built = false;
@@ -1308,7 +1408,7 @@ nserror hotlist_fini(const char *path)
 		return err;
 	}
 
-	LOG("Finalised hotlist");
+	NSLOG(netsurf, INFO, "Finalised hotlist");
 
 	return err;
 }
@@ -1351,7 +1451,7 @@ nserror hotlist_add_url(nsurl *url)
 	if (err != NSERROR_OK)
 		return err;
 
-	return NSERROR_OK;
+	return hotlist_schedule_save();
 }
 
 
