@@ -30,7 +30,7 @@
 #ifndef _NETSURF_DESKTOP_OPTIONS_H_
 #define _NETSURF_DESKTOP_OPTIONS_H_
 
-#include "desktop/plot_style.h"
+#include "netsurf/types.h"
 
 /* defines for system colour table */
 #define NSOPTION_SYS_COLOUR_START NSOPTION_sys_colour_ActiveBorder
@@ -289,3 +289,8 @@ NSOPTION_COLOUR(sys_colour_ThreeDShadow, 0x00d5d5d5)
 NSOPTION_COLOUR(sys_colour_Window, 0x00f1f1f1)
 NSOPTION_COLOUR(sys_colour_WindowFrame, 0x004e4e4e)
 NSOPTION_COLOUR(sys_colour_WindowText, 0x00000000)
+
+/** Filter for non-verbose logging */
+NSOPTION_STRING(log_filter, NETSURF_BUILTIN_LOG_FILTER)
+/** Filter for verbose logging */
+NSOPTION_STRING(verbose_filter, NETSURF_BUILTIN_VERBOSE_FILTER)
