@@ -43,8 +43,9 @@
 #include "netsurf/content.h"
 #include "content/hlcache.h"
 #include "css/css.h"
-#include "render/box.h"
-#include "render/html.h"
+#include "html/box.h"
+#include "html/html_save.h"
+#include "html/html.h"
 
 #include "netsurf/misc.h"
 #include "desktop/gui_internal.h"
@@ -151,9 +152,12 @@ static bool save_complete_ctx_has_content(save_complete_ctx *ctx,
 	return false;
 }
 
-static bool save_complete_save_buffer(save_complete_ctx *ctx,
-		const char *leafname, const char *data, size_t data_len,
-		lwc_string *mime_type)
+static bool
+save_complete_save_buffer(save_complete_ctx *ctx,
+			  const char *leafname,
+			  const uint8_t *data,
+			  size_t data_len,
+			  lwc_string *mime_type)
 {
 	nserror ret;
 	FILE *fp;
@@ -168,7 +172,7 @@ static bool save_complete_save_buffer(save_complete_ctx *ctx,
 	fp = fopen(fname, "wb");
 	if (fp == NULL) {
 		free(fname);
-		LOG("fopen(): errno = %i", errno);
+		NSLOG(netsurf, INFO, "fopen(): errno = %i", errno);
 		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
@@ -195,20 +199,23 @@ static bool save_complete_save_buffer(save_complete_ctx *ctx,
  * \param osize updated with the size of the result.
  * \return converted source, or NULL on out of memory.
  */
-
-static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
-		const char *source, unsigned long size, const nsurl *base,
-		unsigned long *osize)
+static uint8_t *
+save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
+				      const uint8_t *source,
+				      size_t size,
+				      const nsurl *base,
+				      size_t *osize)
 {
-	char *rewritten;
+	uint8_t *rewritten;
 	unsigned long offset = 0;
 	unsigned int imports = 0;
 	nserror error;
 
 	/* count number occurrences of @import to (over)estimate result size */
 	/* can't use strstr because source is not 0-terminated string */
-	for (offset = 0; SLEN("@import") < size &&
-			offset <= size - SLEN("@import"); offset++) {
+	for (offset = 0;
+	     (SLEN("@import") < size) && (offset <= (size - SLEN("@import")));
+	     offset++) {
 		if (source[offset] == '@' &&
 		    ascii_to_lower(source[offset + 1]) == 'i' &&
 		    ascii_to_lower(source[offset + 2]) == 'm' &&
@@ -227,13 +234,16 @@ static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
 
 	offset = 0;
 	while (offset < size) {
-		const char *import_url = NULL;
+		const uint8_t *import_url = NULL;
 		char *import_url_copy;
 		int import_url_len = 0;
 		nsurl *url = NULL;
 		regmatch_t match[11];
-		int m = regexec(&save_complete_import_re, source + offset,
-				11, match, 0);
+		int m = regexec(&save_complete_import_re,
+				(const char *)source + offset,
+				11,
+				match,
+				0);
 		if (m)
 			break;
 
@@ -255,7 +265,8 @@ static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
 		}
 		assert(import_url != NULL);
 
-		import_url_copy = strndup(import_url, import_url_len);
+		import_url_copy = strndup((const char *)import_url,
+					  import_url_len);
 		if (import_url_copy == NULL) {
 			free(rewritten);
 			return NULL;
@@ -311,13 +322,13 @@ static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
 	return rewritten;
 }
 
-static bool save_complete_save_stylesheet(save_complete_ctx *ctx,
-		hlcache_handle *css)
+static bool
+save_complete_save_stylesheet(save_complete_ctx *ctx, hlcache_handle *css)
 {
-	const char *css_data;
-	unsigned long css_size;
-	char *source;
-	unsigned long source_len;
+	const uint8_t *css_data;
+	size_t css_size;
+	uint8_t *source;
+	size_t source_len;
 	struct nscss_import *imports;
 	uint32_t import_count;
 	lwc_string *type;
@@ -338,8 +349,12 @@ static bool save_complete_save_stylesheet(save_complete_ctx *ctx,
 		return false;
 
 	css_data = content_get_source_data(css, &css_size);
-	source = save_complete_rewrite_stylesheet_urls(ctx, css_data, css_size,
-			hlcache_handle_get_url(css), &source_len);
+	source = save_complete_rewrite_stylesheet_urls(
+		ctx,
+		css_data,
+		css_size,
+		hlcache_handle_get_url(css),
+		&source_len);
 	if (source == NULL) {
 		guit->misc->warning("NoMemory", 0);
 		return false;
@@ -404,11 +419,11 @@ static bool save_complete_save_html_stylesheets(save_complete_ctx *ctx,
 	return true;
 }
 
-static bool save_complete_save_html_object(save_complete_ctx *ctx,
-		hlcache_handle *obj)
+static bool
+save_complete_save_html_object(save_complete_ctx *ctx, hlcache_handle *obj)
 {
-	const char *obj_data;
-	unsigned long obj_size;
+	const uint8_t *obj_data;
+	size_t obj_size;
 	lwc_string *type;
 	bool result;
 	char filename[32];
@@ -637,9 +652,9 @@ static bool save_complete_handle_attr_value(save_complete_ctx *ctx,
 	 *
 	 * Attribute:      Elements:
 	 *
-	 * 1)   data         <object>
-	 * 2)   href         <a> <area> <link>
-	 * 3)   src          <script> <input> <frame> <iframe> <img>
+	 * 1)   data         object
+	 * 2)   href         a, area, link
+	 * 3)   src          script, input, frame, iframe, img
 	 * 4)   background   any (except those above)
 	 */
 	/* 1 */
@@ -894,13 +909,13 @@ static bool save_complete_handle_element(save_complete_ctx *ctx,
 		}
 
 		if (content != NULL) {
-			char *rewritten;
-			unsigned long len;
+			uint8_t *rewritten;
+			size_t len;
 
 			/* Rewrite @import rules */
 			rewritten = save_complete_rewrite_stylesheet_urls(
 					ctx,
-					dom_string_data(content),
+					(const uint8_t *)dom_string_data(content),
 					dom_string_byte_length(content),
 					ctx->base,
 					&len);
@@ -1044,7 +1059,7 @@ static bool save_complete_node_handler(dom_node *node,
 	} else if (type == DOM_DOCUMENT_NODE) {
 		/* Do nothing */
 	} else {
-		LOG("Unhandled node type: %d", type);
+		NSLOG(netsurf, INFO, "Unhandled node type: %d", type);
 	}
 
 	return true;
@@ -1075,7 +1090,7 @@ static bool save_complete_save_html_document(save_complete_ctx *ctx,
 	fp = fopen(fname, "wb");
 	if (fp == NULL) {
 		free(fname);
-		LOG("fopen(): errno = %i", errno);
+		NSLOG(netsurf, INFO, "fopen(): errno = %i", errno);
 		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
@@ -1154,7 +1169,7 @@ static bool save_complete_inventory(save_complete_ctx *ctx)
 	fp = fopen(fname, "w");
 	free(fname);
 	if (fp == NULL) {
-		LOG("fopen(): errno = %i", errno);
+		NSLOG(netsurf, INFO, "fopen(): errno = %i", errno);
 		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
@@ -1182,7 +1197,8 @@ static nserror regcomp_wrapper(regex_t *preg, const char *regex, int cflags)
 	if (r) {
 		char errbuf[200];
 		regerror(r, preg, errbuf, sizeof errbuf);
-		LOG("Failed to compile regexp '%s': %s\n", regex, errbuf);
+		NSLOG(netsurf, INFO, "Failed to compile regexp '%s': %s\n",
+		      regex, errbuf);
 		return NSERROR_INIT_FAILED;
 	}
 	return NSERROR_OK;
