@@ -23,6 +23,7 @@
 #include "utils/nsurl.h"
 #include "utils/messages.h"
 #include "utils/nsoption.h"
+#include "utils/utils.h"
 #include "netsurf/browser_window.h"
 #include "netsurf/keypress.h"
 #include "desktop/save_complete.h"
@@ -180,7 +181,7 @@ static void __CDECL menu_about(short item, short title, void *data)
 	  nserror error;
 	  char buf[PATH_MAX];
 
-	  LOG("%s", __FUNCTION__);
+	  NSLOG(netsurf, INFO, "abort menu");
 	  strcpy((char*)&buf, "file://");
 	  strncat((char*)&buf, (char*)"./doc/README.TXT",
 	  PATH_MAX - (strlen("file://")+1) );
@@ -207,7 +208,7 @@ static void __CDECL menu_new_win(short item, short title, void *data)
 	nserror error;
 	const char *addr;
 
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 
 	if (nsoption_charp(homepage_url) != NULL) {
 		addr = nsoption_charp(homepage_url);
@@ -235,7 +236,7 @@ static void __CDECL menu_open_url(short item, short title, void *data)
 {
 	struct gui_window * gw;
 	struct browser_window * bw ;
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 
 	gw = input_window;
 	if( gw == NULL ) {
@@ -250,7 +251,7 @@ static void __CDECL menu_open_url(short item, short title, void *data)
 static void __CDECL menu_open_file(short item, short title, void *data)
 {
 
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 
 	const char * filename = file_select(messages_get("OpenFile"), "");
 	if( filename != NULL ){
@@ -279,7 +280,7 @@ static void __CDECL menu_open_file(short item, short title, void *data)
 
 static void __CDECL menu_close_win(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 	gui_window_destroy( input_window );
@@ -287,7 +288,7 @@ static void __CDECL menu_close_win(short item, short title, void *data)
 
 static void __CDECL menu_save_page(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	static bool init = true;
 	bool is_folder=false;
 	const char * path;
@@ -318,7 +319,7 @@ static void __CDECL menu_quit(short item, short title, void *data)
 {
 	short buff[8];
 	memset( &buff, 0, sizeof(short)*8 );
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	gemtk_wm_send_msg(NULL, AP_TERM, 0, 0, 0, 0);
 }
 
@@ -330,21 +331,21 @@ static void __CDECL menu_cut(short item, short title, void *data)
 
 static void __CDECL menu_copy(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window != NULL )
 		browser_window_key_press( input_window->browser->bw, NS_KEY_COPY_SELECTION);
 }
 
 static void __CDECL menu_paste(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window != NULL )
 		browser_window_key_press( input_window->browser->bw, NS_KEY_PASTE);
 }
 
 static void __CDECL menu_find(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if (input_window != NULL) {
 		if (input_window->search) {
 			window_close_search(input_window->root);
@@ -357,13 +358,13 @@ static void __CDECL menu_find(short item, short title, void *data)
 
 static void __CDECL menu_choices(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	open_settings();
 }
 
 static void __CDECL menu_stop(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 
@@ -377,7 +378,7 @@ static void __CDECL menu_reload(short item, short title, void *data)
 	if(input_window == NULL)
 		return;
 	toolbar_reload_click(input_window->root->toolbar);
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 }
 
 
@@ -407,7 +408,7 @@ static void __CDECL menu_dec_scale(short item, short title, void *data)
 static void __CDECL menu_toolbars(short item, short title, void *data)
 {
 	static int state = 0;
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window != null && input_window->root->toolbar != null ){
 		state = !state;
 		// TODO: implement toolbar hide
@@ -417,7 +418,7 @@ static void __CDECL menu_toolbars(short item, short title, void *data)
 
 static void __CDECL menu_savewin(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if (input_window && input_window->browser) {
 		GRECT rect;
 		wind_get_grect(gemtk_wm_get_handle(input_window->root->win), WF_CURRXYWH,
@@ -437,7 +438,7 @@ static void __CDECL menu_savewin(short item, short title, void *data)
 
 static void __CDECL menu_debug_render(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	html_redraw_debug = !html_redraw_debug;
 	if( input_window != NULL ) {
 		if ( input_window->browser != NULL
@@ -468,7 +469,7 @@ static void __CDECL menu_bg_images(short item, short title, void *data)
 
 static void __CDECL menu_back(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 	toolbar_back_click(input_window->root->toolbar);
@@ -476,7 +477,7 @@ static void __CDECL menu_back(short item, short title, void *data)
 
 static void __CDECL menu_forward(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 	toolbar_forward_click(input_window->root->toolbar);
@@ -484,7 +485,7 @@ static void __CDECL menu_forward(short item, short title, void *data)
 
 static void __CDECL menu_home(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 	toolbar_home_click(input_window->root->toolbar);
@@ -492,24 +493,24 @@ static void __CDECL menu_home(short item, short title, void *data)
 
 static void __CDECL menu_lhistory(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if( input_window == NULL )
 		return;
 }
 
 static void __CDECL menu_ghistory(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	atari_global_history_open();
 }
 
 static void __CDECL menu_add_bookmark(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	if (input_window) {
 		if( browser_window_has_content(input_window->browser->bw) ){
 			atari_hotlist_add_page(
-				nsurl_access(browser_window_get_url(input_window->browser->bw)),
+				nsurl_access(browser_window_access_url(input_window->browser->bw)),
 				NULL
 				);
 		}
@@ -518,26 +519,26 @@ static void __CDECL menu_add_bookmark(short item, short title, void *data)
 
 static void __CDECL menu_bookmarks(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	atari_hotlist_open();
 }
 
 static void __CDECL menu_cookies(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	atari_cookie_manager_open();
 }
 
 static void __CDECL menu_vlog(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 	verbose_log = !verbose_log;
 	menu_icheck(h_gem_menu, MAINMENU_M_VLOG, (verbose_log) ? 1 : 0);
 }
 
 static void __CDECL menu_help_content(short item, short title, void *data)
 {
-	LOG("%s", __FUNCTION__);
+	NSLOG(netsurf, INFO, "%s", __FUNCTION__);
 }
 
 /*
@@ -579,14 +580,16 @@ static void register_menu_str( struct s_menu_item_evnt * mi )
 	while (i > 2) {
 		if ((strncmp("  ", &str[i], 2) == 0) && (strlen(&str[i]) > 2)) {
 			// "Standard" Keyboard Shortcut Element found:
-			LOG("Standard Keyboard Shortcut: \"%s\"\n", &str[i]);
+			NSLOG(netsurf, INFO,
+			      "Standard Keyboard Shortcut: \"%s\"\n", &str[i]);
 			x = i+2;
 			is_std_shortcut = true;
 			break;
 		}
 
 		if( str[i] == '['){
-			LOG("Keyboard Shortcut: \"%s\"\n", &str[i]);
+			NSLOG(netsurf, INFO, "Keyboard Shortcut: \"%s\"\n",
+			      &str[i]);
 			// "Custom" Keyboard Shortcut Element found (identified by [):
 			x = i;
 			break;
@@ -661,8 +664,12 @@ static void register_menu_str( struct s_menu_item_evnt * mi )
 			}
 		}
 
-		LOG("Registered keyboard shortcut for \"%s\" => mod: %d, ""keycode: %ld, ascii: %c\n",
-                    str, accel->mod, accel->keycode, accel->ascii);
+		NSLOG(netsurf, INFO,
+		      "Registered keyboard shortcut for \"%s\" => mod: %d, ""keycode: %ld, ascii: %c\n",
+		      str,
+		      accel->mod,
+		      accel->keycode,
+		      accel->ascii);
 	}
 }
 
