@@ -39,6 +39,7 @@
 #include "netsurf/content.h"
 #include "netsurf/cookie_db.h"
 #include "netsurf/url_db.h"
+#include "netsurf/plotters.h"
 #include "content/backing_store.h"
 
 #include "atari/gemtk/gemtk.h"
@@ -138,11 +139,11 @@ static void atari_poll(void)
     evnt_multi_fast(&aes_event_in, aes_msg_out, &aes_event_out);
     if(gemtk_wm_dispatch_event(&aes_event_in, &aes_event_out, aes_msg_out) == 0) {
 	if( (aes_event_out.emo_events & MU_MESAG) != 0 ) {
-	    LOG("WM: %d\n", aes_msg_out[0]);
+	    NSLOG(netsurf, INFO, "WM: %d\n", aes_msg_out[0]);
 	    switch(aes_msg_out[0]) {
 
 	    case MN_SELECTED:
-		LOG("Menu Item: %d\n", aes_msg_out[4]);
+		NSLOG(netsurf, INFO, "Menu Item: %d\n", aes_msg_out[4]);
 		deskmenu_dispatch_item(aes_msg_out[3], aes_msg_out[4]);
 		break;
 
@@ -194,13 +195,14 @@ gui_window_create(struct browser_window *bw,
 		  gui_window_create_flags flags)
 {
     struct gui_window *gw=NULL;
-    LOG("gw: %p, BW: %p, existing %p, flags: %d\n", gw, bw, existing, (int)flags);
+    NSLOG(netsurf, INFO, "gw: %p, BW: %p, existing %p, flags: %d\n", gw, bw,
+          existing, (int)flags);
 
     gw = calloc(1, sizeof(struct gui_window));
     if (gw == NULL)
 	return NULL;
 
-    LOG("new window: %p, bw: %p\n", gw, bw);
+    NSLOG(netsurf, INFO, "new window: %p, bw: %p\n", gw, bw);
     window_create(gw, bw, existing, WIDGET_STATUSBAR|WIDGET_TOOLBAR|WIDGET_RESIZE\
 		  |WIDGET_SCROLL);
     if (gw->root->win) {
@@ -252,7 +254,7 @@ void gui_window_destroy(struct gui_window *gw)
     if (gw == NULL)
 	return;
 
-    LOG("%s\n", __FUNCTION__);
+    NSLOG(netsurf, INFO, "%s\n", __FUNCTION__);
 
     if (input_window == gw) {
 	gui_set_input_gui_window(NULL);
@@ -292,25 +294,27 @@ void gui_window_destroy(struct gui_window *gw)
 }
 
 /**
- * Find the current dimensions of a browser window's content area.
+ * Find the current dimensions of a atari browser window content area.
  *
- * \param w	 gui_window to measure
- * \param width	 receives width of window
+ * \param gw The gui window to measure content area of.
+ * \param width receives width of window
  * \param height receives height of window
  * \param scaled whether to return scaled values
+ * \return NSERROR_OK on sucess and width and height updated
+ *          else error code.
  */
-static void
-gui_window_get_dimensions(struct gui_window *w,
+static nserror
+gui_window_get_dimensions(struct gui_window *gw,
 			  int *width,
 			  int *height,
 			  bool scaled)
 {
-    if (w == NULL)
-	return;
     GRECT rect;
-    window_get_grect(w->root, BROWSER_AREA_CONTENT, &rect);
+    window_get_grect(gw->root, BROWSER_AREA_CONTENT, &rect);
     *width = rect.g_w;
     *height = rect.g_h;
+
+    return NSERROR_OK;
 }
 
 /**
@@ -374,44 +378,41 @@ void atari_window_set_status(struct gui_window *w, const char *text)
 	window_set_stauts(w->root, (char*)text);
 }
 
-static void atari_window_reformat(struct gui_window *gw)
-{
-    int width = 0, height = 0;
 
-    if (gw != NULL) {
-	gui_window_get_dimensions(gw, &width, &height, true);
-	browser_window_reformat(gw->browser->bw, false, width, height);
-    }
-}
-
-static void gui_window_redraw_window(struct gui_window *gw)
-{
-    CMP_BROWSER b;
-    GRECT rect;
-    if (gw == NULL)
-	return;
-    b = gw->browser;
-    window_get_grect(gw->root, BROWSER_AREA_CONTENT, &rect);
-    window_schedule_redraw_grect(gw->root, &rect);
-}
-
-static void gui_window_update_box(struct gui_window *gw, const struct rect *rect)
+/**
+ * Invalidates an area of an atari browser window
+ *
+ * \param gw gui_window
+ * \param rect area to redraw or NULL for the entire window area
+ * \return NSERROR_OK on success or appropriate error code
+ */
+static nserror
+atari_window_invalidate_area(struct gui_window *gw,
+			     const struct rect *rect)
 {
     GRECT area;
-    struct gemtk_wm_scroll_info_s *slid;
 
-    if (gw == NULL)
-	return;
-
-    slid = gemtk_wm_get_scroll_info(gw->root->win);
+    if (gw == NULL) {
+	return NSERROR_BAD_PARAMETER;
+    }
 
     window_get_grect(gw->root, BROWSER_AREA_CONTENT, &area);
-    area.g_x += rect->x0 - (slid->x_pos * slid->x_unit_px);
-    area.g_y += rect->y0 - (slid->y_pos * slid->y_unit_px);
-    area.g_w = rect->x1 - rect->x0;
-    area.g_h = rect->y1 - rect->y0;
+
+    if (rect != NULL) {
+	    struct gemtk_wm_scroll_info_s *slid;
+
+	    slid = gemtk_wm_get_scroll_info(gw->root->win);
+
+	    area.g_x += rect->x0 - (slid->x_pos * slid->x_unit_px);
+	    area.g_y += rect->y0 - (slid->y_pos * slid->y_unit_px);
+	    area.g_w = rect->x1 - rect->x0;
+	    area.g_h = rect->y1 - rect->y0;
+    }
+
     //dbg_grect("update box", &area);
     window_schedule_redraw_grect(gw->root, &area);
+
+    return NSERROR_OK;
 }
 
 bool gui_window_get_scroll(struct gui_window *w, int *sx, int *sy)
@@ -424,17 +425,31 @@ bool gui_window_get_scroll(struct gui_window *w, int *sx, int *sy)
     return( true );
 }
 
-static void gui_window_set_scroll(struct gui_window *w, int sx, int sy)
+/**
+ * Set the scroll position of a atari browser window.
+ *
+ * Scrolls the viewport to ensure the specified rectangle of the
+ *   content is shown. The atari implementation scrolls the contents so
+ *   the specified point in the content is at the top of the viewport.
+ *
+ * \param gw gui window to scroll
+ * \param rect The rectangle to ensure is shown.
+ * \return NSERROR_OK on success or apropriate error code.
+ */
+static nserror
+gui_window_set_scroll(struct gui_window *gw, const struct rect *rect)
 {
-    if (   (w == NULL)
-	   || (w->browser->bw == NULL)
-	   || (!browser_window_has_content(w->browser->bw)))
-	return;
+    if ((gw == NULL) ||
+	(gw->browser->bw == NULL) ||
+	(!browser_window_has_content(gw->browser->bw))) {
+	return NSERROR_BAD_PARAMETER;
+    }
 
-    LOG("scroll (gui_window: %p) %d, %d\n", w, sx, sy);
-    window_scroll_by(w->root, sx, sy);
-    return;
+    NSLOG(netsurf, INFO, "scroll (gui_window: %p) %d, %d\n", gw, rect->x0,
+          rect->y0);
+    window_scroll_by(gw->root, rect->x0, rect->y0);
 
+    return NSERROR_OK;
 }
 
 /**
@@ -679,7 +694,7 @@ static void gui_window_new_content(struct gui_window *w)
     slid->x_pos = 0;
     slid->y_pos = 0;
     gemtk_wm_update_slider(w->root->win, GEMTK_WM_VH_SLIDER);
-    gui_window_redraw_window(w);
+    atari_window_invalidate_area(w, NULL);
 }
 
 
@@ -751,23 +766,33 @@ static void gui_set_clipboard(const char *buffer, size_t length,
     }
 }
 
-static void gui_401login_open(nsurl *url, const char *realm,
-			      nserror (*cb)(bool proceed, void *pw), void *cbpw)
+static nserror gui_401login_open(nsurl *url, const char *realm,
+        const char *username, const char *password,
+        nserror (*cb)(const char *username,
+                const char *password,
+                void *pw),
+        void *cbpw)
 {
-        bool bres;
-        char * out = NULL;
-        bres = login_form_do( url, (char*)realm, &out);
-        if (bres) {
-                LOG("url: %s, realm: %s, auth: %s\n", nsurl_access(url), realm, out);
-                urldb_set_auth_details(url, realm, out);
-        }
-        if (out != NULL) {
-                free( out );
-        }
-        if (cb != NULL) {
-                cb(bres, cbpw);
-        }
+    bool bres;
+    char * u_out = NULL;
+    char * p_out = NULL;
 
+    bres = login_form_do(url, (char*)realm, &u_out, &p_out);
+    if (bres) {
+        NSLOG(netsurf, INFO, "url: %s, realm: %s, auth: %s:%s\n",
+                nsurl_access(url), realm, u_out, p_out);
+    }
+    if (cb != NULL) {
+        cb(u_out, p_out, cbpw);
+    }
+    if (u_out != NULL) {
+        free(u_out);
+    }
+    if (p_out != NULL) {
+        free(p_out);
+    }
+
+    return NSERROR_OK;
 }
 
 static nserror
@@ -776,7 +801,7 @@ gui_cert_verify(nsurl *url, const struct ssl_cert_info *certs,
 		void *cbpw)
 {
         struct sslcert_session_data *data;
-        LOG("url %s", nsurl_access(url));
+        NSLOG(netsurf, INFO, "url %s", nsurl_access(url));
 
         // TODO: localize string
         int b = form_alert(1, "[2][SSL Verify failed, continue?][Continue|Abort|Details...]");
@@ -799,7 +824,8 @@ gui_cert_verify(nsurl *url, const struct ssl_cert_info *certs,
 
 void gui_set_input_gui_window(struct gui_window *gw)
 {
-    LOG("Setting input window from: %p to %p\n", input_window, gw);
+    NSLOG(netsurf, INFO, "Setting input window from: %p to %p\n",
+          input_window, gw);
     input_window = gw;
 }
 
@@ -810,7 +836,7 @@ struct gui_window * gui_get_input_window(void)
 
 static void gui_quit(void)
 {
-    LOG("quitting");
+    NSLOG(netsurf, INFO, "quitting");
 
     struct gui_window *gw = window_list;
     struct gui_window *tmp = window_list;
@@ -839,9 +865,9 @@ static void gui_quit(void)
 
     rsrc_free();
 
-    LOG("Shutting down plotter");
+    NSLOG(netsurf, INFO, "Shutting down plotter");
     plot_finalise();
-    LOG("done");
+    NSLOG(netsurf, INFO, "done");
 }
 
 /**
@@ -853,7 +879,7 @@ process_cmdline(int argc, char** argv)
     int opt;
     bool set_default_dimensions = true;
 
-    LOG("argc %d, argv %p", argc, argv);
+    NSLOG(netsurf, INFO, "argc %d, argv %p", argc, argv);
 
     if ((nsoption_int(window_width) != 0) && (nsoption_int(window_height) != 0)) {
 
@@ -946,7 +972,7 @@ static nserror set_defaults(struct nsoption_s *defaults)
     /* Set defaults for absent option strings */
     nsoption_setnull_charp(cookie_file, strdup("cookies"));
     if (nsoption_charp(cookie_file) == NULL) {
-	LOG("Failed initialising string options");
+	NSLOG(netsurf, INFO, "Failed initialising string options");
 	return NSERROR_BAD_PARAMETER;
     }
     return NSERROR_OK;
@@ -963,7 +989,7 @@ static void gui_init(int argc, char** argv)
     OBJECT * cursors;
 
     atari_find_resource(buf, "netsurf.rsc", "./res/netsurf.rsc");
-    LOG("Using RSC file: %s ", (char *)&buf);
+    NSLOG(netsurf, INFO, "Using RSC file: %s ", (char *)&buf);
     if (rsrc_load(buf)==0) {
 
 	char msg[1024];
@@ -999,15 +1025,16 @@ static void gui_init(int argc, char** argv)
     create_cursor(MFORM_EX_FLAG_USERFORM, CURSOR_HELP,
 		  cursors, &gem_cursors.help);
 
-    LOG("Enabling core select menu");
+    NSLOG(netsurf, INFO, "Enabling core select menu");
     nsoption_set_bool(core_select_menu, true);
 
-    LOG("Loading url.db from: %s", nsoption_charp(url_file));
+    NSLOG(netsurf, INFO, "Loading url.db from: %s", nsoption_charp(url_file));
     if( strlen(nsoption_charp(url_file)) ) {
 	urldb_load(nsoption_charp(url_file));
     }
 
-    LOG("Loading cookies from: %s", nsoption_charp(cookie_file));
+    NSLOG(netsurf, INFO, "Loading cookies from: %s",
+          nsoption_charp(cookie_file));
     if( strlen(nsoption_charp(cookie_file)) ) {
 	urldb_load_cookies(nsoption_charp(cookie_file));
     }
@@ -1015,11 +1042,16 @@ static void gui_init(int argc, char** argv)
     if (process_cmdline(argc,argv) != true)
 	die("unable to process command line.\n");
 
-    LOG("Initializing NKC...");
+    NSLOG(netsurf, INFO, "Initializing NKC...");
     nkc_init();
 
-    LOG("Initializing plotters...");
-    plot_init(nsoption_charp(atari_font_driver));
+    NSLOG(netsurf, INFO, "Initializing plotters...");
+    struct redraw_context ctx = {
+	    .interactive = true,
+	    .background_images = true,
+	    .plot = &atari_plotters
+    };
+    plot_init(&ctx, nsoption_charp(atari_font_driver));
 
     aes_event_in.emi_m1leave = MO_LEAVE;
     aes_event_in.emi_m1.g_w = 1;
@@ -1045,13 +1077,11 @@ static void gui_init(int argc, char** argv)
 static struct gui_window_table atari_window_table = {
     .create = gui_window_create,
     .destroy = gui_window_destroy,
-    .redraw = gui_window_redraw_window,
-    .update = gui_window_update_box,
+    .invalidate = atari_window_invalidate_area,
     .get_scroll = gui_window_get_scroll,
     .set_scroll = gui_window_set_scroll,
     .get_dimensions = gui_window_get_dimensions,
     .update_extent = gui_window_update_extent,
-    .reformat = atari_window_reformat,
 
     .set_title = gui_window_set_title,
     .set_url = gui_window_set_url,
@@ -1156,18 +1186,18 @@ int main(int argc, char** argv)
     ret = messages_add_from_file(messages);
 
     /* common initialisation */
-    LOG("Initialising core...");
+    NSLOG(netsurf, INFO, "Initialising core...");
     ret = netsurf_init(store);
     if (ret != NSERROR_OK) {
 	die("NetSurf failed to initialise");
     }
 
-    LOG("Initializing GUI...");
+    NSLOG(netsurf, INFO, "Initializing GUI...");
     gui_init(argc, argv);
 
     graf_mouse( ARROW , NULL);
 
-    LOG("Creating initial browser window...");
+    NSLOG(netsurf, INFO, "Creating initial browser window...");
     addr = option_homepage_url;
     if (strncmp(addr, "file://", 7) && strncmp(addr, "http://", 7)) {
 	if (stat(addr, &stat_buf) == 0) {
@@ -1189,7 +1219,7 @@ int main(int argc, char** argv)
     if (ret != NSERROR_OK) {
 	atari_warn_user(messages_get_errorcode(ret), 0);
     } else {
-	LOG("Entering Atari event mainloop...");
+	NSLOG(netsurf, INFO, "Entering Atari event mainloop...");
 	while (!atari_quit) {
 	    atari_poll();
 	}
@@ -1203,7 +1233,11 @@ int main(int argc, char** argv)
     fclose(stdout);
     fclose(stderr);
 #endif
-    LOG("exit_gem");
+    NSLOG(netsurf, INFO, "exit_gem");
+
+    /* finalise logging */
+    nslog_finalise();
+
     exit_gem();
 
     return 0;
