@@ -42,6 +42,9 @@
 #include <Roster.h>
 #include <Screen.h>
 #include <String.h>
+#ifdef __HAIKU__
+#include <LocaleRoster.h>
+#endif
 
 extern "C" {
 
@@ -112,7 +115,7 @@ static int sEventPipe[2];
 /* exported function defined in beos/gui.h */
 nserror beos_warn_user(const char *warning, const char *detail)
 {
-	LOG("warn_user: %s (%s)", warning, detail);
+	NSLOG(netsurf, INFO, "warn_user: %s (%s)", warning, detail);
 	BAlert *alert;
 	BString text(warning);
 	if (detail)
@@ -354,14 +357,15 @@ static void check_homedir(void)
 
 	if (err < B_OK) {
 		/* we really can't continue without a home directory. */
-		LOG("Can't find user settings directory - nowhere to store state!");
+		NSLOG(netsurf, INFO,
+		      "Can't find user settings directory - nowhere to store state!");
 		die("NetSurf needs to find the user settings directory in order to run.\n");
 	}
 
 	path.Append("NetSurf");
 	err = create_directory(path.Path(), 0644); 
 	if (err < B_OK) {
-		LOG("Unable to create %s", path.Path());
+		NSLOG(netsurf, INFO, "Unable to create %s", path.Path());
 		die("NetSurf could not create its settings directory.\n");
 	}
 }
@@ -387,7 +391,7 @@ static nsurl *gui_get_resource_url(const char *path)
 		path = "favicon.png";
 
 	u << path;
-	LOG("(%s) -> '%s'\n", path, u.String());
+	NSLOG(netsurf, INFO, "(%s) -> '%s'\n", path, u.String());
 	nsurl_create(u.String(), &url);
 	return url;
 }
@@ -518,9 +522,18 @@ static BPath get_messages_path()
 
 	BPath p;
 	f.FindPath(B_FIND_PATH_APPS_DIRECTORY, "netsurf/res", p);
-	// TODO: use Haiku's BLocale stuff
-	BString lang(getenv("LC_MESSAGES"));
-	lang.Truncate(2);
+	BString lang;
+#ifdef __HAIKU__
+	BMessage preferredLangs;
+	if (BLocaleRoster::Default()->GetPreferredLanguages(&preferredLangs) == B_OK) {
+		preferredLangs.FindString("language", 0, &lang);
+		lang.Truncate(2);
+	}
+#endif
+	if (lang.Length() < 1) {
+		lang.SetTo(getenv("LC_MESSAGES"));
+		lang.Truncate(2);
+	}
 	BDirectory d(p.Path());
 	if (!d.Contains(lang.String(), B_DIRECTORY_NODE))
 		lang = "en";
@@ -588,7 +601,7 @@ static void gui_init(int argc, char** argv)
 		die("Unable to load throbber image.\n");
 
 	find_resource(buf, "Choices", "%/Choices");
-	LOG("Using '%s' as Preferences file", buf);
+	NSLOG(netsurf, INFO, "Using '%s' as Preferences file", buf);
 	options_file_location = strdup(buf);
 	nsoption_read(buf, NULL);
 
@@ -631,12 +644,12 @@ static void gui_init(int argc, char** argv)
 
 	if (nsoption_charp(cookie_file) == NULL) {
 		find_resource(buf, "Cookies", "%/Cookies");
-		LOG("Using '%s' as Cookies file", buf);
+		NSLOG(netsurf, INFO, "Using '%s' as Cookies file", buf);
 		nsoption_set_charp(cookie_file, strdup(buf));
 	}
 	if (nsoption_charp(cookie_jar) == NULL) {
 		find_resource(buf, "Cookies", "%/Cookies");
-		LOG("Using '%s' as Cookie Jar file", buf);
+		NSLOG(netsurf, INFO, "Using '%s' as Cookie Jar file", buf);
 		nsoption_set_charp(cookie_jar, strdup(buf));
 	}
 	if ((nsoption_charp(cookie_file) == NULL) || 
@@ -645,13 +658,13 @@ static void gui_init(int argc, char** argv)
 
 	if (nsoption_charp(url_file) == NULL) {
 		find_resource(buf, "URLs", "%/URLs");
-		LOG("Using '%s' as URL file", buf);
+		NSLOG(netsurf, INFO, "Using '%s' as URL file", buf);
 		nsoption_set_charp(url_file, strdup(buf));
 	}
 
         if (nsoption_charp(ca_path) == NULL) {
                 find_resource(buf, "certs", "/etc/ssl/certs");
-                LOG("Using '%s' as certificate path", buf);
+                NSLOG(netsurf, INFO, "Using '%s' as certificate path", buf);
                 nsoption_set_charp(ca_path, strdup(buf));
         }
 
@@ -763,17 +776,21 @@ void nsbeos_gui_poll(void)
 	timeout.tv_sec = (long)(next_schedule / 1000000LL);
 	timeout.tv_usec = (long)(next_schedule % 1000000LL);
 
-	//LOG("gui_poll: select(%d, ..., %Ldus", max_fd, next_schedule);
+	NSLOG(netsurf, DEEPDEBUG,
+	      "gui_poll: select(%d, ..., %Ldus",
+	      max_fd, next_schedule);
 	fd_count = select(max_fd, &read_fd_set, &write_fd_set, &exc_fd_set, 
 		&timeout);
-	//LOG("select: %d\n", fd_count);
+	NSLOG(netsurf, DEEPDEBUG, "select: %d\n", fd_count);
 
 	if (fd_count > 0 && FD_ISSET(sEventPipe[0], &read_fd_set)) {
 		BMessage *message;
 		int len = read(sEventPipe[0], &message, sizeof(void *));
-		//LOG("gui_poll: BMessage ? %d read", len);
+		NSLOG(netsurf, DEEPDEBUG, "gui_poll: BMessage ? %d read", len);
 		if (len == sizeof(void *)) {
-			//LOG("gui_poll: BMessage.what %-4.4s\n", &(message->what));
+			NSLOG(netsurf, DEEPDEBUG,
+			      "gui_poll: BMessage.what %-4.4s\n",
+			      (char *)&(message->what));
 			nsbeos_dispatch_event(message);
 		}
 	}
@@ -817,7 +834,9 @@ void nsbeos_gui_view_source(struct hlcache_handle *content)
 	BPath path;
 	status_t err;
 	size_t size;
-	const char *source = content_get_source_data(content, &size);
+	const uint8_t *source;
+
+        source = content_get_source_data(content, &size);
 
 	if (!content || !source) {
 		beos_warn_user("MiscError", "No document source");
@@ -840,13 +859,28 @@ void nsbeos_gui_view_source(struct hlcache_handle *content)
 		 * allow it to be re-used next time NetSurf is started. The
 		 * memory overhead from doing this is under 1 byte per
 		 * filename. */
-		const char *filename = filename_request();
-		if (!filename) {
+		BString filename(filename_request());
+		if (filename.IsEmpty()) {
 			beos_warn_user("NoMemory", 0);
 			return;
 		}
+
+		lwc_string *mime = content_get_mime_type(content);
+
+		/* provide an extension, as Pe still doesn't sniff the MIME */
+		if (mime) {
+			BMimeType type(lwc_string_data(mime));
+			BMessage extensions;
+			if (type.GetFileExtensions(&extensions) == B_OK) {
+				BString ext;
+				if (extensions.FindString("extensions", &ext) == B_OK)
+					filename << "." << ext;
+			}
+			/* we unref(mime) later on, we just leak on error */
+		}
+
 		path.SetTo(TEMP_FILENAME_PREFIX);
-		path.Append(filename);
+		path.Append(filename.String());
 		BFile file(path.Path(), B_WRITE_ONLY | B_CREATE_FILE);
 		err = file.InitCheck();
 		if (err < B_OK) {
@@ -858,7 +892,7 @@ void nsbeos_gui_view_source(struct hlcache_handle *content)
 			beos_warn_user("IOError", strerror(err));
 			return;
 		}
-		lwc_string *mime = content_get_mime_type(content);
+
 		if (mime) {
 			file.WriteAttr("BEOS:TYPE", B_MIME_STRING_TYPE, 0LL, 
 				lwc_string_data(mime), lwc_string_length(mime) + 1);
@@ -877,6 +911,7 @@ void nsbeos_gui_view_source(struct hlcache_handle *content)
 
 	// apps to try
 	const char *editorSigs[] = {
+		"text/x-source-code",
 		"application/x-vnd.beunited.pe",
 		"application/x-vnd.XEmacs",
 		"application/x-vnd.Haiku-StyledEdit",
@@ -891,10 +926,11 @@ void nsbeos_gui_view_source(struct hlcache_handle *content)
 			BMessenger msgr(editorSigs[i], team);
 			if (msgr.SendMessage(&m) >= B_OK)
 				break;
+
 		}
 		
 		err = be_roster->Launch(editorSigs[i], (BMessage *)&m, &team);
-		if (err >= B_OK)
+		if (err >= B_OK || err == B_ALREADY_RUNNING)
 			break;
 	}
 }
@@ -1018,10 +1054,20 @@ int main(int argc, char** argv)
 	BResources resources;
 	resources.SetToImage((const void*)main);
 	size_t size = 0;
-	
+
+	BString lang;
+#ifdef __HAIKU__
+	BMessage preferredLangs;
+	if (BLocaleRoster::Default()->GetPreferredLanguages(&preferredLangs) == B_OK) {
+		preferredLangs.FindString("language", 0, &lang);
+	}
+#endif
+	if (lang.Length() < 1)
+		lang.SetTo(getenv("LC_MESSAGES"));
+
 	char path[12];
-	sprintf(path,"%.2s/Messages", getenv("LC_MESSAGES"));
-	fprintf(stderr, "Loading messages from resource %s\n", path);
+	sprintf(path,"%.2s/Messages", lang.String());
+	NSLOG(netsurf, INFO, "Loading messages from resource %s\n", path);
 
 	const uint8_t* res = (const uint8_t*)resources.LoadResource('data', path, &size);
 	if (size > 0 && res != NULL) {
@@ -1044,6 +1090,12 @@ int main(int argc, char** argv)
 
 	netsurf_exit();
 
+	/* finalise options */
+	nsoption_finalise(nsoptions, nsoptions_default);
+
+	/* finalise logging */
+	nslog_finalise();
+
 	return 0;
 }
 
