@@ -54,6 +54,7 @@
 #include "framebuffer/clipboard.h"
 #include "framebuffer/fetch.h"
 #include "framebuffer/bitmap.h"
+#include "framebuffer/local_history.h"
 
 
 #define NSFB_TOOLBAR_DEFAULT_LAYOUT "blfsrutc"
@@ -119,7 +120,7 @@ static void die(const char *error)
  */
 static nserror fb_warn_user(const char *warning, const char *detail)
 {
-	LOG("%s %s", warning, detail);
+	NSLOG(netsurf, INFO, "%s %s", warning, detail);
 	return NSERROR_OK;
 }
 
@@ -152,7 +153,7 @@ widget_scroll_y(struct gui_window *gw, int y, bool abs)
 	int content_width, content_height;
 	int height;
 
-	LOG("window scroll");
+	NSLOG(netsurf, INFO, "window scroll");
 	if (abs) {
 		bwidget->pany = y - bwidget->scrolly;
 	} else {
@@ -236,7 +237,7 @@ fb_pan(fbtk_widget_t *widget,
 	height = fbtk_get_height(widget);
 	width = fbtk_get_width(widget);
 
-	LOG("panning %d, %d", bwidget->panx, bwidget->pany);
+	NSLOG(netsurf, INFO, "panning %d, %d", bwidget->panx, bwidget->pany);
 
 	x = fbtk_get_absx(widget);
 	y = fbtk_get_absy(widget);
@@ -412,7 +413,8 @@ fb_browser_window_redraw(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 
 	bwidget = fbtk_get_userpw(widget);
 	if (bwidget == NULL) {
-		LOG("browser widget from widget %p was null", widget);
+		NSLOG(netsurf, INFO,
+		      "browser widget from widget %p was null", widget);
 		return -1;
 	}
 
@@ -464,7 +466,7 @@ process_cmdline(int argc, char** argv)
 		{0, 0, 0,  0 }
 	}; /* no long options */
 
-	LOG("argc %d, argv %p", argc, argv);
+	NSLOG(netsurf, INFO, "argc %d, argv %p", argc, argv);
 
 	fename = "sdl";
 	febpp = 32;
@@ -533,7 +535,7 @@ static nserror set_defaults(struct nsoption_s *defaults)
 
 	if (nsoption_charp(cookie_file) == NULL ||
 	    nsoption_charp(cookie_jar) == NULL) {
-		LOG("Failed initialising cookie options");
+		NSLOG(netsurf, INFO, "Failed initialising cookie options");
 		return NSERROR_BAD_PARAMETER;
 	}
 
@@ -611,7 +613,7 @@ static void framebuffer_run(void)
 
 static void gui_quit(void)
 {
-	LOG("gui_quit");
+	NSLOG(netsurf, INFO, "gui_quit");
 
 	urldb_save_cookies(nsoption_charp(cookie_jar));
 
@@ -638,7 +640,8 @@ fb_browser_window_click(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 	    cbi->event->type != NSFB_EVENT_KEY_UP)
 		return 0;
 
-	LOG("browser window clicked at %d,%d", cbi->x, cbi->y);
+	NSLOG(netsurf, INFO, "browser window clicked at %d,%d", cbi->x,
+	      cbi->y);
 
 	switch (cbi->event->type) {
 	case NSFB_EVENT_KEY_DOWN:
@@ -823,7 +826,7 @@ fb_browser_window_input(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 	static fbtk_modifier_type modifier = FBTK_MOD_CLEAR;
 	int ucs4 = -1;
 
-	LOG("got value %d", cbi->event->value.keycode);
+	NSLOG(netsurf, INFO, "got value %d", cbi->event->value.keycode);
 
 	switch (cbi->event->type) {
 	case NSFB_EVENT_KEY_DOWN:
@@ -948,8 +951,8 @@ fb_browser_window_input(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 				browser_window_key_press(gw->bw, NS_KEY_REDO);
 				break;
 			}
-			/* Z or Y pressed but not undo or redo;
-			 * Fall through to default handling */
+			/* Z or Y pressed but not undo or redo; */
+			/* Fall through */
 
 		default:
 			ucs4 = fbtk_keycode_to_ucs4(cbi->event->value.keycode,
@@ -1150,7 +1153,7 @@ fb_localhistory_btn_clik(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 	if (cbi->event->type != NSFB_EVENT_KEY_UP)
 		return 0;
 
-	fb_localhistory_map(gw->localhistory);
+	fb_local_history_present(fbtk, gw->bw);
 
 	return 0;
 }
@@ -1199,7 +1202,7 @@ create_toolbar(struct gui_window *gw,
 		toolbar_layout = NSFB_TOOLBAR_DEFAULT_LAYOUT;
 	}
 
-	LOG("Using toolbar layout %s", toolbar_layout);
+	NSLOG(netsurf, INFO, "Using toolbar layout %s", toolbar_layout);
 
 	itmtype = toolbar_layout;
 
@@ -1233,7 +1236,7 @@ create_toolbar(struct gui_window *gw,
 	       (*itmtype != 0) && 
 	       (xdir !=0)) {
 
-		LOG("toolbar adding %c", *itmtype);
+		NSLOG(netsurf, INFO, "toolbar adding %c", *itmtype);
 
 
 		switch (*itmtype) {
@@ -1375,7 +1378,9 @@ create_toolbar(struct gui_window *gw,
 		default:
 			widget = NULL;
 			xdir = 0;
-			LOG("Unknown element %c in toolbar layout", *itmtype);
+			NSLOG(netsurf, INFO,
+			      "Unknown element %c in toolbar layout",
+			      *itmtype);
 		        break;
 
 		}
@@ -1384,7 +1389,7 @@ create_toolbar(struct gui_window *gw,
 			xpos += (xdir * (fbtk_get_width(widget) + padding));
 		}
 
-		LOG("xpos is %d", xpos);
+		NSLOG(netsurf, INFO, "xpos is %d", xpos);
 
 		itmtype += xdir;
 	}
@@ -1583,7 +1588,7 @@ resize_browser_widget(struct gui_window *gw, int x, int y,
 		int width, int height)
 {
 	fbtk_set_pos_and_size(gw->browser, x, y, width, height);
-	browser_window_reformat(gw->bw, false, width, height);
+	browser_window_schedule_reformat(gw->bw);
 }
 
 static void
@@ -1594,7 +1599,7 @@ create_normal_browser_window(struct gui_window *gw, int furniture_width)
 	int statusbar_width = 0;
 	int toolbar_height = nsoption_int(fb_toolbar_size);
 
-	LOG("Normal window");
+	NSLOG(netsurf, INFO, "Normal window");
 
 	gw->window = fbtk_create_window(fbtk, 0, 0, 0, 0, 0);
 
@@ -1625,7 +1630,8 @@ create_normal_browser_window(struct gui_window *gw, int furniture_width)
 				      false);
 	fbtk_set_handler(gw->status, FBTK_CBT_POINTERENTER, set_ptr_default_move, NULL);
 
-	LOG("status bar %p at %d,%d", gw->status, fbtk_get_absx(gw->status), fbtk_get_absy(gw->status));
+	NSLOG(netsurf, INFO, "status bar %p at %d,%d", gw->status,
+	      fbtk_get_absx(gw->status), fbtk_get_absy(gw->status));
 
 	/* create horizontal scrollbar */
 	gw->hscroll = fbtk_create_hscroll(gw->window,
@@ -1782,7 +1788,6 @@ gui_window_create(struct browser_window *bw,
 	gw->bw = bw;
 
 	create_normal_browser_window(gw, nsoption_int(fb_furniture_size));
-	gw->localhistory = fb_create_localhistory(bw, fbtk, nsoption_int(fb_furniture_size));
 
 	/* map and request redraw of gui window */
 	fbtk_set_mapping(gw->window, true);
@@ -1803,21 +1808,33 @@ gui_window_destroy(struct gui_window *gw)
 	free(gw);
 }
 
-static void
-gui_window_redraw_window(struct gui_window *g)
-{
-	fb_queue_redraw(g->browser, 0, 0, fbtk_get_width(g->browser), fbtk_get_height(g->browser) );
-}
 
-static void
-gui_window_update_box(struct gui_window *g, const struct rect *rect)
+/**
+ * Invalidates an area of a framebuffer browser window
+ *
+ * \param g The netsurf window being invalidated.
+ * \param rect area to redraw or NULL for the entire window area
+ * \return NSERROR_OK on success or appropriate error code
+ */
+static nserror
+fb_window_invalidate_area(struct gui_window *g, const struct rect *rect)
 {
 	struct browser_widget_s *bwidget = fbtk_get_userpw(g->browser);
-	fb_queue_redraw(g->browser,
-			rect->x0 - bwidget->scrollx,
-			rect->y0 - bwidget->scrolly,
-			rect->x1 - bwidget->scrollx,
-			rect->y1 - bwidget->scrolly);
+
+	if (rect != NULL) {
+		fb_queue_redraw(g->browser,
+				rect->x0 - bwidget->scrollx,
+				rect->y0 - bwidget->scrolly,
+				rect->x1 - bwidget->scrollx,
+				rect->y1 - bwidget->scrolly);
+	} else {
+		fb_queue_redraw(g->browser,
+				0,
+				0,
+				fbtk_get_width(g->browser),
+				fbtk_get_height(g->browser));
+	}
+	return NSERROR_OK;
 }
 
 static bool
@@ -1832,34 +1849,56 @@ gui_window_get_scroll(struct gui_window *g, int *sx, int *sy)
 	return true;
 }
 
-static void
-gui_window_set_scroll(struct gui_window *gw, int sx, int sy)
+/**
+ * Set the scroll position of a framebuffer browser window.
+ *
+ * Scrolls the viewport to ensure the specified rectangle of the
+ *   content is shown. The framebuffer implementation scrolls the contents so
+ *   the specified point in the content is at the top of the viewport.
+ *
+ * \param gw gui_window to scroll
+ * \param rect The rectangle to ensure is shown.
+ * \return NSERROR_OK on success or apropriate error code.
+ */
+static nserror
+gui_window_set_scroll(struct gui_window *gw, const struct rect *rect)
 {
 	struct browser_widget_s *bwidget = fbtk_get_userpw(gw->browser);
 	float scale = browser_window_get_scale(gw->bw);
 
 	assert(bwidget);
 
-	widget_scroll_x(gw, sx * scale, true);
-	widget_scroll_y(gw, sy * scale, true);
+	widget_scroll_x(gw, rect->x0 * scale, true);
+	widget_scroll_y(gw, rect->y0 * scale, true);
+
+	return NSERROR_OK;
 }
 
 
-static void
-gui_window_get_dimensions(struct gui_window *g,
+/**
+ * Find the current dimensions of a framebuffer browser window content area.
+ *
+ * \param gw The gui window to measure content area of.
+ * \param width receives width of window
+ * \param height receives height of window
+ * \param scaled whether to return scaled values
+ * \return NSERROR_OK on sucess and width and height updated.
+ */
+static nserror
+gui_window_get_dimensions(struct gui_window *gw,
 			  int *width,
 			  int *height,
 			  bool scaled)
 {
-	float scale = browser_window_get_scale(g->bw);
-
-	*width = fbtk_get_width(g->browser);
-	*height = fbtk_get_height(g->browser);
+	*width = fbtk_get_width(gw->browser);
+	*height = fbtk_get_height(gw->browser);
 
 	if (scaled) {
+		float scale = browser_window_get_scale(gw->bw);
 		*width /= scale;
 		*height /= scale;
 	}
+	return NSERROR_OK;
 }
 
 static void
@@ -2038,26 +2077,15 @@ gui_window_remove_caret(struct gui_window *g)
 	}
 }
 
-static void framebuffer_window_reformat(struct gui_window *gw)
-{
-	/** @todo if we ever do zooming reformat should be implemented */
-	LOG("window:%p", gw);
-
-	/*
-	  browser_window_reformat(gw->bw, false, width, height);
-	*/
-}
 
 static struct gui_window_table framebuffer_window_table = {
 	.create = gui_window_create,
 	.destroy = gui_window_destroy,
-	.redraw = gui_window_redraw_window,
-	.update = gui_window_update_box,
+	.invalidate = fb_window_invalidate_area,
 	.get_scroll = gui_window_get_scroll,
 	.set_scroll = gui_window_set_scroll,
 	.get_dimensions = gui_window_get_dimensions,
 	.update_extent = gui_window_update_extent,
-	.reformat = framebuffer_window_reformat,
 
 	.set_url = gui_window_set_url,
 	.set_status = gui_window_set_status,
@@ -2076,7 +2104,8 @@ static struct gui_misc_table framebuffer_misc_table = {
 	.quit = gui_quit,
 };
 
-/** Entry point from OS.
+/**
+ * Entry point from OS.
  *
  * /param argc The number of arguments in the string vector.
  * /param argv The argument string vector.
@@ -2106,7 +2135,7 @@ main(int argc, char** argv)
 		die("NetSurf operation table failed registration");
         }
 
-	respaths = fb_init_resource(NETSURF_FB_RESPATH":"NETSURF_FB_FONTPATH);
+	respaths = fb_init_resource_path(NETSURF_FB_RESPATH":"NETSURF_FB_FONTPATH);
 
 	/* initialise logging. Not fatal if it fails but not much we
 	 * can do about it either.
@@ -2160,7 +2189,7 @@ main(int argc, char** argv)
 
 	/* create an initial browser window */
 
-	LOG("calling browser_window_create");
+	NSLOG(netsurf, INFO, "calling browser_window_create");
 
 	ret = nsurl_create(feurl, &url);
 	if (ret == NSERROR_OK) {
@@ -2182,11 +2211,14 @@ main(int argc, char** argv)
 	netsurf_exit();
 
 	if (fb_font_finalise() == false)
-		LOG("Font finalisation failed.");
+		NSLOG(netsurf, INFO, "Font finalisation failed.");
 
 	/* finalise options */
 	nsoption_finalise(nsoptions, nsoptions_default);
 
+	/* finalise logging */
+	nslog_finalise();
+
 	return 0;
 }
 
