@@ -25,7 +25,8 @@
 
 #include <rufl.h>
 
-struct gui_layout_table *riscos_layout_table;
+struct plot_font_style;
+extern struct gui_layout_table *riscos_layout_table;
 
 /** desktop font, size and style being used */
 extern char ro_gui_desktop_font_family[];
@@ -35,9 +36,9 @@ extern rufl_style ro_gui_desktop_font_style;
 void nsfont_init(void);
 bool nsfont_exists(const char *font_family);
 const char *nsfont_fallback_font(void);
-bool nsfont_paint(const plot_font_style_t *fstyle, const char *string,
+bool nsfont_paint(const struct plot_font_style *fstyle, const char *string,
 		size_t length, int x, int y);
-void nsfont_read_style(const plot_font_style_t *fstyle,
+void nsfont_read_style(const struct plot_font_style *fstyle,
 		const char **font_family, unsigned int *font_size,
 		rufl_style *font_style);
 void ro_gui_wimp_get_desktop_font(void);
