@@ -16,8 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Window themes (implementation).
+/**
+ * \file
+ * Window themes implementation.
  */
 
 #include <alloca.h>
@@ -47,7 +48,7 @@
 #include "riscos/hotlist.h"
 #include "riscos/menus.h"
 #include "riscos/theme.h"
-#include "riscos/treeview.h"
+#include "riscos/toolbar.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
 #include "riscos/wimputils.h"
@@ -180,8 +181,10 @@ static void ro_gui_theme_get_available_in_dir(const char *directory)
 				(osgbpb_info_list *) &info, 1, context,
 				sizeof(info), 0, &read_count, &context);
 		if (error) {
-			LOG("xosgbpb_dir_entries_info: 0x%x: %s",
-				error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "xosgbpb_dir_entries_info: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			if (error->errnum == 0xd6)	/* no such dir */
 				return;
 			ro_warn_user("MiscError", error->errmess);
@@ -364,7 +367,7 @@ bool ro_gui_theme_add_descriptor(const char *folder, const char *leafname)
 	/* create a full filename */
 	filename = malloc(strlen(folder) + strlen(leafname) + 2);
 	if (!filename) {
-	  	LOG("No memory for malloc");
+	  	NSLOG(netsurf, INFO, "No memory for malloc");
 	  	ro_warn_user("NoMemory", 0);
 	  	return false;
 	}
@@ -374,7 +377,8 @@ bool ro_gui_theme_add_descriptor(const char *folder, const char *leafname)
 	error = xosfind_openinw(osfind_NO_PATH, filename, 0,
 			&file_handle);
 	if (error) {
-		LOG("xosfind_openinw: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "xosfind_openinw: 0x%x: %s",
+		      error->errnum, error->errmess);
 		ro_warn_user("FileError", error->errmess);
 		free(filename);
 		return false;
@@ -389,7 +393,8 @@ bool ro_gui_theme_add_descriptor(const char *folder, const char *leafname)
 			0, &output_left);
 	xosfind_closew(file_handle);
 	if (error) {
-		LOG("xosbgpb_read_atw: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "xosbgpb_read_atw: 0x%x: %s",
+		      error->errnum, error->errmess);
 		ro_warn_user("FileError", error->errmess);
 		free(filename);
 		return false;
@@ -403,7 +408,7 @@ bool ro_gui_theme_add_descriptor(const char *folder, const char *leafname)
 	current = (struct theme_descriptor *)calloc(1,
 			sizeof(struct theme_descriptor));
 	if (!current) {
-		LOG("calloc failed");
+		NSLOG(netsurf, INFO, "calloc failed");
 		ro_warn_user("NoMemory", 0);
 		free(filename);
 		return false;
@@ -521,7 +526,7 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 		descriptor->theme = (struct theme *)calloc(1,
 				sizeof(struct theme));
 		if (!descriptor->theme) {
-			LOG("calloc() failed");
+			NSLOG(netsurf, INFO, "calloc() failed");
 			ro_warn_user("NoMemory", 0);
 			continue;
 		}
@@ -531,7 +536,10 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 		error = xosfile_read_stamped_no_path(descriptor->filename,
 				&obj_type, 0, 0, &file_size, 0, 0);
 		if (error) {
-			LOG("xosfile_read_stamped_no_path: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "xosfile_read_stamped_no_path: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			ro_warn_user("FileError", error->errmess);
 			continue;
 		}
@@ -539,7 +547,7 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 			continue;
 		raw_data = malloc(file_size);
 		if (!raw_data) {
-			LOG("malloc() failed");
+			NSLOG(netsurf, INFO, "malloc() failed");
 			ro_warn_user("NoMemory", 0);
 			continue;
 		}
@@ -547,7 +555,10 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 				(byte *)raw_data, 0, 0, 0, 0, 0);
 		if (error) {
 			free(raw_data);
-			LOG("xosfile_load_stamped_no_path: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "xosfile_load_stamped_no_path: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			ro_warn_user("FileError", error->errmess);
 			continue;
 		}
@@ -556,7 +567,10 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 		error = xsquash_decompress_return_sizes(-1, &workspace_size, 0);
 		if (error) {
 			free(raw_data);
-			LOG("xsquash_decompress_return_sizes: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "xsquash_decompress_return_sizes: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			ro_warn_user("MiscError", error->errmess);
 			continue;
 		}
@@ -566,7 +580,7 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 		if ((!decompressed) || (!workspace)) {
 			free(decompressed);
 			free(raw_data);
-			LOG("malloc() failed");
+			NSLOG(netsurf, INFO, "malloc() failed");
 			ro_warn_user("NoMemory", 0);
 			continue;
 		}
@@ -581,7 +595,8 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 		free(raw_data);
 		if (error) {
 			free(decompressed);
-			LOG("xsquash_decompress: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO, "xsquash_decompress: 0x%x: %s",
+			      error->errnum, error->errmess);
 			ro_warn_user("MiscError", error->errmess);
 			continue;
 		}
@@ -599,7 +614,10 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 					descriptor->theme->sprite_area,
 					sprite_name, 16, i, 0);
 			if (error) {
-				LOG("xosspriteop_return_name: 0x%x: %s", error->errnum, error->errmess);
+				NSLOG(netsurf, INFO,
+				      "xosspriteop_return_name: 0x%x: %s",
+				      error->errnum,
+				      error->errmess);
 				ro_warn_user("MiscError", error->errmess);
 				continue;
 			}
@@ -614,7 +632,10 @@ bool ro_gui_theme_open(struct theme_descriptor *descriptor, bool list)
 					&dimensions.x, &dimensions.y,
 					(osbool *) 0, &mode);
 			if (error) {
-				LOG("xosspriteop_read_sprite_info: 0x%x: %s", error->errnum, error->errmess);
+				NSLOG(netsurf, INFO,
+				      "xosspriteop_read_sprite_info: 0x%x: %s",
+				      error->errnum,
+				      error->errmess);
 				ro_warn_user("MiscError", error->errmess);
 				continue;
 			}
