@@ -90,4 +90,9 @@ const char *messages_get_errorcode(nserror code);
 
 char *messages_get_buff(const char *key, ...);
 
+/**
+ * Free memory used by the standard Messages hash
+ */
+void messages_destroy(void);
+
 #endif
