@@ -65,7 +65,8 @@ user_agent_build_string(void)
 
         core_user_agent_string = ua_string;
 
-        LOG("Built user agent \"%s\"", core_user_agent_string);
+        NSLOG(netsurf, INFO, "Built user agent \"%s\"",
+              core_user_agent_string);
 }
 
 /* This is a function so that later we can override it trivially */
