@@ -468,7 +468,8 @@ bool utf8_save_text(const char *utf8_text, const char *path)
 
 	ret = guit->utf8->utf8_to_local(utf8_text, strlen(utf8_text), &conv);
 	if (ret != NSERROR_OK) {
-		LOG("failed to convert to local encoding, return %d", ret);
+		NSLOG(netsurf, INFO,
+		      "failed to convert to local encoding, return %d", ret);
 		return false;
 	}
 
@@ -476,7 +477,7 @@ bool utf8_save_text(const char *utf8_text, const char *path)
 	if (out) {
 		int res = fputs(conv, out);
 		if (res < 0) {
-			LOG("Warning: writing data failed");
+			NSLOG(netsurf, INFO, "Warning: writing data failed");
 		}
 
 		res = fputs("\n", out);
